﻿// ----- The Library Project
//       Start-to-Finish Visual C# 2015
//       Copyright (c) 2016 by Tim Patrick

using System;
using System.Data;
using System.Windows.Forms;
using static Library.General;

namespace Library
{
    public partial class ChangeUser : Form
    {
        public ChangeUser()
        {
            InitializeComponent();
        }

        private void AdjustOption()
        {
            // ----- Adjust the option setting based on the user entry.
            OptionPatron.Checked = (LoginID.Text.Trim().Length == 0);
            OptionAdmin.Checked = !OptionPatron.Checked;
        }

        private void LoginID_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            LoginID.SelectAll();
        }

        private void Password_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            Password.SelectAll();
        }

        private void LoginID_TextChanged(object sender, EventArgs e)
        {
            // ----- Refresh the option.
            AdjustOption();
        }

        private void Password_TextChanged(object sender, EventArgs e)
        {
            // ----- Refresh the option.
            AdjustOption();
        }

        private void ActOK_Click(object sender, EventArgs e)
        {
            // ----- Accept the user's input.
            string sqlText;
            DataCommand sqlRun;
            DataTable dbInfo;
            DataRow dbRow;
            string comparePassword;

            // ----- Different code for patron or user mode.
            if (OptionPatron.Checked == true)
            {
                // ----- Patron mode is easy.
                LoggedInUserID = -1L;
                LoggedInUserName = "";
                LoggedInGroupID = -1L;
                ReprocessSecuritySet();
                this.DialogResult = DialogResult.OK;
                return;
            }

            // ----- Check for a valid entry.
            if (LoginID.Text.Trim().Length == 0)
            {
                MessageBox.Show("To connect as an administrative user, you must provide a valid login ID.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }

            // ----- Check the user name and password.
            sqlText = "SELECT * FROM UserName WHERE UPPER(LoginID) = @TestLogin";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddText("TestLogin", LoginID.Text.Trim().ToUpper());
            try
            {
                dbInfo = CreateDataTable(sqlRun);
            }
            catch (Exception ex)
            {
                GeneralError("ChangeUser.ActOK_Click", ex);
                return;
            }

            if (dbInfo.Rows.Count == 0)
            {
                MessageBox.Show("Invalid login ID or password.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                dbInfo.Dispose();
                dbInfo = null;
                return;
            }
            dbRow = dbInfo.Rows[0];

            // ----- Check for an inactive user.
            if (DBGetBool(dbRow["Active"]) == false)
            {
                MessageBox.Show("The administrator has disabled your account.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                dbInfo.Dispose();
                dbInfo = null;
                return;
            }

            // ----- Check the password.
            if (Convert.IsDBNull(dbRow["Password"]) == true)
                comparePassword = EncryptPassword(LoginID.Text.Trim(), "");
            else
                comparePassword = DBGetText(dbRow["Password"]);
            if (EncryptPassword(LoginID.Text.Trim(), Password.Text.Trim()) != comparePassword)
            {
                MessageBox.Show("Invalid login ID or password.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                dbInfo.Dispose();
                dbInfo = null;
                return;
            }

            // ***** Insert Code Snippet #7 here.

            // ----- Successful login.
            LoggedInUserID = DBGetLong(dbRow["ID"]);
            LoggedInUserName = DBGetText(dbRow["LoginID"]);
            LoggedInGroupID = DBGetLong(dbRow["GroupID"]);
            dbInfo.Dispose();
            dbInfo = null;
            ReprocessSecuritySet();
            this.DialogResult = DialogResult.OK;
        }
    }
}
