﻿// ----- The Library Project
//       Start-to-Finish Visual C# 2015
//       Copyright (c) 2016 by Tim Patrick

using System;
using System.Data.SqlClient;
using System.Threading;
using System.Windows.Forms;
using static Library.General;

namespace Library
{
    static class Program
    {
        [STAThread]
        static void Main()
        {
			// ----- The application starts here.
			int productionDBVersion;
			SqlConnection testConnect;
            Mutex librarySync;
            Splash splashScreen;

            // ----- Perhaps another copy is already running. Use a mutex shared among
            //       all copies of the app to see if another copy exists.
            librarySync = new Mutex(true, "ACME.Library.CoreApp");
            if ((librarySync == null) || (librarySync.WaitOne(TimeSpan.Zero, true) == false))
            {
                // ----- Send a message to the other copy.
                NativeMethods.PostMessage((IntPtr)NativeMethods.HWND_BROADCAST,
                    NativeMethods.WM_LibraryNotify, IntPtr.Zero, IntPtr.Zero);
                Application.Exit();
                return;
            }

            // ----- Perform Windows-related initialization.
            Application.EnableVisualStyles();
            Application.SetCompatibleTextRenderingDefault(false);

            // ----- Add the global event handler for uncaught errors.
            Application.ThreadException += new ThreadExceptionEventHandler(GlobalErrorHandler);
            Application.SetUnhandledExceptionMode(
                UnhandledExceptionMode.CatchException);

            // ----- Show the splash screen.
            splashScreen = new Splash();
            splashScreen.Show();
            splashScreen.Refresh();

			// ----- Make sure we can connect to the database.
			testConnect = ConnectDatabase();
			if (testConnect == null)
			{
                splashScreen.Close();
                librarySync.ReleaseMutex();
                Application.Exit();
                return;
			}
            testConnect.Dispose();

            // ----- Check the database version.
            if (int.TryParse(GetSystemValue("DatabaseVersion"), out productionDBVersion) == false)
                productionDBVersion = 0;
			if (productionDBVersion != UseDBVersion)
			{
                splashScreen.Close();
				MessageBox.Show(
                    "The program cannot continue due to an incompatible database. " +
					$"The current database version is '{productionDBVersion}'. " +
                    $"The application version is '{UseDBVersion}'.",
					ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Error);
                librarySync.ReleaseMutex();
                Application.Exit();
                return;
			}

            // ----- Close the splash screen after a reasonable delay.
            splashScreen.ShowAwhile();
            splashScreen.Close();

            // ----- Time to show the main form.
            Application.Run(new MainForm());

            // ----- Let another copy run later.
            librarySync.ReleaseMutex();
        }

        private static void GlobalErrorHandler(object sender, ThreadExceptionEventArgs e)
        {
            // ----- Some unhandled error occurred. Just report the error and keep
            //       the program running.
            GeneralError("Library.UnhandledError", e.Exception);
        }
    }
}
