﻿// ----- The Library Project
//       Start-to-Finish Visual C# 2015
//       Copyright (c) 2016 by Tim Patrick

using System;
using System.Data;
using System.Windows.Forms;
using static Library.General;

namespace Library
{
    public partial class KeywordAdd : Form
    {
        private long ActiveID;

        public KeywordAdd()
        {
            InitializeComponent();
        }

        public long AddRecord()
        {
            // ----- Add a new keyword.
            ActiveID = -1L;
            this.ShowDialog();
            if (this.DialogResult == DialogResult.OK)
                return ActiveID;
            else
                return -1L;
        }

        private bool SaveFormData()
        {
            // ----- Save the record to the database.
            string sqlText;
            long newID;
            DataTable dbInfo;
            DataCommand sqlRun;

            this.Cursor = Cursors.WaitCursor;

            // ----- See if this keyword already exists.
            sqlText = "SELECT ID FROM Keyword WHERE FullName = @TestName";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddText("TestName", RecordKeyword.Text.Trim().ToLower());
            try
            {
                dbInfo = CreateDataTable(sqlRun);
            }
            catch (Exception ex)
            {
                this.Cursor = Cursors.Default;
                GeneralError("KeywordAdd.SaveFormData.Check", ex);
                return false;
            }
            if (dbInfo.Rows.Count > 0)
                ActiveID = DBGetLong(dbInfo.Rows[0] ["ID"]);
            dbInfo.Dispose();
            dbInfo = null;

            // ----- Create a new record if needed.
            if (ActiveID == -1L)
            {
                // ----- Create a new entry.
                sqlText = "INSERT INTO Keyword (FullName) " +
                    "OUTPUT INSERTED.ID VALUES (@NewName)";
                sqlRun = new DataCommand(sqlText);
                sqlRun.AddText("NewName", RecordKeyword.Text.Trim().ToLower());
                try
                {
                    newID = DBGetLong(ExecuteSQLReturn(sqlRun));
                }
                catch (Exception ex)
                {
                    this.Cursor = Cursors.Default;
                    GeneralError("KeywordAdd.SaveFormData.Add", ex);
                    return false;
                }
                ActiveID = newID;
            }

            // ----- Success.
            this.Cursor = Cursors.Default;
            return true;
        }

        private bool ValidateFormData()
        {
            // ----- Keyword is a required field.
            if (RecordKeyword.Text.Trim().Length == 0)
            {
                MessageBox.Show("Please supply a keyword.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                RecordKeyword.Focus();
                return false;
            }

            // ----- Success.
            return true;
        }

        private void RecordKeyword_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordKeyword.SelectAll();
        }

        private void ActOK_Click(object sender, EventArgs e)
        {
            // ----- The user wants to save changes.
            if (ValidateFormData() == false)
                return;
            if (SaveFormData() == false)
                return;

            // ----- Continue with AddRecord()
            this.DialogResult = DialogResult.OK;
        }
    }
}
