﻿// ----- The Library Project
//       Start-to-Finish Visual C# 2015
//       Copyright (c) 2016 by Tim Patrick

using System;
using static Library.General;

namespace Library
{
    public class HolidaySet
    {
        private string HolidayType;
        private string HolidayDetail;
        private bool IsValid;

        public HolidaySet(string entryType, string entryDetail)
        {
            // ----- Create a new holiday entry instance.
            int checkNum;

            HolidayType = "";
            if (entryType.Trim().Length > 1)
                HolidayType = entryType.ToUpper().Trim().Substring(0, 1);
            HolidayDetail = entryDetail;

            // ----- See if the details are valid.
            IsValid = true;
            switch (HolidayType)
            {
                case "A":
                    // ----- The detail should be in mm/dd format.
                    IsValid = CheckDateTime(entryDetail + "/2004");
                    break;
                case "E":
                    // ----- The detail is a number from 1 to 7.
                    if (int.TryParse(entryDetail, out checkNum) == true)
                    {
                        if ((checkNum < 1) | (checkNum > 7))
                            IsValid = false;
                    }
                    else
                        IsValid = false;
                    break;
                case "O":
                    // ----- The detail should be a valid date.
                    IsValid = CheckDateTime(entryDetail);
                    break;
                default:
                    // ---- Invalid. This should never happen.
                    IsValid = false;
                    break;
            }
        }

        public bool IsHoliday(DateTime whatDate)
        {
            // ----- Given a date, see if it matches the entry type in this instance.
            string buildDate;
            DateTime checkDate;
            int checkNum;

            // ----- If this record is invalid, then it is never a holiday match.
            if (IsValid == false)
                return false;

            switch (HolidayType)
            {
                case "A":
                    // ----- Annual.
                    buildDate = string.Format("{0}/{1}", HolidayDetail, whatDate.Year);
                    if (DateTime.TryParse(buildDate, out checkDate) == true)
                        return (checkDate == whatDate);
                    else
                    {
                        // ----- Must be 2/29 on a non-leap-year.
                        return false;
                    }
                case "E":
                    // ----- Day of the week.
                    if (int.TryParse(HolidayDetail, out checkNum) == true)
                        return (checkNum == (int)whatDate.DayOfWeek);
                    break;
                case "O":
                    // ----- See if this is an exact one-time match.
                    if (DateTime.TryParse(HolidayDetail, out checkDate) == true)
                        return (checkDate == whatDate);
                    break;
            }
            return false;
        }
    }
}
