﻿// ----- The Library Project
//       Start-to-Finish Visual C# 2015
//       Copyright (c) 2016 by Tim Patrick

using System;
using System.Data;
using System.Windows.Forms;
using static Library.General;

namespace Library
{
    public partial class ItemAuthorEdit : Form
    {
        private long ItemID;
        private long AuthorID;
        private long OrigTypeID;

        public ItemAuthorEdit()
        {
            InitializeComponent();
        }

        public long EditRecord(long useItemID, long useAuthorID, string useItemName,
            string useAuthorName, long useOrigNameType)
        {
            // ----- Edit the name type of an existing item/author relationship.
            //       If the user changes the name type, return the name type ID.
            //       Otherwise, return -1.
            ItemID = useItemID;
            AuthorID = useAuthorID;
            OrigTypeID = useOrigNameType;
            ItemName.Text = useItemName;
            AuthorName.Text = useAuthorName;
            this.ShowDialog();
            if (this.DialogResult == DialogResult.OK)
                return (long)(ListItemData)NameType.SelectedItem;
            else
                return -1L;
        }

        private bool SaveFormData()
        {
            // ----- Save changes to the database.
            string sqlText;
            DataCommand sqlRun;

            this.Cursor = Cursors.WaitCursor;

            // ----- Update the record in the database.
            sqlText = "UPDATE ItemAuthor SET AuthorType = @NewType " +
                "WHERE ItemID = @WhichItem  AND AuthorID = @WhichAuthor";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("NewType", DBComboParam(NameType));
            sqlRun.AddLong("WhichItem", ItemID);
            sqlRun.AddLong("WhichAuthor", AuthorID);
            try
            {
                ExecuteSQL(sqlRun);
            }
            catch (Exception ex)
            {
                GeneralError("ItemAuthorEdit.SaveFormData", ex);
                return false;
            }
            finally
            {
                this.Cursor = Cursors.Default;
            }

            // ----- Success.
            return true;
        }

        private bool ValidateFormData()
        {
            // ----- The name type must be selected.
            if ((long)(ListItemData)NameType.SelectedItem == -1L)
            {
                MessageBox.Show("Please select a name type.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                NameType.Focus();
                return false;
            }

            // ----- Success.
            return true;
        }

        private void ActOK_Click(object sender, EventArgs e)
        {
            // ----- The user wants to save changes.
            if (ValidateFormData() == false)
                return;
            if (SaveFormData() == false)
                return;
            this.DialogResult = DialogResult.OK;
        }

        private void ItemAuthorEdit_Load(object sender, EventArgs e)
        {
            // ----- Prepare the form.
            string sqlText;
            DataTable dbInfo;

            // ----- Load in the list of name types.
            NameType.Items.Add(new ListItemData("<Select a Name Type>", -1L));
            NameType.SelectedIndex = 0;
            sqlText = "SELECT * FROM CodeAuthorType ORDER BY FullName";
            try
            {
                dbInfo = CreateDataTable(sqlText);
            }
            catch (Exception ex)
            {
                GeneralError("ItemAuthorEdit.ItemAuthorEdit_Load", ex);
                return;
            }

            foreach (DataRow dbRow in dbInfo.Rows)
                NameType.Items.Add(new ListItemData(
                    DBGetText(dbRow["FullName"]), DBGetLong(dbRow["ID"])));
            dbInfo.Dispose();
            dbInfo = null;

            // ----- Set the existing name type.
            NameType.SelectedIndex = NameType.Items.IndexOf(OrigTypeID);
            if (NameType.SelectedIndex == -1)
                NameType.SelectedIndex = NameType.Items.IndexOf(-1L);
        }
    }
}
