﻿// ----- The Library Project
//       Start-to-Finish Visual C# 2015
//       Copyright (c) 2016 by Tim Patrick

using System;
using System.Collections;
using System.Data;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Printing;
using System.Windows.Forms;
using static Library.General;

namespace Library
{
    public partial class BarcodePrint : Form
    {
        private enum BarcodeOutput: int
        {
            Item = 1,
            Patron = 2,
            Misc = 3
        }

        private bool PreviewMode;
        private int LabelsX;
        private int LabelsY;
        private int PageWidth;
        private int PageHeight;
        private int PageMarginTop;
        private int PageMarginBottom;
        private int PageMarginLeft;
        private int PageMarginRight;
        private int PageIntraColumn;
        private int PageIntraRow;
        private int PageLabelWidth;
        private int PageLabelHeight;
        private int PageSoFar;
        private ArrayList LabelElements;
        private string BarcodeFontName;
        private bool Barcode39;
        private PrintDocument BarcodeDoc;

        public BarcodePrint()
        {
            InitializeComponent();
        }

        private void BarcodePrint_Load(object sender, EventArgs e)
        {
            // ----- Prepare the form.
            string sqlText;
            DataTable dbInfo;

            // ----- Initialize.
            LabelsX = -1;
            LabelsY = -1;

            // ----- Load in the list of number types.
            RecordType.Items.Add(new ListItemData("Item Barcodes", (long)BarcodeOutput.Item));
            RecordType.Items.Add(new ListItemData("Patron Barcodes", (long)BarcodeOutput.Patron));
            RecordType.Items.Add(new ListItemData("Miscellaneous Barcodes", (long)BarcodeOutput.Misc));
            RecordType.SelectedIndex = 0;

            // ----- Load in the available page templates.
            RecordPage.Items.Add(new ListItemData("<Not Selected>", -1L));
            sqlText = "SELECT ID, FullName FROM BarcodeSheet ORDER BY FullName";
            try
            {
                dbInfo = CreateDataTable(sqlText);
            }
            catch (Exception ex)
            {
                GeneralError("BarcodePrint.BarcodePrint_Load", ex);
                return;
            }

            foreach (DataRow dbRow in dbInfo.Rows)
                RecordPage.Items.Add(new ListItemData(
                    DBGetText(dbRow["FullName"]), DBGetLong(dbRow["ID"])));
            dbInfo.Dispose();
            dbInfo = null;
            RecordPage.SelectedIndex = 0;

            // ----- Don't load in the label templates at this time.
            RecordTemplate.Items.Add(new ListItemData("<Not Selected>", -1L));
            RecordTemplate.SelectedIndex = 0;
        }

        private void RecordPage_SelectedIndexChanged(object sender, EventArgs e)
        {
            // ----- Adjust the display based on the selected page.
            string sqlText;
            DataCommand sqlRun;
            DataTable dbInfo;
            DataRow oneRow;
            long whichPage;

            // ----- Clear the previous details selection.
            RecordTemplate.Items.Clear();
            RecordTemplate.Items.Add(new ListItemData("<Not Selected>", -1L));
            RecordTemplate.SelectedIndex = 0;
            RecordTemplate.Enabled = false;
            LabelTemplate.Enabled = false;
            RecordCountPerPage.Text = "No template selected.";
            RefreshEndCount();

            // ----- Check for no selection.
            if (RecordPage.SelectedIndex == -1)
                return;
            whichPage = (long)(ListItemData)RecordPage.SelectedItem;
            if (whichPage == -1L)
                return;
            this.Cursor = Cursors.WaitCursor;

            // ----- Load in the count of the labels.
            sqlText = "SELECT ColumnsCount, RowsCount FROM BarcodeSheet " +
                "WHERE ID = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", whichPage);
            try
            {
                dbInfo = CreateDataTable(sqlRun);
            }
            catch (Exception ex)
            {
                this.Cursor = Cursors.Default;
                GeneralError("BarcodePrint.RecordPage_SelectedIndexChanged.Labels", ex);
                return;
            }
            if (dbInfo.Rows.Count == 0)
            {
                this.Cursor = Cursors.Default;
                dbInfo.Dispose();
                return;
            }
            oneRow = dbInfo.Rows[0];
            LabelsX = DBGetInteger(oneRow["ColumnsCount"]);
            LabelsY = DBGetInteger(oneRow["RowsCount"]);
            dbInfo.Dispose();

            // ----- Update the count of the labels per sheet.
            if ((LabelsX * LabelsY) == 0)
                RecordCountPerPage.Text = "No labels per page.";
            else if ((LabelsX * LabelsY) == 1)
                RecordCountPerPage.Text = "1 label per page (1 by 1)";
            else
                RecordCountPerPage.Text = (LabelsX * LabelsY).ToString() +
                    " labels per page (" + LabelsX.ToString() +
                    " by " + LabelsY.ToString() + ")";
            RefreshEndCount();

            // ----- Load in the list of templates.
            sqlText = "SELECT ID, FullName FROM BarcodeLabel " +
                "WHERE BarcodeSheet = @RecordID ORDER BY FullName";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", whichPage);
            try
            {
                dbInfo = CreateDataTable(sqlRun);
            }
            catch (Exception ex)
            {
                this.Cursor = Cursors.Default;
                GeneralError("BarcodePrint.RecordPage_SelectedIndexChanged.Templates", ex);
                return;
            }
            if (dbInfo.Rows.Count == 0)
            {
                this.Cursor = Cursors.Default;
                dbInfo.Dispose();
                return;
            }
            foreach (DataRow dbRow in dbInfo.Rows)
                RecordTemplate.Items.Add(new ListItemData(
                    DBGetText(dbRow["FullName"]), DBGetLong(dbRow["ID"])));
            dbInfo.Dispose();
            dbInfo = null;
            RecordTemplate.Enabled = true;
            LabelTemplate.Enabled = true;
            this.Cursor = Cursors.Default;
        }

        private void RecordType_SelectedIndexChanged(object sender, EventArgs e)
        {
            // ----- Load in the starting value of the labels based on the selection.
            decimal startCount;
            BarcodeOutput whichTarget;

            // ----- What type of counting will we use?
            if (RecordType.SelectedIndex == -1)
                return;
            whichTarget = (BarcodeOutput)(long)(ListItemData)RecordType.SelectedItem;
            if (whichTarget == BarcodeOutput.Item)
            {
                // ----- Item labels.
                if (decimal.TryParse(GetSystemValue("NextBarcodeItem"), out startCount) == false)
                    startCount = 1M;
            }
            else if (whichTarget == BarcodeOutput.Patron)
            {
                // ----- Patron labels.
                if (decimal.TryParse(GetSystemValue("NextBarcodePatron"), out startCount) == false)
                    startCount = 1M;
            }
            else
            {
                // ----- Miscellaneous labels.
                if (decimal.TryParse(GetSystemValue("NextBarcodeMisc"), out startCount) == false)
                    startCount = 1M;
            }
            if (startCount <= 0M)
                startCount = 1M;

            // ----- Update the display.
            RecordStart.Text = startCount.ToString();
            RefreshEndCount();
        }

        private void RecordStart_TextChanged(object sender, EventArgs e)
        {
            // ----- Update the ending value.
            RefreshEndCount();
        }

        private void RecordStart_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordStart.SelectAll();
        }

        private void RecordPages_TextChanged(object sender, EventArgs e)
        {
            // ----- Update the ending value.
            RefreshEndCount();
        }

        private void RecordPages_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordPages.SelectAll();
        }

        private void RefreshEndCount()
        {
            // ----- Update the end count based on the start count.
            decimal endCount;
            decimal testStart;
            int testPage;

            // ----- Assume no end point.
            RecordStop.Text = "Not defined.";
            if ((LabelsX <= 0) | (LabelsY <= 0))
                return;
            if (decimal.TryParse(RecordStart.Text, out testStart) == false)
                return;
            if (int.TryParse(RecordPages.Text, out testPage) == false)
                return;

            // ----- Update the display.
            endCount = testStart + (decimal)(LabelsX * LabelsY * testPage) - 1M;
            RecordStop.Text = endCount.ToString();
        }

        private bool VerifyFields()
        {
            // ----- Get the name of the barcode font.
            if ((BarcodeFontName == null) || (BarcodeFontName.Length == 0))
            {
                BarcodeFontName = GetBarcodeFont();
                if (BarcodeFontName.Length == 0)
                    return false;
                Barcode39 = GetBoolean(GetSystemValue("BarcodeCode39"));
            }

            // ----- A valid page template is required.
            if ((long)(ListItemData)RecordPage.SelectedItem == -1L)
            {
                MessageBox.Show("Please select a Page Template.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                RecordPage.Focus();
                return false;
            }
            if ((LabelsX <= 0) | (LabelsY <= 0))
            {
                MessageBox.Show("The selected Page Template will generate no valid labels.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                RecordPage.Focus();
                return false;
            }

            // ----- A valid label template is required.
            if ((long)(ListItemData)RecordTemplate.SelectedItem == -1L)
            {
                MessageBox.Show("Please select a Label Template.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                RecordTemplate.Focus();
                return false;
            }

            // ----- The starting value must be valid.
            if (CheckDecimal(RecordStart.Text) == false)
            {
                MessageBox.Show("Start Value must be numeric.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                RecordStart.Focus();
                return false;
            }

            // ----- The pages value must be valid.
            if (CheckInt(RecordPages.Text) == false)
            {
                MessageBox.Show("Pages must be numeric.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                RecordPages.Focus();
                return false;
            }

            // ----- Success.
            return true;
        }

        private void ActPreview_Click(object sender, EventArgs e)
        {
            // ----- The user wants to preview the labels.
            PrintPreviewDialog barcodePreview;

            // ----- Make sure the user supplied valid data.
            if (VerifyFields() == false)
                return;

            // ----- Load in all of the page-specific details to be used in printing.
            if (LoadPageDetails() == false)
                return;

            // ----- Create the preview dialog.
            this.Cursor = Cursors.WaitCursor;
            PageSoFar = 0;
            PreviewMode = true;
            BarcodeDoc = new PrintDocument();

            // ----- Display the preview.
            BarcodeDoc.PrintPage += BarcodeDoc_PrintPage;
            barcodePreview = new PrintPreviewDialog();
            barcodePreview.Document = BarcodeDoc;
            barcodePreview.ShowDialog();
            BarcodeDoc = null;
            this.Cursor = Cursors.Default;
        }

        private void ActPrint_Click(object sender, EventArgs e)
        {
            // ----- The user wants to print the labels.
            BarcodeOutput whichTarget;
            PrintDialog barcodePrintDialog;

            // ----- Make sure the user supplied valid data.
            if (VerifyFields() == false)
                return;

            // ----- Load in all of the page-specific details to be used in printing.
            if (LoadPageDetails() == false)
                return;

            // ----- Prompt to print the labels, and then print.
            this.Cursor = Cursors.WaitCursor;
            PageSoFar = 0;
            PreviewMode = false;
            BarcodeDoc = new PrintDocument();
            BarcodeDoc.PrintPage += BarcodeDoc_PrintPage;
            barcodePrintDialog = new PrintDialog();
            barcodePrintDialog.Document = BarcodeDoc;
            barcodePrintDialog.UseEXDialog = true;
            if (barcodePrintDialog.ShowDialog() != DialogResult.OK)
            {
                BarcodeDoc = null;
                this.Cursor = Cursors.Default;
                return;
            }
            BarcodeDoc.Print();
            BarcodeDoc = null;
            this.Cursor = Cursors.Default;

            // ----- Ask the user about the success of the print.
            if (MessageBox.Show("Did the labels print successfully?",
                    ProgramTitle, MessageBoxButtons.YesNo,
                    MessageBoxIcon.Question) == DialogResult.No)
                return;

            // ----- Update the database with the print values.
            whichTarget = (BarcodeOutput)(long)(ListItemData)RecordType.SelectedItem;
            if (whichTarget == BarcodeOutput.Item)
            {
                // ----- Item labels.
                SetSystemValue("NextBarcodeItem",
                    (Convert.ToDecimal(RecordStop.Text) + 1).ToString());
            }
            else if (whichTarget == BarcodeOutput.Patron)
            {
                // ----- Patron labels.
                SetSystemValue("NextBarcodePatron",
                    (Convert.ToDecimal(RecordStop.Text) + 1).ToString());
            }
            else
            {
                // ----- Miscellaneous labels.
                SetSystemValue("NextBarcodeMisc",
                    (Convert.ToDecimal(RecordStop.Text) + 1).ToString());
            }

            // ----- Close the form.
            ActClose.PerformClick();
        }

        private bool LoadPageDetails()
        {
            // ----- Retrieve the label formatting details from the database.
            //       Return True on success.
            string sqlText;
            DataCommand sqlRun;
            DataTable dbInfo;
            DataRow oneRow;
            char fromUnit;
            BarcodeItemBarcodeImage newBarcodeImage;
            BarcodeItemBarcodeNumber newBarcodeText;
            BarcodeItemText newStaticText;
            BarcodeItemLine newStaticLine;
            BarcodeItemRect newStaticRect;

            this.Cursor = Cursors.WaitCursor;

            // ----- Load in the page structure details. Store all values in Twips.
            sqlText = "SELECT * FROM BarcodeSheet WHERE ID = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", (long)(ListItemData)RecordPage.SelectedItem);
            try
            {
                dbInfo = CreateDataTable(sqlRun);
            }
            catch (Exception ex)
            {
                this.Cursor = Cursors.Default;
                GeneralError("BarcodePrint.LoadPageDetails.Sheet", ex);
                return false;
            }
            oneRow = dbInfo.Rows[0];
            fromUnit = Convert.ToChar(DBGetText(oneRow["UnitType"]));
            PageWidth = (int)ConvertPageUnits(fromUnit, 'T', DBGetDecimal(oneRow["PageWidth"]));
            PageHeight = (int)ConvertPageUnits(fromUnit, 'T', DBGetDecimal(oneRow["PageHeight"]));
            PageMarginTop = (int)ConvertPageUnits(fromUnit, 'T', DBGetDecimal(oneRow["MarginTop"]));
            PageMarginBottom = (int)ConvertPageUnits(fromUnit, 'T', DBGetDecimal(oneRow["MarginBottom"]));
            PageMarginLeft = (int)ConvertPageUnits(fromUnit, 'T', DBGetDecimal(oneRow["MarginLeft"]));
            PageMarginRight = (int)ConvertPageUnits(fromUnit, 'T', DBGetDecimal(oneRow["MarginRight"]));
            PageIntraColumn = (int)ConvertPageUnits(fromUnit, 'T', DBGetDecimal(oneRow["IntraColumn"]));
            PageIntraRow = (int)ConvertPageUnits(fromUnit, 'T', DBGetDecimal(oneRow["IntraRow"]));
            dbInfo.Dispose();
            PageLabelWidth = (int)((PageWidth - PageMarginLeft - PageMarginRight -
                ((LabelsX - 1) * PageIntraColumn)) / (decimal)LabelsX);
            PageLabelHeight = (int)((PageHeight - PageMarginTop - PageMarginBottom -
                ((LabelsY - 1) * PageIntraRow)) / (decimal)LabelsY);

            // ----- Get the units to be used for label items.
            sqlText = "SELECT UnitType FROM BarcodeLabel WHERE ID = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", (long)(ListItemData)RecordTemplate.SelectedItem);
            try
            {
                dbInfo = CreateDataTable(sqlRun);
            }
            catch (Exception ex)
            {
                this.Cursor = Cursors.Default;
                GeneralError("BarcodePrint.LoadPageDetails.Units", ex);
                return false;
            }
            oneRow = dbInfo.Rows[0];
            fromUnit = Convert.ToChar(DBGetText(oneRow["UnitType"]));
            dbInfo.Dispose();

            // ----- Load in the items that appear on each label.
            LabelElements = new ArrayList();
            sqlText = "SELECT * FROM BarcodeLabelItem " + "WHERE BarcodeLabel = @RecordID ORDER BY Priority";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", (long)(ListItemData)RecordTemplate.SelectedItem);
            try
            {
                dbInfo = CreateDataTable(sqlRun);
            }
            catch (Exception ex)
            {
                this.Cursor = Cursors.Default;
                GeneralError("BarcodePrint.LoadPageDetails.Items", ex);
                return false;
            }
            foreach (DataRow dbRow in dbInfo.Rows)
            {
                // ----- Process this object.
                switch (DBGetText(dbRow["ItemType"]).ToUpper())
                {
                    case "B":
                        // ----- Barcode image.
                        newBarcodeImage = new BarcodeItemBarcodeImage();
                        newBarcodeImage.Alignment = (ContentAlignment)DBGetInteger(dbRow["Alignment"]);
                        newBarcodeImage.BarcodeColor = Color.FromArgb(DBGetInteger(dbRow["Color1"]));
                        newBarcodeImage.BarcodeSize = DBGetDecimal(dbRow["FontSize"]);
                        newBarcodeImage.Left = ConvertPageUnits(fromUnit, 'T', DBGetDecimal(dbRow["PosLeft"]));
                        newBarcodeImage.Top = ConvertPageUnits(fromUnit, 'T', DBGetDecimal(dbRow["PosTop"]));
                        newBarcodeImage.Width = ConvertPageUnits(fromUnit, 'T', DBGetDecimal(dbRow["PosWidth"]));
                        newBarcodeImage.Height = ConvertPageUnits(fromUnit, 'T', DBGetDecimal(dbRow["PosHeight"]));
                        newBarcodeImage.RotationAngle = (short)DBGetInteger(dbRow["Rotation"]);
                        newBarcodeImage.PadDigits = (byte)DBGetInteger(dbRow["PadDigits"]);
                        LabelElements.Add(newBarcodeImage);
                        break;
                    case "L":
                        // ----- Line.
                        newStaticLine = new BarcodeItemLine();
                        newStaticLine.LineColor = Color.FromArgb(DBGetInteger(dbRow["Color1"]));
                        newStaticLine.X1 = ConvertPageUnits(fromUnit, 'T', DBGetDecimal(dbRow["PosLeft"]));
                        newStaticLine.X2 = ConvertPageUnits(fromUnit, 'T', DBGetDecimal(dbRow["PosWidth"]));
                        newStaticLine.Y1 = ConvertPageUnits(fromUnit, 'T', DBGetDecimal(dbRow["PosTop"]));
                        newStaticLine.Y2 = ConvertPageUnits(fromUnit, 'T', DBGetDecimal(dbRow["PosHeight"]));
                        LabelElements.Add(newStaticLine);
                        break;
                    case "N":
                        // ----- Barcode text.
                        newBarcodeText = new BarcodeItemBarcodeNumber();
                        newBarcodeText.Alignment = (ContentAlignment)DBGetInteger(dbRow["Alignment"]);
                        newBarcodeText.Left = ConvertPageUnits(fromUnit, 'T', DBGetDecimal(dbRow["PosLeft"]));
                        newBarcodeText.Top = ConvertPageUnits(fromUnit, 'T', DBGetDecimal(dbRow["PosTop"]));
                        newBarcodeText.Width = ConvertPageUnits(fromUnit, 'T', DBGetDecimal(dbRow["PosWidth"]));
                        newBarcodeText.Height = ConvertPageUnits(fromUnit, 'T', DBGetDecimal(dbRow["PosHeight"]));
                        newBarcodeText.RotationAngle = (short)DBGetInteger(dbRow["Rotation"]);
                        newBarcodeText.TextColor = Color.FromArgb(DBGetInteger(dbRow["Color1"]));
                        newBarcodeText.Font = new Font(DBGetText(dbRow["FontName"]),
                            (float)DBGetDecimal(dbRow["FontSize"]),
                            BuildFontStyle(DBGetText(dbRow["FontStyle"])));
                        newBarcodeText.PadDigits = (byte)DBGetInteger(dbRow["PadDigits"]);
                        LabelElements.Add(newBarcodeText);
                        break;
                    case "R":
                        // ----- Rectangle.
                        newStaticRect = new BarcodeItemRect();
                        newStaticRect.BorderColor = Color.FromArgb(DBGetInteger(dbRow["Color1"]));
                        newStaticRect.FillColor = Color.FromArgb(DBGetInteger(dbRow["Color2"]));
                        newStaticRect.Left = ConvertPageUnits(fromUnit, 'T', DBGetDecimal(dbRow["PosLeft"]));
                        newStaticRect.Top = ConvertPageUnits(fromUnit, 'T', DBGetDecimal(dbRow["PosTop"]));
                        newStaticRect.Width = ConvertPageUnits(fromUnit, 'T', DBGetDecimal(dbRow["PosWidth"]));
                        newStaticRect.Height = ConvertPageUnits(fromUnit, 'T', DBGetDecimal(dbRow["PosHeight"]));
                        newStaticRect.RotationAngle = (short)DBGetInteger(dbRow["Rotation"]);
                        LabelElements.Add(newStaticRect);
                        break;
                    case "T":
                        // ----- Static text.
                        newStaticText = new BarcodeItemText();
                        newStaticText.DisplayText = DBGetText(dbRow["StaticText"]);
                        newStaticText.Alignment = (ContentAlignment)DBGetInteger(dbRow["Alignment"]);
                        newStaticText.Left = ConvertPageUnits(fromUnit, 'T', DBGetDecimal(dbRow["PosLeft"]));
                        newStaticText.Top = ConvertPageUnits(fromUnit, 'T', DBGetDecimal(dbRow["PosTop"]));
                        newStaticText.Width = ConvertPageUnits(fromUnit, 'T', DBGetDecimal(dbRow["PosWidth"]));
                        newStaticText.Height = ConvertPageUnits(fromUnit, 'T', DBGetDecimal(dbRow["PosHeight"]));
                        newStaticText.RotationAngle = (short)DBGetInteger(dbRow["Rotation"]);
                        newStaticText.TextColor = Color.FromArgb(DBGetInteger(dbRow["Color1"]));
                        newStaticText.Font = new Font(DBGetText(dbRow["FontName"]),
                            (float)DBGetDecimal(dbRow["FontSize"]),
                            BuildFontStyle(DBGetText(dbRow["FontStyle"])));
                        LabelElements.Add(newStaticText);
                        break;
                }
            }
            dbInfo.Dispose();

            // ----- Success.
            this.Cursor = Cursors.Default;
            return true;
        }

        private void BarcodeDoc_PrintPage(object sender, PrintPageEventArgs e)
        {
            // ----- Print one page of labels.
            Pen pixelPen;
            int leftOffset;
            int topOffset;
            int hardLeft;
            int hardTop;
            decimal barcodeNumber;
            GraphicsState holdState;

            // ----- Display the outline of labels.
            if (PreviewMode == true)
            {
                pixelPen = new Pen(Color.Cyan, 1 / e.Graphics.DpiX);
                for (int rowScan = 1; rowScan <= LabelsY; rowScan++)
                {
                    for (int colScan = 1; colScan <= LabelsX; colScan++)
                    {
                        leftOffset = (int)ConvertPageUnits('T', 'H', PageMarginLeft +
                            ((colScan - 1) * (PageIntraColumn + PageLabelWidth)));
                        topOffset = (int)ConvertPageUnits('T', 'H', PageMarginTop +
                            ((rowScan - 1) * (PageIntraRow + PageLabelHeight)));
                        e.Graphics.DrawRectangle(Pens.Cyan, leftOffset, topOffset,
                            (float)ConvertPageUnits('T', 'H', PageLabelWidth),
                            (float)ConvertPageUnits('T', 'H', PageLabelHeight));
                    }
                }
                pixelPen.Dispose();
            }

            // ----- Are we printing to a real printer?
            if (PreviewMode == false)
            {
                // ----- Real print.
                hardLeft = (int)ConvertPageUnits('H', 'T', (decimal)(e.PageBounds.Width -
                    e.Graphics.VisibleClipBounds.Width) / 2.0M);
                hardTop = (int)ConvertPageUnits('H', 'T', (decimal)(e.PageBounds.Height -
                    e.Graphics.VisibleClipBounds.Height) / 2.0M);
            }
            else
            {
                // ----- Print preview.
                hardLeft = 0;
                hardTop = 0;
            }

            // ----- Determine the starting barcode number on this page.
            barcodeNumber = Convert.ToDecimal(RecordStart.Text) + (decimal)(PageSoFar * (LabelsX * LabelsY));

            // ----- Process each label.
            for (int rowScan = 1; rowScan <= LabelsY; rowScan++)
            {
                for (int colScan = 1; colScan <= LabelsX; colScan++)
                {
                    // ----- Find the upper-left corner of the label.
                    leftOffset = PageMarginLeft + ((colScan - 1) *
                        (PageIntraColumn + PageLabelWidth)) - hardLeft;
                    topOffset = PageMarginTop + ((rowScan - 1) *
                        (PageIntraRow + PageLabelHeight)) - hardTop;

                    // ----- Save the current graphics state. Move into twips mode, and
                    //       position the origin to the upper-left corner of the label.
                    holdState = e.Graphics.Save();
                    e.Graphics.ScaleTransform(100.0F / 1440.0F, 100.0F / 1440.0F);
                    e.Graphics.TranslateTransform(leftOffset, topOffset);

                    // ----- Print the label.
                    PrintOneLabel(e.Graphics, barcodeNumber);
                    barcodeNumber += 1M;

                    // ----- Restore the original graphics state.
                    e.Graphics.Restore(holdState);
                }
            }

            // ----- Move to the next page.
            PageSoFar += 1;
            if (PageSoFar >= Convert.ToInt32(RecordPages.Text))
            {
                // ----- If this was preview mode, the user might want to click on
                //       the "Print" button included on the preview dialog. Since
                //       we print preview screens differently, we need to ensure
                //       the a true print through this button will print a non-
                //       preview set of labels.
                PreviewMode = false;
                e.HasMorePages = false;
            }
            else
            {
                e.HasMorePages = true;
            }
        }

        private void PrintOneLabel(Graphics g, decimal barcodeNumber)
        {
            // ----- Print out one barcode label.
            string textMessage;
            BarcodeItemGeneric useGeneric;
            BarcodeItemBarcodeImage useBarImage;
            BarcodeItemBarcodeNumber useBarText;
            BarcodeItemText useText;
            BarcodeItemLine useLine;
            BarcodeItemRect useRect;
            Font useFont;
            GraphicsState holdState;
            Pen pixelPen;
            SolidBrush pixelBrush;
            Color textColor;
            int textAngle;
            ContentAlignment textAlign;
            StringFormat textFormat = new StringFormat();
            float X1;
            float X2;
            float Y1;
            float Y2;
            IEnumerator scanLabel;

            // ----- NOTE: The calling procedure has adjusted the graphics display.
            //       All output is measured in twips, and the origin is at the upper-
            //       left corner of the label to be printed.

            // ----- Process each item in the list.
            scanLabel = LabelElements.GetEnumerator();
            while (scanLabel.MoveNext())
            {
                // ----- Save the current state of the graphics area.
                holdState = g.Save();

                // ----- Different code for each type of drawing object.
                useGeneric = (BarcodeItemGeneric)scanLabel.Current;
                switch (useGeneric.ItemType)
                {
                    case "B":
                        // ----- Barcode image.
                        useBarImage = (BarcodeItemBarcodeImage)useGeneric;
                        textMessage = barcodeNumber.ToString();
                        if (textMessage.Length < useBarImage.PadDigits)
                            textMessage = new string('0',
                                useBarImage.PadDigits - textMessage.Length) + textMessage;
                        if (Barcode39 == true)
                            textMessage = "*" + textMessage + "*";
                        textColor = useBarImage.BarcodeColor;
                        textAngle = useBarImage.RotationAngle;
                        textAlign = useBarImage.Alignment;

                        // ----- Get the font to use.
                        useFont = new Font(BarcodeFontName,
                            (float)(useBarImage.BarcodeSize * 14.4M), FontStyle.Regular);

                        // ----- Get the coordinates in twips.
                        X1 = (float)useBarImage.Left;
                        X2 = (float)useBarImage.Width;
                        Y1 = (float)useBarImage.Top;
                        Y2 = (float)useBarImage.Height;

                        // ----- The rest is the same as drawing normal text.
                        goto DisplayTheText;
                    case "L":
                        // ----- Line.
                        useLine = (BarcodeItemLine)useGeneric;

                        // ----- Create a one-pixel pen to use.
                        pixelPen = new Pen(useLine.LineColor, 1 / g.DpiX);

                        // ----- Get the coordinates in twips.
                        X1 = (float)useLine.X1;
                        X2 = (float)useLine.X2;
                        Y1 = (float)useLine.Y1;
                        Y2 = (float)useLine.Y2;

                        // ----- Draw the line.
                        g.DrawLine(pixelPen, X1, Y1, X2, Y2);
                        pixelPen.Dispose();
                        break;
                    case "N":
                        // ----- Barcode text.
                        useBarText = (BarcodeItemBarcodeNumber)useGeneric;
                        textMessage = barcodeNumber.ToString();
                        if (textMessage.Length < useBarText.PadDigits)
                            textMessage = new string('0',
                                useBarText.PadDigits - textMessage.Length) + textMessage;
                        textColor = useBarText.TextColor;
                        textAngle = useBarText.RotationAngle;
                        textAlign = useBarText.Alignment;

                        // ----- Get the font to use.
                        useFont = new Font(useBarText.Font.Name,
                            (float)(useBarText.Font.SizeInPoints * 14.4), useBarText.Font.Style);

                        // ----- Get the coordinates in twips.
                        X1 = (float)useBarText.Left;
                        X2 = (float)useBarText.Width;
                        Y1 = (float)useBarText.Top;
                        Y2 = (float)useBarText.Height;

                        // ----- The rest is the same as drawing normal text.
                        goto DisplayTheText;
                    case "R":
                        // ----- Rectangle.
                        useRect = (BarcodeItemRect)useGeneric;

                        // ----- Create a one-pixel pen to use.
                        pixelPen = new Pen(useRect.BorderColor, 1 / g.DpiX);
                        pixelBrush = new SolidBrush(useRect.FillColor);

                        // ----- Get the coordinates in twips.
                        X1 = (float)useRect.Left;
                        X2 = (float)useRect.Width;
                        Y1 = (float)useRect.Top;
                        Y2 = (float)useRect.Height;

                        // ----- Rotate the surface if requested.
                        if (useRect.RotationAngle != 0)
                        {
                            g.TranslateTransform(X1, Y1);
                            g.RotateTransform(useRect.RotationAngle);
                            X1 = 0.0F;
                            Y1 = 0.0F;
                        }

                        // ----- Draw the rectangle.
                        g.FillRectangle(pixelBrush, X1, Y1, X2, Y2);
                        g.DrawRectangle(pixelPen, X1, Y1, X2, Y2);
                        pixelBrush.Dispose();
                        pixelPen.Dispose();
                        break;
                    case "T":
                        // ----- Text.
                        useText = (BarcodeItemText)useGeneric;
                        textMessage = useText.DisplayText;
                        textColor = useText.TextColor;
                        textAngle = useText.RotationAngle;
                        textAlign = useText.Alignment;

                        // ----- Get the font to use.
                        useFont = new Font(useText.Font.Name,
                            (float)(useText.Font.SizeInPoints * 14.4), useText.Font.Style);

                        // ----- Get the coordinates in points.
                        X1 = (float)useText.Left;
                        X2 = (float)useText.Width;
                        Y1 = (float)useText.Top;
                        Y2 = (float)useText.Height;

        DisplayTheText:
                        // ----- Rotate around the upper-left corner of the text box
                        //       if needed.
                        if (textAngle != 0)
                        {
                            g.TranslateTransform(X1, Y1);
                            g.RotateTransform(textAngle);
                            X1 = 0.0F;
                            Y1 = 0.0F;
                        }

                        // ----- Set the clipping region.
                        g.SetClip(new RectangleF(X1, Y1, X2, Y2));

                        // ----- Position the text within the bounding box.
                        switch (textAlign)
                        {
                            case ContentAlignment.TopLeft:
                                textFormat.Alignment = StringAlignment.Near;
                                textFormat.LineAlignment = StringAlignment.Near;
                                break;
                            case ContentAlignment.MiddleLeft:
                                textFormat.Alignment = StringAlignment.Near;
                                textFormat.LineAlignment = StringAlignment.Center;
                                break;
                            case ContentAlignment.BottomLeft:
                                textFormat.Alignment = StringAlignment.Near;
                                textFormat.LineAlignment = StringAlignment.Far;
                                break;
                            case ContentAlignment.TopCenter:
                                textFormat.Alignment = StringAlignment.Center;
                                textFormat.LineAlignment = StringAlignment.Near;
                                break;
                            case ContentAlignment.MiddleCenter:
                                textFormat.Alignment = StringAlignment.Center;
                                textFormat.LineAlignment = StringAlignment.Center;
                                break;
                            case ContentAlignment.BottomCenter:
                                textFormat.Alignment = StringAlignment.Center;
                                textFormat.LineAlignment = StringAlignment.Far;
                                break;
                            case ContentAlignment.TopRight:
                                textFormat.Alignment = StringAlignment.Far;
                                textFormat.LineAlignment = StringAlignment.Near;
                                break;
                            case ContentAlignment.MiddleRight:
                                textFormat.Alignment = StringAlignment.Far;
                                textFormat.LineAlignment = StringAlignment.Center;
                                break;
                            case ContentAlignment.BottomRight:
                                textFormat.Alignment = StringAlignment.Far;
                                textFormat.LineAlignment = StringAlignment.Far;
                                break;
                        }
                        textFormat.FormatFlags =
                            textFormat.FormatFlags | StringFormatFlags.NoWrap;

                        // ----- Print the text.
                        g.DrawString(textMessage, useFont, new SolidBrush(textColor),
                            new RectangleF(X1, Y1, X2, Y2), textFormat);
                        useFont.Dispose();
                        break;
                }

                // ----- Restore the original transformed state of the graphics surface.
                g.Restore(holdState);
            }
        }
    }
}
