﻿// ----- The Library Project
//       Start-to-Finish Visual C# 2015
//       Copyright (c) 2016 by Tim Patrick

using System;
using System.Windows.Forms;
using static Library.General;

namespace Library
{
    public partial class ListEditRecords : Form
    {
        private BaseCodeForm DetailEditor;

        public ListEditRecords()
        {
            InitializeComponent();
        }

        public void ManageRecords(BaseCodeForm useDetail)
        {
            // ----- Set up the form for use with this code set.
            bool exceededMatches = false;

            DetailEditor = useDetail;
            RecordsTitle.Text = DetailEditor.GetEditTitle();
            RecordsInfo.Text = DetailEditor.GetEditDescription();
            this.Text = DetailEditor.GetEditTitle();
            ActAdd.Visible = DetailEditor.CanUserAdd();
            ActEdit.Visible = DetailEditor.CanUserEdit();
            ActDelete.Visible = DetailEditor.CanUserDelete();
            ActLookup.Visible = DetailEditor.UsesSearch();
            DetailEditor.FillListWithRecords(RecordsList, ref exceededMatches);
            RefreshItemCount(exceededMatches);
            this.ShowDialog();
        }

        private void RefreshButtons()
        {
            // ----- Update the buttons as needed.
            ActEdit.Enabled = (RecordsList.SelectedIndex != -1);
            ActDelete.Enabled = (RecordsList.SelectedIndex != -1);
        }

        private void RefreshItemCount(bool exceededMatches)
        {
            // ----- Update the list count display.
            if (exceededMatches)
                ResultCount.Text = "Results limited to first " +
                    RecordsList.Items.Count.ToString() + " items";
            else if (RecordsList.Items.Count == 1)
                ResultCount.Text = "1 record displayed";
            else
                ResultCount.Text = RecordsList.Items.Count.ToString() +
                    " records displayed";
        }

        private void ActClose_Click(object sender, EventArgs e)
        {
            // ----- Close the form.
            DetailEditor.Close();
            DetailEditor = null;
            this.Close();
        }

        private void ActAdd_Click(object sender, EventArgs e)
        {
            // ----- Let the user add a record.
            long newID;
            int newPosition;

            // ----- Prompt the user.
            newID = DetailEditor.AddRecord();
            if (newID == -1L)
                return;

            // ----- Add this record to the list.
            newPosition = RecordsList.Items.Add((new ListItemData(
                DetailEditor.FormatRecordName(newID), newID)));
            RecordsList.SelectedIndex = newPosition;
            RefreshButtons();
            RefreshItemCount(false);
        }

        private void ActEdit_Click(object sender, EventArgs e)
        {
            // ----- Let the user edit a record.
            long newID;
            ListItemData listItem;

            // ----- Retrieve the item to edit.
            if (RecordsList.SelectedIndex == -1)
                return;
            listItem = (ListItemData)RecordsList.SelectedItem;

            // ----- Prompt the user.
            newID = DetailEditor.EditRecord(listItem.ItemData);
            if (newID == -1L)
                return;

            // ----- Modify the text of this item.
            listItem.ItemText = DetailEditor.FormatRecordName(newID);
            RecordsList.Items[RecordsList.SelectedIndex] = listItem;
            RefreshItemCount(false);
        }

        private void ActDelete_Click(object sender, EventArgs e)
        {
            // ----- Let the user edit a record.
            ListItemData listItem;

            // ----- Retrieve the item to edit.
            if (RecordsList.SelectedIndex == -1)
                return;
            listItem = (ListItemData)RecordsList.SelectedItem;

            // ----- Prompt the user.
            if (DetailEditor.DeleteRecord(listItem.ItemData) == false)
                return;

            // ----- Remove this item from the display.
            RecordsList.Items.RemoveAt(RecordsList.SelectedIndex);
            RefreshButtons();
            RefreshItemCount(false);
        }

        private void ActLookup_Click(object sender, EventArgs e)
        {
            // ----- Prompt for user limitation of the displayed items.
            bool exceededMatches = false;

            DetailEditor.SearchForRecord(RecordsList, ref exceededMatches);
            RefreshButtons();
            RefreshItemCount(exceededMatches);
        }

        private void RecordsList_Click(object sender, EventArgs e)
        {
            // ----- Enable controls as needed.
            RefreshButtons();
        }

        private void RecordsList_DoubleClick(object sender, EventArgs e)
        {
            // ----- Same as the edit button.
            if (ActEdit.Visible == true)
                ActEdit.PerformClick();
        }

        private void RecordsList_SelectedIndexChanged(object sender, EventArgs e)
        {
            // ----- Enable controls as needed.
            RefreshButtons();
        }
    }
}
