﻿// ----- The Library Project
//       Start-to-Finish Visual C# 2015
//       Copyright (c) 2016 by Tim Patrick

using System;
using System.Drawing;
using System.Reflection;
using System.Windows.Forms;
using static Library.General;

namespace Library
{
    public partial class LocateLicense : Form
    {
        private bool LocationModified = false;

        public LocateLicense()
        {
            InitializeComponent();
        }

        public bool ChangeLicense()
        {
            this.ShowDialog();
            return LocationModified;
        }

        private void DisplayLicenseStatus()
        {
            // ----- Show the current status of the license.
            LicenseFileDetail licenseDetails;
            Assembly currentAssembly;
            Version versionInfo;

            // ----- Get the license details.
            LicenseStatusColor.BackColor = Color.Red;
            licenseDetails = ExamineLicense();
            switch (licenseDetails.Status)
            {
                case LicenseStatus.ValidLicense:
                    LicenseStatusColor.BackColor = Color.Green;
                    LicenseStatusText.Text = "The license is valid.";
                    break;
                case LicenseStatus.MissingLicenseFile:
                    LicenseStatusText.Text = "The license file could not be found.";
                    break;
                case LicenseStatus.CorruptLicenseFile:
                    LicenseStatusText.Text = "The license file was corrupt or contained invalid data.";
                    break;
                case LicenseStatus.InvalidSignature:
                    LicenseStatusText.Text = "The license file's contents did not match its digital signature.";
                    break;
                case LicenseStatus.NotYetLicensed:
                    LicenseStatusText.Text = "Although the license file is valid, it cannot be used until " +
                        licenseDetails.LicenseDate.ToString("D") + ".";
                    break;
                case LicenseStatus.LicenseExpired:
                    LicenseStatusText.Text = "Although the license file is valid, it expired on " +
                        licenseDetails.ExpireDate.ToString("D") + ".";
                    break;
                case LicenseStatus.VersionMismatch:
                    currentAssembly = Assembly.GetEntryAssembly();
                    if (currentAssembly == null)
                        currentAssembly = Assembly.GetCallingAssembly();
                    versionInfo = currentAssembly.GetName().Version;
                    LicenseStatusText.Text =
                        $"The application version of {versionInfo} " +
                        "does not match the licensed version of " +
                        licenseDetails.CoveredVersion + ".";
                    break;
            }
        }

        private void ActLocate_Click(object sender, EventArgs e)
        {
            // ----- Prompt the user for a new license file.
            OpenFileDialog locateLicenseDialog;

            locateLicenseDialog = new OpenFileDialog();
            locateLicenseDialog.Filter = "License Files|*.lic|XML Files|*.xml|All Files|*.*";
            locateLicenseDialog.Title = "Open License File";
            if (locateLicenseDialog.ShowDialog() != DialogResult.OK)
                return;

            // ----- Store the new path.
            Properties.Settings.Default.LicenseFileLocation = locateLicenseDialog.FileName;
            LocationModified = true;

            // ----- Update the display.
            DisplayLicenseStatus();
            LicensePath.Text = Properties.Settings.Default.LicenseFileLocation;
        }

        private void LocateLicense_Load(object sender, EventArgs e)
        {
            // ----- Display the current status.
            DisplayLicenseStatus();

            // ----- Also display the license location.
            LicensePath.Text = Properties.Settings.Default.LicenseFileLocation;
        }
    }
}
