﻿// ----- The Library Project
//       Start-to-Finish Visual C# 2015
//       Copyright (c) 2016 by Tim Patrick

using System;
using System.Data;
using System.Drawing;
using System.Windows.Forms;
using static Library.General;

namespace Library
{
    public partial class Holiday : Library.BaseCodeForm
    {
        private long ActiveID;
        private bool StartingOver;

        public Holiday()
        {
            InitializeComponent();
        }

        public override long AddRecord()
        {
            // ----- Add a new record.
            ActiveID = -1L;
            PrepareFormFields();
            this.ShowDialog();
            if (this.DialogResult == DialogResult.OK)
                return ActiveID;
            else
                return -1L;
        }

        public override bool CanUserAdd()
        {
            // ----- Check the user for security access: add.
            return SecurityProfile[(int)LibrarySecurity.ManageHolidays];
        }

        public override bool CanUserEdit()
        {
            // ----- Check the user for security access: edit.
            return SecurityProfile[(int)LibrarySecurity.ManageHolidays];
        }

        public override bool CanUserDelete()
        {
            // ----- Check the user for security access: delete.
            return SecurityProfile[(int)LibrarySecurity.ManageHolidays];
        }

        public override bool DeleteRecord(long recordID)
        {
            // ----- The user wants to delete the records.
            string sqlText;
            DataCommand sqlRun;

            // ----- Confirm with the user.
            if (MessageBox.Show("Do you really wish to delete the selected holiday?",
                    ProgramTitle, MessageBoxButtons.YesNo, MessageBoxIcon.Question) !=
                    DialogResult.Yes)
                return false;

            // ----- Delete the record.
            sqlText = "DELETE FROM Holiday WHERE ID = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", recordID);
            try
            {
                ExecuteSQL(sqlRun);
            }
            catch (Exception ex)
            {
                GeneralError("Holiday.DeleteRecord", ex);
                return false;
            }
            return true;
        }

        public override long EditRecord(long recordID)
        {
            // ----- Edit an existing record.
            ActiveID = recordID;
            PrepareFormFields();
            this.ShowDialog();
            if (this.DialogResult == DialogResult.OK)
                return ActiveID;
            else
                return -1L;
        }

        public override void FillListWithRecords(ListBox destList, ref bool exceededMatches)
        {
            // ----- Given a list, fill it with all defined items.
            string sqlText;
            DataTable dbInfo;

            // ----- Clear any existing values.
            destList.Items.Clear();
            exceededMatches = false;

            // ----- Retrieve the values from the database.
            sqlText = "SELECT * FROM Holiday ORDER BY EntryType, EntryDetail";
            try
            {
                dbInfo = CreateDataTable(sqlText);
            }
            catch (Exception ex)
            {
                GeneralError("Holiday.FillListWithRecords", ex);
                return;
            }

            foreach (DataRow dbRow in dbInfo.Rows)
                destList.Items.Add(new ListItemData(
                    FormatOneEntry(dbRow), DBGetLong(dbRow["ID"])));
            dbInfo.Dispose();
            dbInfo = null;
        }

        private string FormatOneEntry(DataRow dbRow)
        {
            // ----- Given an active record, format its content for display.
            string itemText;
            DateTime holdDate;
            int holdValue;

            // ----- Different formatting for different entry types.
            itemText = "";
            switch (DBGetText(dbRow["EntryType"]).ToUpper())
            {
                case "A":
                    // ----- Annually.
                    itemText = DBGetText(dbRow["EntryDetail"]) + "/2004";
                    if (DateTime.TryParse(itemText, out holdDate) == true)
                        itemText = "Annually - " + holdDate.ToString("MMMM d");
                    else
                        itemText = "Annually - Invalid Entry";
                    break;
                case "E":
                    // ----- Weekly.
                    if (int.TryParse(DBGetText(dbRow["EntryDetail"]), out holdValue) == false)
                        holdValue = -1;
                    if ((holdValue < (int)DayOfWeek.Sunday) |
                            (holdValue > (int)DayOfWeek.Saturday))
                        itemText = "Weekly - Invalid Entry";
                    else
                        itemText = "Weekly - Every " + System.Threading.Thread.CurrentThread.
                            CurrentCulture.DateTimeFormat.GetDayName((DayOfWeek)holdValue);
                    break;
                case "O":
                    // ----- One-time.
                    itemText = DBGetText(dbRow["EntryDetail"]);
                    if (DateTime.TryParse(itemText, out holdDate) == true)
                        itemText = holdDate.ToString("yyyy - MMMM d, yyyy");
                    else
                        itemText = "???? - Invalid Entry";
                    break;
            }
            itemText += " [" + DBGetText(dbRow["FullName"]) + "]";
            return itemText;
        }

        public override string FormatRecordName(long recordID)
        {
            // ----- Given an ID, return the name.
            string sqlText;
            DataCommand sqlRun;
            DataTable dbInfo;
            string result;

            // ----- Initialize.
            result = "";

            // ----- Retrieve the value from the database.
            sqlText = "SELECT * FROM Holiday WHERE ID = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", recordID);
            try
            {
                dbInfo = CreateDataTable(sqlRun);
            }
            catch (Exception ex)
            {
                GeneralError("Holiday.FormatRecordName", ex);
                return "Error";
            }

            // ----- Format the holiday for display.
            result = FormatOneEntry(dbInfo.Rows[0]);

            // ----- Finished.
            dbInfo.Dispose();
            dbInfo = null;
            return result;
        }

        public override string GetEditDescription()
        {
            // ----- Return a description of this editor.
            return "Use these codes to establish the different holidays, which can " +
                "not be used for item due dates.";
        }

        public override string GetEditTitle()
        {
            // ----- Return the title-bar text for this editor.
            return "Holidays";
        }

        private void PrepareFormFields()
        {
            // ----- Fill in the data.
            string sqlText;
            DataCommand sqlRun;
            DataTable dbInfo;
            DataRow oneRow;
            string holdText;
            int holdValue;
            DateTime holdDate;

            // ----- Load in the list values.
            if (RecordType.Items.Count == 0)
            {
                // ----- Fill in the list of item types.
                RecordType.Items.Add(new ListItemData("<Not Selected>", ' '));
                RecordType.Items.Add(new ListItemData("Annual Holiday", 'A'));
                RecordType.Items.Add(new ListItemData("Weekly Holiday", 'E'));
                RecordType.Items.Add(new ListItemData("One-time Holiday", 'O'));

                // ----- Fill in the list of months.
                RecordMonth.Items.Clear();
                for (int counter = 1; counter <= 12; counter++)
                    RecordMonth.Items.Add(new ListItemData(
                        System.Threading.Thread.CurrentThread.CurrentCulture.
                        DateTimeFormat.GetMonthName(counter), (long)counter));

                // ----- Fill in the list of days.
                RecordDay.Items.Clear();
                for (long counter = 1; counter <= 31; counter++)
                    RecordDay.Items.Add(new ListItemData(counter.ToString(), counter));

                // ----- Fill in the list of weekdays.
                RecordWeekday.Items.Clear();
                for (int counter = 0; counter < 7; counter++)
                {
                    RecordWeekday.Items.Add(new ListItemData(
                        System.Threading.Thread.CurrentThread.CurrentCulture.
                        DateTimeFormat.GetDayName((System.DayOfWeek)counter), (long)counter));
                }
            }

            // ----- Clear the existing data.
            StartingOver = true;
            RecordName.Text = "";
            RecordType.SelectedIndex = RecordType.Items.IndexOf(' ');
            RecordMonth.SelectedIndex = RecordMonth.Items.IndexOf(1L);
            RecordDay.SelectedIndex = RecordDay.Items.IndexOf(1L);
            RecordWeekday.SelectedIndex = RecordWeekday.Items.IndexOf(
                Convert.ToInt64(DayOfWeek.Sunday));
            RecordOneTime.Value = DateTime.Today;
            if (ActiveID == -1L)
                return;

            // ----- Load in the values stored in the database.
            sqlText = "SELECT * FROM Holiday WHERE ID = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", ActiveID);
            try
            {
                dbInfo = CreateDataTable(sqlRun);
            }
            catch (Exception ex)
            {
                GeneralError("Holiday.PrepareFormFields", ex);
                return;
            }

            if (dbInfo.Rows.Count > 0)
            {
                oneRow = dbInfo.Rows[0];
                RecordName.Text = DBGetText(oneRow["FullName"]);
                RecordType.SelectedIndex = RecordType.Items.IndexOf(
                    DBGetText(oneRow["EntryType"]).ToUpper()[0]);
                if (RecordType.SelectedIndex == -1)
                {
                    // ----- Something wrong with this record.
                    RecordType.SelectedIndex = RecordType.Items.IndexOf(' ');
                }
                else
                {
                    switch (DBGetText(oneRow["EntryType"]).ToUpper())
                    {
                        case "A":
                            // ----- Annually.
                            holdText = DBGetText(oneRow["EntryDetail"]) + "/2004";
                            if (DateTime.TryParse(holdText, out holdDate) == true)
                            {
                                RecordMonth.SelectedIndex = RecordMonth.Items.IndexOf(
                                    (long)holdDate.Month);
                                RecordDay.SelectedIndex = RecordDay.Items.IndexOf(
                                    (long)holdDate.Day);
                            }
                            break;
                        case "E":
                            // ----- Weekly.
                            if (int.TryParse(DBGetText(oneRow["EntryDetail"]),
                                    out holdValue) == false)
                                holdValue = -1;
                            if ((holdValue >= (int)DayOfWeek.Sunday) &
                                    (holdValue <= (int)DayOfWeek.Saturday))
                                RecordWeekday.SelectedIndex =
                                    RecordWeekday.Items.IndexOf((long)holdValue);
                            break;
                        case "O":
                            // ----- One-time.
                            if (DateTime.TryParse(DBGetText(oneRow["EntryDetail"]),
                                    out holdDate) == true)
                                RecordOneTime.Value = holdDate;
                            break;
                    }
                }
            }
            dbInfo.Dispose();
            dbInfo = null;
        }

        private bool SaveFormData()
        {
            // ----- Save changes to the record.
            string sqlText;
            DataCommand sqlRun;
            string entryType;
            string entryDetail;
            long newID;

            this.Cursor = Cursors.WaitCursor;

            // ----- Get the entry detail content.
            entryType = ((char)(ListItemData)RecordType.SelectedItem).ToString();
            switch (entryType)
            {
                case "A":
                    // ----- A: Annually.
                    entryDetail = string.Format("{0}/{1}",
                        (long)(ListItemData)RecordMonth.SelectedItem,
                        (long)(ListItemData)RecordDay.SelectedItem);
                    break;
                case "E":
                    // ----- W: Weekly.
                    entryDetail = ((long)(ListItemData)RecordWeekday.SelectedItem).ToString();
                    break;
                default:
                    // ----- O: One-time.
                    entryDetail = RecordOneTime.Value.ToString("yyyy/MM/dd");
                    break;
            }

            // ----- Save the data.
            if (ActiveID == -1L)
            {
                // ----- Create a new entry.
                sqlText = "INSERT INTO Holiday(FullName, EntryType, EntryDetail) " +
                    "OUTPUT INSERTED.ID VALUES (@NewName, @NewType, @NewDetail)";
                sqlRun = new DataCommand(sqlText);
                sqlRun.AddText("NewName", RecordName.Text.Trim());
                sqlRun.AddText("NewType", entryType);
                sqlRun.AddText("NewDetail", entryDetail);
                try
                {
                    newID = DBGetLong(ExecuteSQLReturn(sqlRun), -1L);
                }
                catch (Exception ex)
                {
                    GeneralError("Holiday.SaveFormData.Add", ex);
                    return false;
                }
                finally
                {
                    this.Cursor = Cursors.Default;
                }
                ActiveID = newID;
            }
            else
            {
                // ----- Update the existing entry.
                sqlText = "UPDATE Holiday SET FullName = @NewName, " +
                    "EntryType = @NewType, EntryDetail = @NewDetail " +
                    "WHERE ID = @RecordID";
                sqlRun = new DataCommand(sqlText);
                sqlRun.AddText("NewName", RecordName.Text.Trim());
                sqlRun.AddText("NewType", entryType);
                sqlRun.AddText("NewDetail", entryDetail);
                sqlRun.AddLong("RecordID", ActiveID);
                try
                {
                    ExecuteSQL(sqlRun);
                }
                catch (Exception ex)
                {
                    GeneralError("Holiday.SaveFormData.Update", ex);
                    return false;
                }
                finally
                {
                    this.Cursor = Cursors.Default;
                }
            }

            // ----- Success.
            return true;
        }

        private bool ValidateFormData()
        {
            // ----- Make sure the user supplied valid data. Return True on success.
            string holdText;

            // ----- Name is a required field.
            if (RecordName.Text.Trim().Length == 0)
            {
                MessageBox.Show("Please supply the holiday name.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                RecordName.Focus();
                return false;
            }

            // ----- Make sure a valid type is selected.
            if ((ListItemData)RecordType.SelectedItem == ' ')
            {
                MessageBox.Show("Please select a holiday type.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                RecordName.Focus();
                return false;
            }

            // ----- If an annual holiday is used, check for a valid date. I
            //       used 2004 since it has a leap day in it, but the year used
            //       isn't that important.
            if ((ListItemData)RecordType.SelectedItem == 'A')
            {
                holdText = string.Format("{0}/{1}/2004",
                    ((long)(ListItemData)RecordMonth.SelectedItem).ToString(),
                    ((long)(ListItemData)RecordDay.SelectedItem).ToString());
                if (CheckDateTime(holdText) == false)
                {
                    MessageBox.Show("Please select a valid day for the selected month.",
                        ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    RecordDay.Focus();
                    return false;
                }
            }

            // ----- Success.
            return true;
        }

        private void Holiday_KeyDown(object sender, KeyEventArgs e)
        {
            // ----- F1 shows online help.
            if (e.KeyCode == Keys.F1)
                OnlineHelp(this, "Holiday.htm");
        }

        private void Holiday_Load(object sender, EventArgs e)
        {
            // ----- Adjust the elements on the form.
            PanelAnnually.Location = new Point(PanelWeekly.Location.X, PanelWeekly.Location.Y);
            PanelOneTime.Location = new Point(PanelWeekly.Location.X, PanelWeekly.Location.Y);
        }

        private void ActOK_Click(object sender, EventArgs e)
        {
            // ----- The user wants to save changes.
            if (ValidateFormData() == false)
                return;
            if (SaveFormData() == false)
                return;
            this.DialogResult = DialogResult.OK;
        }

        private void Holiday_Activated(object sender, EventArgs e)
        {
            // ----- Return the focus to the main field.
            if (StartingOver)
                RecordName.Focus();
            StartingOver = false;
        }

        private void RecordName_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordName.SelectAll();
        }

        private void RecordType_SelectedIndexChanged(object sender, EventArgs e)
        {
            // ----- Display the correct detail fields.
            if (RecordType.SelectedIndex == -1)
                return;
            switch ((char)(ListItemData)RecordType.SelectedItem)
            {
                case ' ':
                    // ----- Not selected.
                    PanelAnnually.Visible = false;
                    PanelWeekly.Visible = false;
                    PanelOneTime.Visible = false;
                    break;
                case 'A':
                    // ----- Annually.
                    PanelAnnually.Visible = true;
                    PanelWeekly.Visible = false;
                    PanelOneTime.Visible = false;
                    break;
                case 'E':
                    // ----- Weekly.
                    PanelAnnually.Visible = false;
                    PanelWeekly.Visible = true;
                    PanelOneTime.Visible = false;
                    break;
                case 'O':
                    // ----- One-time.
                    PanelAnnually.Visible = false;
                    PanelWeekly.Visible = false;
                    PanelOneTime.Visible = true;
                    break;
            }
        }
    }
}
