﻿// ----- The Library Project
//       Start-to-Finish Visual C# 2015
//       Copyright (c) 2016 by Tim Patrick

using System;
using System.Data;
using System.Drawing;
using System.Windows.Forms;
using static Library.General;

namespace Library
{
    public partial class AuthorAddLocate : Form
    {
        private int ActivePanel;

        private const int PanelType = 1;
        private const int PanelCriteria = 2;
        private const int PanelResults = 3;

        public AuthorAddLocate()
        {
            InitializeComponent();
        }

        private int FindUserRecords(string lastText, string firstText)
        {
            // ----- Search for author names using user-specified criteria.
            string sqlText;
            DataCommand sqlRun;
            DataTable dbInfo;
            int matches;

            // ----- Clear any existing values.
            SearchResults.Items.Clear();
            this.Cursor = Cursors.WaitCursor;

            // ----- Retrieve the values from the database.
            matches = 0;
            sqlText = "SELECT * FROM Author WHERE LastName LIKE @TestLast";
            if (firstText.Length != 0)
                sqlText += " AND FirstName LIKE @TestFirst";
            sqlText += " ORDER BY LastName, FirstName";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddText("TestLast", lastText);
            if (firstText.Length != 0)
                sqlRun.AddText("TestFirst", firstText);
            try
            {
                dbInfo = CreateDataTable(sqlRun);
            }
            catch (Exception ex)
            {
                this.Cursor = Cursors.Default;
                GeneralError("AuthorAddLocate.FindUserRecords", ex);
                return -1;
            }

            foreach (DataRow dbRow in dbInfo.Rows)
            {
                // ----- Add one item.
                SearchResults.Items.Add(new ListItemData(
                    dbRow.FormatAuthorName(), DBGetLong(dbRow["ID"])));
                matches += 1;
                if (matches >= SearchMatchLimit)
                    break;
            }
            dbInfo.Dispose();
            dbInfo = null;
            this.Cursor = Cursors.Default;
            return matches;
        }

        public long PromptUser(ref long nameTypeID)
        {
            // ----- Prompt the user for a author. Return the matching ID. Return
            //       -1 if no change made.
            ActivePanel = PanelType;
            this.ShowDialog();
            if (this.DialogResult == DialogResult.OK)
            {
                nameTypeID = (long)(ListItemData)NameType.SelectedItem;
                if ((long)(ListItemData)SearchResults.SelectedItem == -1L)
                {
                    // ----- Prompt for a new author using the original
                    //       typed name.
                    return (new Author()).AddRecord(
                        LastName.Text.Trim(), FirstName.Text.Trim());
                }
                else
                {
                    // ----- The user selected a match.
                    return (long)(ListItemData)SearchResults.SelectedItem;
                }
            }
            else
            {
                // ----- Cancelled by user.
                nameTypeID = -1L;
                return -1L;
            }
        }

        private void ActBack_Click(object sender, EventArgs e)
        {
            // ----- Go back to the previous panel.
            switch (ActivePanel)
            {
                case PanelCriteria:
                    // ----- Go back to the Name Type panel.
                    ActivePanel = PanelType;
                    SecondPanel.Visible = false;
                    FirstPanel.Visible = true;
                    ActBack.Enabled = false;
                    NameType.Focus();
                    break;
                case PanelResults:
                    // ----- Go back to the Last/First Name panel.
                    ActivePanel = PanelCriteria;
                    ThirdPanel.Visible = false;
                    SecondPanel.Visible = true;
                    ActNext.Text = "&Next >";
                    LastName.Focus();
                    break;
            }
        }

        private void ActNext_Click(object sender, EventArgs e)
        {
            // ----- Take action on the user input.
            string lastText;
            string firstText;
            int matches;

            // ----- Different code for each of the panels.
            if (ActivePanel == PanelType)
            {
                // ----- Make sure a name type is selected.
                if ((long)(ListItemData)NameType.SelectedItem == -1L)
                {
                    MessageBox.Show("Please select a name type from the list.",
                        ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    NameType.Focus();
                    return;
                }

                // ----- Move to the search panel.
                ActivePanel = PanelCriteria;
                SecondPanel.Visible = true;
                FirstPanel.Visible = false;
                ActBack.Enabled = true;
                LastName.Focus();
            }
            else if (ActivePanel == PanelCriteria)
            {
                // ----- Determine the search string.
                lastText = LastName.Text.Trim();
                if (lastText.Length == 0)
                {
                    MessageBox.Show("Please supply a last name.",
                        ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    LastName.Focus();
                    return;
                }
                if (lastText.Contains("*") == false)
                    lastText += "*";
                lastText = lastText.Replace("*", "%");
                firstText = FirstName.Text.Trim();
                if (firstText.Contains("*") == false)
                    firstText += "*";
                firstText = firstText.Replace("*", "%");

                // ----- Search for matches.
                matches = FindUserRecords(lastText, firstText);
                if (matches == -1)
                    return;

                // ----- If there are no matches, then either error out, or
                //       add a new record, depending on the user's security.
                //       If there are matches, show them.
                if (matches == 0)
                {
                    if (SecurityProfile[(int)LibrarySecurity.ManageAuthors])
                    {
                        // ----- Prompt to add a new record.
                        SearchResults.ClearSelected();
                        SearchResults.Items.Add(new ListItemData(
                            "<No matches. Add new author or name.>", -1L));
                        SearchResults.SelectedIndex = 0;

                        // ----- Return to the PromptUser() routine.
                        this.DialogResult = DialogResult.OK;
                    }
                    else
                    {
                        // ----- Give an error.
                        MessageBox.Show("No matches.", ProgramTitle,
                            MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                        return;
                    }
                }
                else
                {
                    // ----- If the user can add new author, add that option.
                    if (SecurityProfile[(int)LibrarySecurity.ManageAuthors])
                        SearchResults.Items.Insert(0, new ListItemData(
                            "<None of these matches. Add new author or name.>", -1L));
                    ActivePanel = PanelResults;
                    ThirdPanel.Visible = true;
                    SecondPanel.Visible = false;
                    ActNext.Text = "Finish";
                    SearchResults.Focus();
                }
            }
            else
            {
                // ----- Check for a selected result.
                if (SearchResults.SelectedIndex == -1)
                {
                    MessageBox.Show("Please select a match from the list.",
                        ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    SearchResults.Focus();
                    return;
                }

                // ----- Return to the PromptUser() routine.
                this.DialogResult = DialogResult.OK;
            }
        }

        private void SearchResults_DoubleClick(object sender, EventArgs e)
        {
            // ----- Same as the finish button.
            if (SearchResults.SelectedIndex == -1)
                return;
            ActNext.PerformClick();
        }

        private void AuthorAddLocate_KeyDown(object sender, KeyEventArgs e)
        {
            // ----- F1 shows online help.
            if (e.KeyCode == Keys.F1)
                OnlineHelp(this, "AuthorAddLocate.htm");
        }

        private void AuthorAddLocate_Load(object sender, EventArgs e)
        {
            // ----- Prepare the form.
            string sqlText;
            DataTable dbInfo;

            // ----- Adjust the wizard panels.
            SecondPanel.Location = new Point(
                FirstPanel.Location.X, FirstPanel.Location.Y);
            ThirdPanel.Location = new Point(
                FirstPanel.Location.X, FirstPanel.Location.Y);

            // ----- Fill in the list of named items.
            NameType.Items.Add(new ListItemData("<Select a Name Type>", -1L));
            NameType.SelectedIndex = 0;
            sqlText = "SELECT * FROM CodeAuthorType ORDER BY FullName";
            try
            {
                dbInfo = CreateDataTable(sqlText);
            }
            catch (Exception ex)
            {
                GeneralError("AuthorAddLocate.AuthorAddLocate_Load", ex);
                return;
            }

            foreach (DataRow dbRow in dbInfo.Rows)
            {
                // ----- Add one item.
                NameType.Items.Add(new ListItemData(
                    DBGetText(dbRow["FullName"]), DBGetLong(dbRow["ID"])));
            }
            dbInfo.Dispose();
            dbInfo = null;
        }

        private void LastName_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            LastName.SelectAll();
        }

        private void FirstName_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            FirstName.SelectAll();
        }
    }
}
