﻿// ----- The Library Project
//       Start-to-Finish Visual C# 2015
//       Copyright (c) 2016 by Tim Patrick

using System;
using System.Collections.Generic;
using System.Data;
using System.Data.SqlClient;
using System.Drawing;
using System.Reflection;
using System.Security.Cryptography;
using System.Text;
using System.Windows.Forms;

namespace Library
{
    internal static class General
    {
        // ----- Public constants.
        public const string ProgramTitle = "The Library Project";
        public const string NotAuthorizedMessage = "You are not authorized to perform this task.";
        public const int UseDBVersion = 1;

        // ----- Constants for the MatchingImages image list.
        public const int MatchPresent = 0;
        public const int MatchNone = 1;

        public enum LookupMethods: int
        {
            ByTitle = 1,
            ByAuthor = 2,
            BySubject = 3,
            ByKeywordAny = 4,
            ByKeywordAll = 5,
            ByPublisher = 6,
            BySeries = 7,
            BySeriesID = 8,
            ByBarcode = 9,
            ByDatabaseID = 10,
            ByAuthorID = 11,
            BySubjectID = 12,
            ByPublisherID = 13
        }

        // ----- Security values.
        public enum LibrarySecurity: int
        {
            ManageAuthors = 1,
            ManageAuthorTypes = 2,
            ManageCopyStatus = 3,
            ManageMediaTypes = 4,
            ManageSeries = 5,
            ManageGroups = 6,
            ManageItems = 7,
            ManagePatrons = 8,
            ManagePublishers = 9,
            ManageValues = 10,
            ManageUsers = 11,
            ProcessFees = 12,
            ManageLocations = 13,
            CheckOutItems = 14,
            CheckInItems = 15,
            AdminFeatures = 16,
            DailyProcessing = 17,
            RunReports = 18,
            PatronOverride = 19,
            ManageBarcodeTemplates = 20,
            ManageHolidays = 21,
            ManagePatronGroups = 22,
            ViewAdminPatronMessages = 23
        }
        public const LibrarySecurity MaxLibrarySecurity = LibrarySecurity.ViewAdminPatronMessages;

        // ----- Global variables.
        public static long LoggedInUserID;
        public static string LoggedInUserName;
        public static long LoggedInGroupID;
        public static bool[] SecurityProfile = new bool[(int)MaxLibrarySecurity + 1];

        private static string ActiveConnectionString = "";

        public static string CenterText(string origText, int textWidth)
        {
            // ----- Center a piece of text in a field width. If the text is too
            //       wide, truncate it.
            string resultText;

            resultText = origText.Trim();
            if (resultText.Length >= textWidth)
            {
                // ----- Truncate as needed.
                return origText.Substring(0, textWidth).Trim();
            }
            else
            {
                // ----- Start with extra spaces.
                return new string(' ',
                    (textWidth - origText.Length) / 2) + resultText;
            }
        }

        public static bool CheckDateTime(string content)
        {
            // ----- Return True if a string contains a valid DateTime.
            DateTime result;

            return DateTime.TryParse(content, out result);
        }

        public static bool CheckDecimal(string content)
        {
            // ----- Return True if a string contains a valid decimal.
            decimal result;

            return decimal.TryParse(content, out result);
        }

        public static bool CheckInt(string content)
        {
            // ----- Return True if a string contains a valid int.
            int result;

            return int.TryParse(content, out result);
        }

        public static bool CheckLong(string content)
        {
            // ----- Return True if a string contains a valid long.
            long result;

            return long.TryParse(content, out result);
        }

        public static SqlConnection ConnectDatabase()
        {
            // ----- Connect to the database. Throw exception on failure.
            SqlConnection libraryDB;

            // ----- Build the connection string.
            // !!! WARNING: Hardcoded for now.
            if (ActiveConnectionString.Length == 0)
                ActiveConnectionString = "Data Source=MYSYSTEM;" +
                   "Initial Catalog=Library;Integrated Security=true";

            // ----- Attempt to open the database.
            libraryDB = new SqlConnection(ActiveConnectionString);
            libraryDB.Open();

            // ----- Success.
            return libraryDB;
        }

        public static int CountSubStr(string mainText, string subText)
        {
            // ----- Return a count of the number of times that a subText occurs in
            //       a string (mainText).
            int totalTimes;
            int startPos;
            int foundPos;

            if ((mainText == null) || (mainText.Length == 0))
                return 0;
            if ((subText == null) || (subText.Length == 0))
                return 0;
            totalTimes = 0;
            startPos = 0;

            // ----- Keep searching until we don't find it no more!
            do
            {
                // ----- Search for the subText.
                foundPos = mainText.IndexOf(subText, startPos);
                if (foundPos == -1)
                    break;
                totalTimes += 1;

                // ----- Move to just after the occurrence.
                startPos = foundPos + subText.Length;
            } while (true);

            // ----- Return the count.
            return totalTimes;
        }

        public static DataTable CreateDataTable(DataCommand theAction)
        {
            // ----- Given a SQL statement with parameters, return a data table.
            SqlCommand dbCommand;
            SqlDataAdapter dbAdapter;
            DataTable dbTable;
            SqlConnection localConnection = null;

            // ----- Connect to the database, if not yet done. Throw exception on failure.
            if (theAction.Connection == null)
                localConnection = ConnectDatabase();

            // ----- Try to run the statement. Throw exception on failure.
            try
            {
                dbCommand = theAction.GetCommandObject();
                if (localConnection != null)
                    dbCommand.Connection = localConnection;
                dbAdapter = new SqlDataAdapter(dbCommand);
                dbTable = new DataTable();
                dbAdapter.Fill(dbTable);
            }
            catch
            {
                // ----- Reissue error after Finally clause.
                throw;
            }
            finally
            {
                // ----- Disconnect if needed.
                if (localConnection != null)
                    localConnection.Close();
            }
            return dbTable;
        }

        public static DataTable CreateDataTable(DataCommand theAction, LibraryTransaction transInfo)
        {
            // ----- Given a SQL statement with parameters, return a data table.
            SqlCommand dbCommand;
            SqlDataAdapter dbAdapter;
            DataTable dbTable;

            // ----- Try to run the statement. Throw exception on failure.
            dbCommand = theAction.GetCommandObject();
            dbCommand.Connection = transInfo.Connection;
            dbCommand.Transaction = transInfo.Transaction;
            dbAdapter = new SqlDataAdapter(dbCommand);
            dbTable = new DataTable();
            dbAdapter.Fill(dbTable);
            return dbTable;
        }

        public static DataTable CreateDataTable(string sqlText)
        {
            // ----- Given a SQL statement, return a data table.
            SqlCommand dbCommand;
            SqlDataAdapter dbAdapter;
            DataTable dbTable;

            // ----- Connect to the database. Throw exception on failure.
            using (SqlConnection libraryDB = ConnectDatabase())
            {
                // ----- Try to run the statement. Throw exception on failure.
                dbCommand = new SqlCommand(sqlText, libraryDB);
                dbAdapter = new SqlDataAdapter(dbCommand);
                dbTable = new DataTable();
                dbAdapter.Fill(dbTable);
            }
            return dbTable;
        }

        public static DataTable CreateDataTable(string sqlText, LibraryTransaction transInfo)
        {
            // ----- Given a SQL statement and a transaction, return a data table.
            SqlCommand dbCommand;
            SqlDataAdapter dbAdapter;
            DataTable dbTable;

            // ----- Try to run the statement. Throw exception on failure.
            dbCommand = new SqlCommand(sqlText, transInfo.Connection);
            dbCommand.Transaction = transInfo.Transaction;
            dbAdapter = new SqlDataAdapter(dbCommand);
            dbTable = new DataTable();
            dbAdapter.Fill(dbTable);
            return dbTable;
        }

        public static string DBComboParam(ComboBox whichField)
        {
            // ----- Prepare the value of a ComboBox for use in the database.
            ListItemData listItem = null;

            // ----- Get the stored data for this item.
            if (whichField.SelectedItem == null)
                return "";
            listItem = (ListItemData)whichField.SelectedItem;

            // ----- An item value of "-1" has special meaning: unselected.
            if (listItem.ItemData == -1L)
                return "";
            else
                return listItem.ItemData.ToString();
        }

        public static bool DBGetBool(object dataField)
        {
            // ----- Return the Boolean equivalent of an optional database field.
            return Convert.ToBoolean(DBGetInteger(dataField));
        }

        public static DateTime DBGetDate(object dataField)
        {
            // ----- Return the date equivalent of an optional database field.
            DateTime holdDate;

            if (dataField == null)
                return DateTime.MinValue;
            else if (Convert.IsDBNull(dataField) == true)
                return DateTime.MinValue;
            else if (DateTime.TryParse(dataField.ToString(), out holdDate) == false)
                return DateTime.MinValue;
            else
                return holdDate;
        }

        public static decimal DBGetDecimal(object dataField)
        {
            // ----- Return the decimal equivalent of an optional database field.
            if (Convert.IsDBNull(dataField) == true)
                return 0M;
            else
                return Convert.ToDecimal(dataField);
        }

        public static int DBGetInteger(object dataField)
        {
            // ----- Return the integer equivalent of an optional database field.
            if (Convert.IsDBNull(dataField) == true)
                return 0;
            else
                return Convert.ToInt32(dataField);
        }

        public static long DBGetLong(object dataField, long useDefault = 0L)
        {
            // ----- Return the long equivalent of an optional database field.
            if (dataField == null)
                return useDefault;
            else if (Convert.IsDBNull(dataField) == true)
                return useDefault;
            else
                return Convert.ToInt64(dataField);
        }

        public static string DBGetText(object dataField, string defaultText = "")
        {
            // ----- Return the text equivalent of an optional database field.
            if (Convert.IsDBNull(dataField) == true)
                return defaultText;
            else
                return Convert.ToString(dataField);
        }

        public static string DigitsOnly(string origText)
        {
            // ----- Return only the digits found in a string.
            string destText;

            // ----- Examine each character.
            destText = "";
            for (int counter = 0; counter < origText.Length; counter++)
            {
                if (char.IsDigit(origText[counter]) == true)
                    destText += origText[counter];
            }
            return destText;
        }

        public static string EncryptPassword(string loginID, string passwordText)
        {
            // ----- Given a username and a password, encrypt the password so that it
            //       is not easy to decrypt. There is no limit on password length since
            //       it is going to be hashed anyway.
            HMACSHA1 hashingFunction;
            byte[] secretKey;
            byte[] hashValue;
            string result = "";

            // ----- Prepare the secret key. Force it to uppercase for consistency,
            //       and then stuff it in a byte array.
            secretKey = (new UnicodeEncoding()).GetBytes(loginID.ToUpper());

            // ----- Create the hashing component using Managed SHA-1.
            hashingFunction = new HMACSHA1(secretKey, true);

            // ----- Calculate the hash value. One simple line of code.
            hashValue = hashingFunction.ComputeHash((new UnicodeEncoding()).GetBytes(passwordText));

            // ----- The hash value is ready, but I like things in plain text when
            //       possible. Let's convert it to a long hex string.
            for (int counter = 0; counter < hashValue.Length; counter++)
                result += Convert.ToString(hashValue[counter], 16).ToUpper();

            // ----- Stored passwords are limited to 20 characters.
            if (result.Length > 20)
                result = result.Substring(0, 20);
            return result;
        }

        public static void ExecuteSQL(DataCommand theAction)
        {
            // ----- Given a SQL statement with parameters, run it.
            SqlCommand dbCommand;
            SqlConnection localConnection = null;

            // ----- Connect to the database, if not yet done. Throw exception on failure.
            if (theAction.Connection == null)
                localConnection = ConnectDatabase();

            // ----- Try to run the statement. Throw exception on failure.
            try
            {
                dbCommand = theAction.GetCommandObject();
                if (localConnection != null)
                    dbCommand.Connection = localConnection;
                dbCommand.ExecuteNonQuery();
            }
            catch
            {
                // ----- Reissue error after Finally clause.
                throw;
            }
            finally
            {
                // ----- Disconnect if needed.
                if (localConnection != null)
                    localConnection.Close();
            }
        }

        public static void ExecuteSQL(DataCommand theAction, LibraryTransaction transInfo)
        {
            // ----- Given a SQL statement with parameters, run it.
            SqlCommand dbCommand;

            // ----- Try to run the statement. Throw exception on failure.
            dbCommand = theAction.GetCommandObject();
            dbCommand.Connection = transInfo.Connection;
            dbCommand.Transaction = transInfo.Transaction;
            dbCommand.ExecuteNonQuery();
        }

        public static void ExecuteSQL(string sqlText)
        {
            // ----- Given a SQL statement, run it.
            SqlCommand dbCommand;

            // ----- Connect to the database. Throw exception on failure.
            using (SqlConnection libraryDB = ConnectDatabase())
            {
                // ----- Try to run the statement. Throw exception on failure.
                dbCommand = new SqlCommand(sqlText, libraryDB);
                dbCommand.ExecuteNonQuery();
            }
        }

        public static void ExecuteSQL(string sqlText, LibraryTransaction transInfo)
        {
            // ----- Given a SQL statement, run it.
            SqlCommand dbCommand;

            // ----- Connect to the database. Throw exception on failure.
            dbCommand = new SqlCommand(sqlText, transInfo.Connection);
            dbCommand.Transaction = transInfo.Transaction;
            dbCommand.ExecuteNonQuery();
        }

        public static object ExecuteSQLReturn(DataCommand theAction)
        {
            // ----- Given a SQL statement with parameters, run it, and return a result.
            SqlCommand dbCommand;
            SqlConnection localConnection = null;

            // ----- Connect to the database, if not yet done. Throw exception on failure.
            if (theAction.Connection == null)
                localConnection = ConnectDatabase();

            // ----- Try to run the statement. Throw exception on failure.
            try
            {
                dbCommand = theAction.GetCommandObject();
                if (localConnection != null)
                    dbCommand.Connection = localConnection;
                return dbCommand.ExecuteScalar();
            }
            catch
            {
                // ----- Reissue error after Finally clause.
                throw;
            }
            finally
            {
                // ----- Disconnect if needed.
                if (localConnection != null)
                    localConnection.Close();
            }
        }

        public static object ExecuteSQLReturn(DataCommand theAction, LibraryTransaction transInfo)
        {
            // ----- Given a SQL statement with parameters, run it, and return a result.
            SqlCommand dbCommand;

            // ----- Try to run the statement. Throw exception on failure.
            dbCommand = theAction.GetCommandObject();
            dbCommand.Connection = transInfo.Connection;
            dbCommand.Transaction = transInfo.Transaction;
            return dbCommand.ExecuteScalar();
        }

        public static object ExecuteSQLReturn(string sqlText)
        {
            // ----- Given a SQL statement, run it, and return a result.
            SqlCommand dbCommand;

            // ----- Connect to the database. Throw exception on failure.
            using (SqlConnection libraryDB = ConnectDatabase())
            {
                // ----- Try to run the statement. Throw exception on failure.
                dbCommand = new SqlCommand(sqlText, libraryDB);
                return dbCommand.ExecuteScalar();
            }
        }

        public static object ExecuteSQLReturn(string sqlText, LibraryTransaction transInfo)
        {
            // ----- Given a SQL statement, run it, and return a result.
            SqlCommand dbCommand;

            // ----- Connect to the database. Throw exception on failure.
            dbCommand = new SqlCommand(sqlText, transInfo.Connection);
            dbCommand.Transaction = transInfo.Transaction;
            return dbCommand.ExecuteScalar();
        }

        public static void GeneralError(string routineName, Exception theError)
        {
            // ----- Report an error to the user.
            MessageBox.Show("The following error occurred at location '" + routineName + "':" +
                Environment.NewLine + Environment.NewLine + theError.Message,
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
        }

        public static bool GetBoolean(string content)
        {
            // ----- Extract a boolean value from a string. Return False if
            //       the string contains no legitimate value.
            int holdInt;
            bool holdBool;

            if (int.TryParse(content, out holdInt) == true)
                return Convert.ToBoolean(holdInt);
            else if (bool.TryParse(content, out holdBool) == true)
                return holdBool;
            else
                return false;
        }

        public static string GetSubStr(string origString, string delim, int whichField)
        {
            // ----- Extracts a delimited string from another larger string.
            string[] stringParts = null;

            // ----- Handle some errors.
            if (whichField < 0)
                return "";
            if ((origString == null) || (origString.Length == 0))
                return "";
            if ((delim == null) || (delim.Length == 0))
                return "";

            // ----- Break the string up into delimited parts.
            stringParts = origString.Split(new[] {delim}, StringSplitOptions.None);

            // ----- See if the part we want exists and return it.
            if (whichField > stringParts.Length)
                return "";
            else
                return stringParts[whichField - 1];
        }

        public static string GetSystemValue(string valueName)
        {
            // ----- Return the data portion of a system value name-data pair.
            string sqlText;
            DataCommand sqlRun;

            // ----- Retrieve the value.
            sqlText = "SELECT ValueData FROM SystemValue " +
                "WHERE UPPER(ValueName) = @TestName";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddText("TestName", valueName.ToUpper());
            try
            {
                return DBGetText(ExecuteSQLReturn(sqlRun));
            }
            catch (Exception ex)
            {
                GeneralError("GetSystemValue", ex);
                return "";
            }
        }

        public static void InitializeSystem()
        {
            // ----- Initialize global variables here.  

            // ----- Clear security-related values.
            LoggedInUserID = -1L;
            LoggedInUserName = "";
            LoggedInGroupID = -1L;

            for (int counter = 1; counter <= (int)MaxLibrarySecurity; counter++)
                SecurityProfile[counter] = false;
        }

        public static string LeftAndRightText(string origLeft, string origRight, int textWidth)
        {
            // ----- Build a string of the indicated width that has both left-justified
            //       and right-justified text.
            string leftText;
            string rightText;

            leftText = origLeft.Trim();
            rightText = origRight.Trim();
            if ((leftText + rightText).Length >= textWidth)
                return (leftText + rightText).Substring(0, textWidth).Trim();
            else
                return leftText + new string(' ', textWidth -
                    (leftText + rightText).Length) + rightText;
        }

        public static void ReprocessSecuritySet()
        {
            // ----- Reload in the security set for the current user. If no user
            //       is logged in, clear all settings.
            string sqlText;
            DataCommand sqlRun;
            DataTable dbInfo = null;

            // ----- Clear out the existing items.
            for (int counter = 1; counter <= (int)MaxLibrarySecurity; counter++)
                SecurityProfile[counter] = false;

            // ----- Exit if there is no user logged in.
            if ((LoggedInUserID == -1L) | (LoggedInGroupID == -1L))
                return;

            try
            {
                // ----- Load in the security elements for this user.
                sqlText = "SELECT ActivityID FROM GroupActivity " +
                    "WHERE GroupID = @RecordID";
                sqlRun = new DataCommand(sqlText);
                sqlRun.AddLong("RecordID", LoggedInGroupID);
                dbInfo = CreateDataTable(sqlRun);
                foreach (DataRow dbRow in dbInfo.Rows)
                    SecurityProfile[DBGetLong(dbRow["ActivityID"])] = true;
            }
            catch (Exception ex)
            {
                // ----- Some database-related error.
                GeneralError("ReprocessSecuritySet", ex);

                // ----- Un-login the administrator through recursion.
                LoggedInUserID = -1L;
                LoggedInGroupID = -1L;
                ReprocessSecuritySet();
            }
            finally
            {
                if (dbInfo != null)
                    dbInfo.Dispose();
                dbInfo = null;
            }
        }

        public static void SetSystemValue(string valueName, string valueData)
        {
            // ----- Update a record in the SystemValue table.
            string sqlText;
            DataCommand sqlRun;

            try
            {
                // ----- See if the entry already exists.
                sqlText = "SELECT COUNT(*) FROM SystemValue " +
                    "WHERE UPPER(ValueName) = @TestName";
                sqlRun = new DataCommand(sqlText);
                sqlRun.AddText("TestName", valueName.ToUpper());
                if (DBGetInteger(ExecuteSQLReturn(sqlRun)) > 0)
                {
                    // ----- Value already exists.
                    sqlText = "UPDATE SystemValue SET ValueData = @NewData " +
                        "WHERE UPPER(ValueName) = @TestName";
                    sqlRun = new DataCommand(sqlText);
                    sqlRun.AddText("NewData", valueData);
                    sqlRun.AddText("TestName", valueName.ToUpper());
                }
                else
                {
                    // ----- Need to create value.
                    sqlText = "INSERT INTO SystemValue (ValueName, ValueData) " +
                        "VALUES (@NewName, @NewData)";
                    sqlRun = new DataCommand(sqlText);
                    sqlRun.AddText("NewName", valueName);
                    sqlRun.AddText("NewData", valueData);
                }

                // ----- Update the value.
                ExecuteSQL(sqlRun);
            }
            catch (Exception ex)
            {
                GeneralError("SetSystemValue", ex);
            }
        }

        public static LibraryTransaction TransactionBegin()
        {
            // ----- Create a new database transaction. Throw exception on failure.
            LibraryTransaction transInfo;

            // ----- Connect to the database.
            transInfo = new LibraryTransaction();
            transInfo.Connection = ConnectDatabase();
            transInfo.Transaction = transInfo.Connection.BeginTransaction();
            return transInfo;
        }

        public static void TransactionCommit(LibraryTransaction transInfo)
        {
            // ----- Commit the transaction.
            if (transInfo == null)
                return;
            try
            {
                if (transInfo.Transaction != null)
                    transInfo.Transaction.Commit();
            }
            catch
            {
                // ----- Reissue error after Finally clause.
                throw;
            }
            finally
            {
                transInfo.Transaction = null;
                if (transInfo.Connection != null)
                    transInfo.Connection.Close();
                transInfo.Connection = null;
            }
        }

        public static void TransactionRollback(LibraryTransaction transInfo)
        {
            // ----- Rollback the transaction.
            if (transInfo == null)
                return;
            try
            {
                if (transInfo.Transaction != null)
                    transInfo.Transaction.Rollback();
            }
            catch
            {
                // ----- Reissue error after Finally clause.
                throw;
            }
            finally
            {
                transInfo.Transaction = null;
                if (transInfo.Connection != null)
                    transInfo.Connection.Close();
                transInfo.Connection = null;
            }
        }
    }

    public class LibraryTransaction
    {
        public SqlConnection Connection;
        public SqlTransaction Transaction;

        public void Commit()
        {
            // ----- Manage the transaction commit.
            Exception saveError = null;

            // ----- Commit the transaction.
            try
            {
                if (this.Transaction != null)
                    this.Transaction.Commit();
            }
            catch (Exception ex)
            {
                // ----- Save error for later.
                saveError = ex;
            }

            // ----- Close the connection.
            try
            {
                if (this.Connection != null)
                    this.Connection.Close();
            }
            catch
            {
                // ----- Ignore any error.
            }

            // ----- Return error if it exists.
            if (saveError != null)
                throw saveError;
        }

        public void Rollback()
        {
            // ----- Manage the transaction rollback.
            Exception saveError = null;

            // ----- Rollback the transaction.
            try
            {
                if (this.Transaction != null)
                    this.Transaction.Rollback();
            }
            catch (Exception ex)
            {
                // ----- Save error for later.
                saveError = ex;
            }

            // ----- Close the connection.
            try
            {
                if (this.Connection != null)
                    this.Connection.Close();
            }
            catch
            {
                // ----- Ignore any error.
            }

            // ----- Return error if it exists.
            if (saveError != null)
                throw saveError;
        }
    }

    public class DataCommand
    {
        public enum CommandDataType: int
        {
            AsBoolean,
            AsDate,
            AsDateTime,
            AsLong,
            AsInteger,
            AsDecimal,
            AsText
        }

        public class CommandParameter
        {
            public string ParameterName = "";
            public object SourceData;
            public CommandDataType SourceType;
            public CommandDataType TargetType;
        }

        public SqlConnection Connection = null;
        public string CommandText = "";
        public Dictionary<string, CommandParameter> Parameters =
            new Dictionary<string, CommandParameter>();
        public bool IsStoredProcedure = false;

        public DataCommand()
        {
            // ----- Default constructor.
            this.Connection = null;
            this.CommandText = "";
        }

        public DataCommand(string sqlText)
        {
            // ----- Command-only constructor.
            this.Connection = null;
            this.CommandText = sqlText;
            if (sqlText == null)
                this.CommandText = "";
        }

        public DataCommand(string sqlText, SqlConnection connectDB)
        {
            // ----- Full constructor.
            this.Connection = connectDB;
            this.CommandText = sqlText;
            if (sqlText == null)
                this.CommandText = "";
        }

        public SqlCommand GetCommandObject()
        {
            // ----- Build a command object that includes all of the appropriate
            //       parameters.
            SqlCommand result = new SqlCommand();
            SqlParameter oneParam;
            string holdText;
            DateTime holdDate;

            // ----- Fill in the basics.
            result.Connection = this.Connection;
            result.CommandText = this.CommandText;
            if (this.IsStoredProcedure == true)
                result.CommandType = CommandType.StoredProcedure;
            else
                result.CommandType = CommandType.Text;

            foreach (CommandParameter scanParam in this.Parameters.Values)
            {
                // ----- Build a parameter object for the command.
                oneParam = null;
                switch (scanParam.TargetType)
                {
                    case CommandDataType.AsBoolean:
                        // ----- Source will always be Boolean. Convert to Integer.
                        oneParam = new SqlParameter(scanParam.ParameterName, SqlDbType.Bit);
                        oneParam.Value = Convert.ToBoolean(scanParam.SourceData) ? 1 : 0;
                        break;
                    case CommandDataType.AsDate:
                    case CommandDataType.AsDateTime:
                        // ----- Source will be date or string.
                        oneParam = new SqlParameter(scanParam.ParameterName, SqlDbType.DateTime);
                        if ((scanParam.SourceType == CommandDataType.AsDateTime) |
                            (scanParam.SourceType == CommandDataType.AsDate))
                        {
                            // ----- Date source.
                            if (scanParam.TargetType == CommandDataType.AsDate)
                            {
                                // ----- Exclude the time.
                                oneParam.Value = Convert.ToDateTime(scanParam.SourceData).Date;
                            }
                            else
                            {
                                // ----- Include the time.
                                oneParam.Value = Convert.ToDateTime(scanParam.SourceData);
                            }
                        }
                        else
                        {
                            // ----- String source.
                            if (DateTime.TryParse(Convert.ToString(scanParam.SourceData),
                                out holdDate) == false)
                            {
                                // ----- Store NULL.
                                oneParam.Value = System.DBNull.Value;
                            }
                            else
                            {
                                // ----- True date.
                                if (scanParam.TargetType == CommandDataType.AsDate)
                                {
                                    // ----- Exclude the time.
                                    oneParam.Value = holdDate.Date;
                                }
                                else
                                {
                                    // ----- Include the time.
                                    oneParam.Value = holdDate;
                                }
                            }
                        }
                        break;
                    case CommandDataType.AsLong:
                        // ----- String or true Long value.
                        oneParam = new SqlParameter(scanParam.ParameterName, SqlDbType.BigInt);
                        if (scanParam.SourceType == CommandDataType.AsText)
                        {
                            holdText = Convert.ToString(scanParam.SourceData).Trim();
                            if (holdText.Length == 0)
                                oneParam.Value = System.DBNull.Value;
                            else
                                oneParam.Value = Convert.ToInt64(holdText);
                        }
                        else
                        {
                            oneParam.Value = Convert.ToInt64(scanParam.SourceData);
                        }
                        break;
                    case CommandDataType.AsInteger:
                        // ----- String or true Integer value.
                        oneParam = new SqlParameter(scanParam.ParameterName, SqlDbType.Int);
                        if (scanParam.SourceType == CommandDataType.AsText)
                        {
                            holdText = Convert.ToString(scanParam.SourceData).Trim();
                            if (holdText.Length == 0)
                                oneParam.Value = System.DBNull.Value;
                            else
                                oneParam.Value = Convert.ToInt32(holdText);
                        }
                        else
                        {
                            oneParam.Value = Convert.ToInt32(scanParam.SourceData);
                        }
                        break;
                    case CommandDataType.AsDecimal:
                        // ----- String or true Decimal value.
                        oneParam = new SqlParameter(scanParam.ParameterName, SqlDbType.Decimal);
                        if (scanParam.SourceType == CommandDataType.AsText)
                        {
                            holdText = Convert.ToString(scanParam.SourceData).Trim();
                            if (holdText.Length == 0)
                                oneParam.Value = System.DBNull.Value;
                            else
                                oneParam.Value = Convert.ToDecimal(holdText);
                        }
                        else
                        {
                            oneParam.Value = Convert.ToDecimal(scanParam.SourceData);
                        }
                        break;
                    case CommandDataType.AsText:
                        // ----- Char, Varchar, Text, etc.
                        oneParam = new SqlParameter(scanParam.ParameterName, SqlDbType.VarChar);
                        holdText = Convert.ToString(scanParam.SourceData);
                        if ((holdText == null) || (holdText.Trim().Length == 0))
                            oneParam.Value = System.DBNull.Value;
                        else
                            oneParam.Value = holdText;
                        break;
                }

                // ----- Store the parameter in the command.
                if (oneParam != null)
                    result.Parameters.Add(oneParam);
            }

            // ----- Finished.
            return result;
        }

        public void AddBool(string paramName, bool content)
        {
            // ----- Add a new Boolean parameter.
            CommandParameter newEntry = new CommandParameter();

            newEntry.ParameterName = paramName;
            newEntry.SourceData = content;
            newEntry.SourceType = CommandDataType.AsBoolean;
            newEntry.TargetType = CommandDataType.AsBoolean;
            this.Parameters.Add(paramName, newEntry);
        }

        public void AddDate(string paramName, string content = "")
        {
            // ----- Add a new date parameter. Stores NULL when content is "".
            CommandParameter newEntry = new CommandParameter();

            newEntry.ParameterName = paramName;
            newEntry.SourceData = content;
            newEntry.SourceType = CommandDataType.AsText;
            newEntry.TargetType = CommandDataType.AsDate;
            this.Parameters.Add(paramName, newEntry);
        }

        public void AddDate(string paramName, DateTime content)
        {
            // ----- Add a new date parameter.
            CommandParameter newEntry = new CommandParameter();

            newEntry.ParameterName = paramName;
            newEntry.SourceData = content;
            newEntry.SourceType = CommandDataType.AsDate;
            newEntry.TargetType = CommandDataType.AsDate;
            this.Parameters.Add(paramName, newEntry);
        }

        public void AddInteger(string paramName, string content = "")
        {
            // ----- Add a new integer parameter. Stores NULL when content is "".
            CommandParameter newEntry = new CommandParameter();

            newEntry.ParameterName = paramName;
            newEntry.SourceData = content;
            newEntry.SourceType = CommandDataType.AsText;
            newEntry.TargetType = CommandDataType.AsInteger;
            this.Parameters.Add(paramName, newEntry);
        }

        public void AddInteger(string paramName, int content)
        {
            // ----- Add a new integer parameter.
            CommandParameter newEntry = new CommandParameter();

            newEntry.ParameterName = paramName;
            newEntry.SourceData = content;
            newEntry.SourceType = CommandDataType.AsInteger;
            newEntry.TargetType = CommandDataType.AsInteger;
            this.Parameters.Add(paramName, newEntry);
        }

        public void AddID(string paramName, long content)
        {
            // ----- Same as AddLong, but a value of -1 is treated as NULL.
            if (content == -1L)
                this.AddLong(paramName);
            else
                this.AddLong(paramName, content);
        }

        public void AddLong(string paramName, string content = "")
        {
            // ----- Add a new long integer parameter. Stores NULL when content is "".
            CommandParameter newEntry = new CommandParameter();

            newEntry.ParameterName = paramName;
            newEntry.SourceData = content;
            newEntry.SourceType = CommandDataType.AsText;
            newEntry.TargetType = CommandDataType.AsLong;
            this.Parameters.Add(paramName, newEntry);
        }

        public void AddLong(string paramName, long content)
        {
            // ----- Add a new long integer parameter.
            CommandParameter newEntry = new CommandParameter();

            newEntry.ParameterName = paramName;
            newEntry.SourceData = content;
            newEntry.SourceType = CommandDataType.AsLong;
            newEntry.TargetType = CommandDataType.AsLong;
            this.Parameters.Add(paramName, newEntry);
        }

        public void AddDecimal(string paramName, string content = "")
        {
            // ----- Add a new long integer parameter. Stores NULL when content is "".
            CommandParameter newEntry = new CommandParameter();

            newEntry.ParameterName = paramName;
            newEntry.SourceData = content;
            newEntry.SourceType = CommandDataType.AsText;
            newEntry.TargetType = CommandDataType.AsDecimal;
            this.Parameters.Add(paramName, newEntry);
        }

        public void AddDecimal(string paramName, decimal content)
        {
            // ----- Add a new long integer parameter.
            CommandParameter newEntry = new CommandParameter();

            newEntry.ParameterName = paramName;
            newEntry.SourceData = content;
            newEntry.SourceType = CommandDataType.AsDecimal;
            newEntry.TargetType = CommandDataType.AsDecimal;
            this.Parameters.Add(paramName, newEntry);
        }

        public void AddText(string paramName, string content = "")
        {
            // ----- Add a new string parameter. Stores NULL when content is "".
            CommandParameter newEntry = new CommandParameter();

            newEntry.ParameterName = paramName;
            newEntry.SourceData = content;
            newEntry.SourceType = CommandDataType.AsText;
            newEntry.TargetType = CommandDataType.AsText;
            this.Parameters.Add(paramName, newEntry);
        }
    }
}
