﻿// ----- The Library Project
//       Start-to-Finish Visual C# 2015
//       Copyright (c) 2016 by Tim Patrick

using System;
using System.Collections.Generic;
using System.Data;
using System.Data.SqlClient;
using System.Drawing;
using System.Reflection;
using System.Windows.Forms;

namespace Library
{
    internal static class General
    {
        // ----- Public constants.
        public const string ProgramTitle = "The Library Project";
        public const string NotAuthorizedMessage = "You are not authorized to perform this task.";
        public const int UseDBVersion = 1;

        // ----- Constants for the MatchingImages image list.
        public const int MatchPresent = 0;
        public const int MatchNone = 1;

        public enum LookupMethods: int
        {
            ByTitle = 1,
            ByAuthor = 2,
            BySubject = 3,
            ByKeywordAny = 4,
            ByKeywordAll = 5,
            ByPublisher = 6,
            BySeries = 7,
            BySeriesID = 8,
            ByBarcode = 9,
            ByDatabaseID = 10,
            ByAuthorID = 11,
            BySubjectID = 12,
            ByPublisherID = 13
        }

        // ----- Security values.
        public enum LibrarySecurity: int
        {
            ManageAuthors = 1,
            ManageAuthorTypes = 2,
            ManageCopyStatus = 3,
            ManageMediaTypes = 4,
            ManageSeries = 5,
            ManageGroups = 6,
            ManageItems = 7,
            ManagePatrons = 8,
            ManagePublishers = 9,
            ManageValues = 10,
            ManageUsers = 11,
            ProcessFees = 12,
            ManageLocations = 13,
            CheckOutItems = 14,
            CheckInItems = 15,
            AdminFeatures = 16,
            DailyProcessing = 17,
            RunReports = 18,
            PatronOverride = 19,
            ManageBarcodeTemplates = 20,
            ManageHolidays = 21,
            ManagePatronGroups = 22,
            ViewAdminPatronMessages = 23
        }
        public const LibrarySecurity MaxLibrarySecurity = LibrarySecurity.ViewAdminPatronMessages;

        public static string CenterText(string origText, int textWidth)
        {
            // ----- Center a piece of text in a field width. If the text is too
            //       wide, truncate it.
            string resultText;

            resultText = origText.Trim();
            if (resultText.Length >= textWidth)
            {
                // ----- Truncate as needed.
                return origText.Substring(0, textWidth).Trim();
            }
            else
            {
                // ----- Start with extra spaces.
                return new string(' ',
                    (textWidth - origText.Length) / 2) + resultText;
            }
        }

        public static bool CheckDateTime(string content)
        {
            // ----- Return True if a string contains a valid DateTime.
            DateTime result;

            return DateTime.TryParse(content, out result);
        }

        public static bool CheckDecimal(string content)
        {
            // ----- Return True if a string contains a valid decimal.
            decimal result;

            return decimal.TryParse(content, out result);
        }

        public static bool CheckInt(string content)
        {
            // ----- Return True if a string contains a valid int.
            int result;

            return int.TryParse(content, out result);
        }

        public static bool CheckLong(string content)
        {
            // ----- Return True if a string contains a valid long.
            long result;

            return long.TryParse(content, out result);
        }

        public static int CountSubStr(string mainText, string subText)
        {
            // ----- Return a count of the number of times that a subText occurs in
            //       a string (mainText).
            int totalTimes;
            int startPos;
            int foundPos;

            if ((mainText == null) || (mainText.Length == 0))
                return 0;
            if ((subText == null) || (subText.Length == 0))
                return 0;
            totalTimes = 0;
            startPos = 0;

            // ----- Keep searching until we don't find it no more!
            do
            {
                // ----- Search for the subText.
                foundPos = mainText.IndexOf(subText, startPos);
                if (foundPos == -1)
                    break;
                totalTimes += 1;

                // ----- Move to just after the occurrence.
                startPos = foundPos + subText.Length;
            } while (true);

            // ----- Return the count.
            return totalTimes;
        }

        public static string DigitsOnly(string origText)
        {
            // ----- Return only the digits found in a string.
            string destText;

            // ----- Examine each character.
            destText = "";
            for (int counter = 0; counter < origText.Length; counter++)
            {
                if (char.IsDigit(origText[counter]) == true)
                    destText += origText[counter];
            }
            return destText;
        }

        public static void GeneralError(string routineName, Exception theError)
        {
            // ----- Report an error to the user.
            MessageBox.Show("The following error occurred at location '" + routineName + "':" +
                Environment.NewLine + Environment.NewLine + theError.Message,
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
        }

        public static bool GetBoolean(string content)
        {
            // ----- Extract a boolean value from a string. Return False if
            //       the string contains no legitimate value.
            int holdInt;
            bool holdBool;

            if (int.TryParse(content, out holdInt) == true)
                return Convert.ToBoolean(holdInt);
            else if (bool.TryParse(content, out holdBool) == true)
                return holdBool;
            else
                return false;
        }

        public static string GetSubStr(string origString, string delim, int whichField)
        {
            // ----- Extracts a delimited string from another larger string.
            string[] stringParts = null;

            // ----- Handle some errors.
            if (whichField < 0)
                return "";
            if ((origString == null) || (origString.Length == 0))
                return "";
            if ((delim == null) || (delim.Length == 0))
                return "";

            // ----- Break the string up into delimited parts.
            stringParts = origString.Split(new[] {delim}, StringSplitOptions.None);

            // ----- See if the part we want exists and return it.
            if (whichField > stringParts.Length)
                return "";
            else
                return stringParts[whichField - 1];
        }

        public static string LeftAndRightText(string origLeft, string origRight, int textWidth)
        {
            // ----- Build a string of the indicated width that has both left-justified
            //       and right-justified text.
            string leftText;
            string rightText;

            leftText = origLeft.Trim();
            rightText = origRight.Trim();
            if ((leftText + rightText).Length >= textWidth)
                return (leftText + rightText).Substring(0, textWidth).Trim();
            else
                return leftText + new string(' ', textWidth -
                    (leftText + rightText).Length) + rightText;
        }
    }
}
