﻿// ----- The Library Project
//       Start-to-Finish Visual C# 2015
//       Copyright (c) 2016 by Tim Patrick

using System;
using System.Data;
using System.Windows.Forms;
using static Library.General;

namespace Library
{
    public partial class ItemCopy : Form
    {
        private long ActiveItem;
        private long ActiveCopy;

        public ItemCopy()
        {
            InitializeComponent();
            this.RecordBarcode.Enter += this.RecordBarcode_Enter;
            this.RecordBarcode.KeyPress += this.RecordBarcode_KeyPress;
            this.RecordBarcode.Leave += this.RecordBarcode_Leave;
        }

        public long AddCopy(long itemID, string useItemName)
        {
            // ----- A default location must be defined.
            if (ConfirmDefaultLocation() == false)
                return -1L;

            // ----- Prompt the user to add a new item copy.
            ActiveItem = itemID;
            ActiveCopy = -1L;
            ItemName.Text = useItemName;
            this.ShowDialog();
            if (this.DialogResult == DialogResult.OK)
                return ActiveCopy;
            else
                return -1L;
        }

        public long EditCopy(long itemID, long copyID, string useItemName)
        {
            // ----- A default location must be defined.
            if (ConfirmDefaultLocation() == false)
                return -1L;

            // ----- Prompt the user to edit an existing item copy.
            ActiveItem = itemID;
            ActiveCopy = copyID;
            ItemName.Text = useItemName;
            this.ShowDialog();
            if (this.DialogResult == DialogResult.OK)
                return ActiveCopy;
            else
                return -1L;
        }

        private bool SaveFormData()
        {
            // ----- Save the record to the database.
            string sqlText;
            DataCommand sqlRun;
            long newID;

            this.Cursor = Cursors.WaitCursor;

            // ----- Different code for new and existing copies.
            if (ActiveCopy == -1L)
            {
                // ----- Add a new record.
                sqlText = "INSERT INTO ItemCopy (ItemID, CopyNumber, Description, " +
                    "Barcode, Available, Reference, Missing, Acquired, Cost, Status, " +
                    "Condition, Location) OUTPUT INSERTED.ID VALUES (@NewItem, @NewCopy, " +
                    "@NewDesc, @NewBarcode, @NewAvailable, @NewReference, @NewMissing, " +
                    "@NewAcquired, @NewCost, @NewStatus, @NewCondition, @NewLocation)";
                sqlRun = new DataCommand(sqlText);
                sqlRun.AddLong("NewItem", ActiveItem);
                sqlRun.AddText("NewCopy", RecordCopy.Text);
                sqlRun.AddText("NewDesc", RecordDescription.Text.Trim());
                sqlRun.AddText("NewBarcode", RecordBarcode.Text.Trim());
                sqlRun.AddBool("NewAvailable", RecordAvailable.Checked);
                sqlRun.AddBool("NewReference", RecordReference.Checked);
                sqlRun.AddBool("NewMissing", RecordMissing.Checked);
                sqlRun.AddDate("NewAcquired", RecordAcquired.Text);
                sqlRun.AddDecimal("NewCost", RecordCost.Text);
                sqlRun.AddLong("NewStatus", DBComboParam(RecordGeneralStatus));
                sqlRun.AddText("NewCondition", RecordCondition.Text.Trim());
                sqlRun.AddLong("NewLocation", DBComboParam(RecordLocation));
                try
                {
                    newID = DBGetLong(ExecuteSQLReturn(sqlRun));
                }
                catch (Exception ex)
                {
                    GeneralError("ItemCopy.SaveFormData.Add", ex);
                    return false;
                }
                finally
                {
                    this.Cursor = Cursors.Default;
                }
                ActiveCopy = newID;
            }
            else
            {
                // ----- Update an existing record.
                sqlText = "UPDATE ItemCopy SET CopyNumber = @NewCopy, " +
                    "Description = @NewDesc, Barcode = @NewBarcode, " +
                    "Available = @NewAvailable, Reference = @NewReference, " +
                    "Missing = @NewMissing, Acquired = @NewAcquired, " +
                    "Cost = @NewCost, Status = @NewStatus, " +
                    "Condition = @NewCondition, Location = @NewLocation " +
                    "WHERE ID = @RecordID";
                sqlRun = new DataCommand(sqlText);
                sqlRun.AddText("NewCopy", RecordCopy.Text);
                sqlRun.AddText("NewDesc", RecordDescription.Text.Trim());
                sqlRun.AddText("NewBarcode", RecordBarcode.Text.Trim());
                sqlRun.AddBool("NewAvailable", RecordAvailable.Checked);
                sqlRun.AddBool("NewReference", RecordReference.Checked);
                sqlRun.AddBool("NewMissing", RecordMissing.Checked);
                sqlRun.AddDate("NewAcquired", RecordAcquired.Text);
                sqlRun.AddDecimal("NewCost", RecordCost.Text);
                sqlRun.AddLong("NewStatus", DBComboParam(RecordGeneralStatus));
                sqlRun.AddText("NewCondition", RecordCondition.Text.Trim());
                sqlRun.AddLong("NewLocation", DBComboParam(RecordLocation));
                sqlRun.AddLong("RecordID", ActiveCopy);
                try
                {
                    ExecuteSQL(sqlRun);
                }
                catch (Exception ex)
                {
                    GeneralError("ItemCopy.SaveFormData.Update", ex);
                    return false;
                }
                finally
                {
                    this.Cursor = Cursors.Default;
                }
            }

            // ----- Success.
            return true;
        }

        private bool ValidateFormData()
        {
            // ----- Confirm the fields on the form before saving.
            string sqlText;
            DataCommand sqlRun;

            // ----- Copy number is required, and it must be numeric.
            if (RecordCopy.Text.Trim().Length == 0)
            {
                MessageBox.Show("Copy Number is required.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                RecordCopy.Focus();
                return false;
            }
            if (DigitsOnly(RecordCopy.Text) != RecordCopy.Text.Trim())
            {
                MessageBox.Show("Copy Number must be numeric.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                RecordCopy.Focus();
                return false;
            }

            // ----- Make sure the copy number is unique.
            sqlText = "SELECT COUNT(*) FROM ItemCopy " +
                "WHERE CopyNumber = @WhichCopy AND ItemID = @WhichItem";
            if (ActiveCopy != -1L)
                sqlText += " AND ID <> @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddInteger("WhichCopy", RecordCopy.Text);
            sqlRun.AddLong("WhichItem", ActiveItem);
            if (ActiveCopy != -1L)
                sqlRun.AddLong("RecordID", ActiveCopy);
            try
            {
                if (DBGetInteger(ExecuteSQLReturn(sqlRun)) > 0)
                {
                    // ----- Duplicate entry.
                    MessageBox.Show("A record already exists with the copy number '" +
                        RecordCopy.Text.Trim() + "'.", ProgramTitle,
                        MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    RecordCopy.Focus();
                    return false;
                }
            }
            catch (Exception ex)
            {
                GeneralError("ItemCopy.ValidateFormData.Copy", ex);
                return false;
            }

            // ----- Barcode, if supplied, must be numeric.
            if (RecordBarcode.Text.Trim().Length != 0)
            {
                if (DigitsOnly(RecordBarcode.Text) != RecordBarcode.Text.Trim())
                {
                    MessageBox.Show("Barcode, if supplied, must be numeric.",
                        ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    RecordBarcode.Focus();
                    return false;
                }
            }

            // ----- Make sure the barcode is not already in use.
            if (RecordBarcode.Text.Trim().Length != 0)
            {
                sqlText = "SELECT COUNT(*) FROM ItemCopy " +
                    "WHERE UPPER(Barcode) = @TestBarcode";
                if (ActiveCopy != -1L)
                    sqlText += " AND ID <> @RecordID";
                sqlRun = new DataCommand(sqlText);
                sqlRun.AddText("TestBarcode", RecordBarcode.Text.Trim().ToUpper());
                sqlRun.AddLong("RecordID", ActiveCopy);
                try
                {
                    if (DBGetInteger(ExecuteSQLReturn(sqlRun)) > 0)
                    {
                        // ----- Duplicate entry.
                        MessageBox.Show("A record already exists with the barcode'" +
                            RecordBarcode.Text.Trim() + "'.", ProgramTitle,
                            MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                        RecordBarcode.Focus();
                        return false;
                    }
                }
                catch (Exception ex)
                {
                    GeneralError("ItemCopy.ValidateFormData.Barcode", ex);
                    return false;
                }
            }

            // ----- Date acquired, if supplied, must be a valid date.
            if (RecordAcquired.Text.Trim().Length != 0)
            {
                if (CheckDateTime(RecordAcquired.Text) == false)
                {
                    MessageBox.Show("Date Acquired, if supplied, must be a valid date.",
                        ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    RecordAcquired.Focus();
                    return false;
                }
            }

            // ----- Item cost, if supplied, must be numeric.
            if (RecordCost.Text.Trim().Length != 0)
            {
                if (CheckDecimal(RecordCost.Text) == false)
                {
                    MessageBox.Show("Item Cost, if supplied, must be numeric.",
                        ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    RecordCost.Focus();
                    return false;
                }
            }

            // ----- General status is required.
            if ((long)(ListItemData)RecordGeneralStatus.SelectedItem == -1L)
            {
                MessageBox.Show("General Status is required.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                RecordGeneralStatus.Focus();
                return false;
            }

            // ----- Location is required.
            if ((long)(ListItemData)RecordLocation.SelectedItem == -1L)
            {
                MessageBox.Show("Location is required.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                RecordLocation.Focus();
                return false;
            }

            // ----- Success.
            return true;
        }

        private void ActOK_Click(object sender, EventArgs e)
        {
            // ----- The user wants to save the record.
            if (ValidateFormData() == false)
                return;
            if (SaveFormData() == false)
                return;
            this.DialogResult = DialogResult.OK;
        }

        private void ItemCopy_Load(object sender, EventArgs e)
        {
            // ----- Prepare the form.
            string sqlText;
            DataCommand sqlRun;
            DataTable dbInfo;
            DataRow oneRow;

            // ----- Load in the list of status codes.
            RecordGeneralStatus.Items.Add(new ListItemData("<Not Selected>", -1L));
            RecordGeneralStatus.SelectedIndex = 0;
            sqlText = "SELECT * FROM CodeCopyStatus ORDER BY FullName";
            try
            {
                dbInfo = CreateDataTable(sqlText);
            }
            catch (Exception ex)
            {
                GeneralError("ItemCopy.ItemCopy_Load.Status", ex);
                return;
            }
            foreach (DataRow dbRow in dbInfo.Rows)
            {
                RecordGeneralStatus.Items.Add(new ListItemData(
                    DBGetText(dbRow["FullName"]), DBGetLong(dbRow["ID"])));
            }
            dbInfo.Dispose();
            dbInfo = null;

            // ----- Load in the list of locations.
            RecordLocation.Items.Add(new ListItemData("<Not Selected>", -1L));
            RecordLocation.SelectedIndex = 0;
            sqlText = "SELECT * FROM CodeLocation ORDER BY FullName";
            try
            {
                dbInfo = CreateDataTable(sqlText);
            }
            catch (Exception ex)
            {
                GeneralError("ItemCopy.ItemCopy_Load.Location", ex);
                return;
            }
            foreach (DataRow dbRow in dbInfo.Rows)
            {
                RecordLocation.Items.Add(new ListItemData(
                    DBGetText(dbRow["FullName"]), DBGetLong(dbRow["ID"])));
                if (DBGetLong(dbRow["ID"]) == DefaultItemLocation)
                    RecordLocation.SelectedIndex = RecordLocation.Items.Count - 1;
            }
            dbInfo.Dispose();
            dbInfo = null;

            // ----- For new items, that's it.
            if (ActiveCopy == -1L)
                return;

            // ----- Load in the existing item.
            sqlText = "SELECT * FROM ItemCopy WHERE ID = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", ActiveCopy);
            try
            {
                dbInfo = CreateDataTable(sqlRun);
            }
            catch (Exception ex)
            {
                GeneralError("ItemCopy.ItemCopy_Load.Item", ex);
                return;
            }
            oneRow = dbInfo.Rows[0];
            RecordCopy.Text = DBGetText(oneRow["CopyNumber"]);
            RecordDescription.Text = DBGetText(oneRow["Description"]);
            RecordBarcode.Text = DBGetText(oneRow["Barcode"]);
            RecordAvailable.Checked = DBGetBool(oneRow["Available"]);
            RecordReference.Checked = DBGetBool(oneRow["Reference"]);
            RecordMissing.Checked = DBGetBool(oneRow["Missing"]);
            if (Convert.IsDBNull(oneRow["Acquired"]) == false)
                RecordAcquired.Text = DBGetDate(oneRow["Acquired"]).ToString("M/d/yyyy");
            if (Convert.IsDBNull(oneRow["Cost"]) == false)
                RecordCost.Text = DBGetDecimal(oneRow["Cost"]).ToString("0.00");
            RecordGeneralStatus.SelectedIndex = RecordGeneralStatus.Items.IndexOf(
                    DBGetLong(oneRow["Status"]));
            if (RecordGeneralStatus.SelectedIndex == -1)
                RecordGeneralStatus.SelectedIndex = RecordGeneralStatus.Items.IndexOf(-1L);
            RecordCondition.Text = DBGetText(oneRow["Condition"]);
            if (Convert.IsDBNull(oneRow["Location"]) == false)
                RecordLocation.SelectedIndex = RecordLocation.Items.IndexOf(
                    DBGetLong(oneRow["Location"]));
            dbInfo.Dispose();
            dbInfo = null;

            // ----- Display the disposition of the item.
            RecordStatus.Text = GetCopyDisposition(ActiveCopy);
        }

        private void RecordCopy_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordCopy.SelectAll();
        }

        private void RecordAcquired_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordAcquired.SelectAll();
        }

        private void RecordCost_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordCost.SelectAll();
        }

        private void RecordCondition_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordCondition.SelectAll();
        }

        private void StatusDetails_Click(object sender, EventArgs e)
        {
            // ----- Show details about the disposition of the item.
            string sqlText;
            DataCommand sqlRun;
            DataTable dbInfo;
            string response = "";
            decimal amountDue;
            DataRow dbRow;

            // ----- Do a quick check for new items.
            if (ActiveCopy == -1L)
            {
                MessageBox.Show("This is a new item.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Information);
                return;
            }
            this.Cursor = Cursors.WaitCursor;

            // ----- Retrieve information about the book.
            sqlText = "SELECT IC.Reference, IC.Missing, PC.Missing AS PatronMissing, " +
                "PC.DueDate, PC.Patron, PT.LastName, PT.FirstName, PC.Fine, PC.Paid, " +
                "PC.CheckOut FROM ItemCopy AS IC LEFT JOIN PatronCopy AS PC " +
                "ON IC.ID = PC.ItemCopy LEFT JOIN Patron AS PT " +
                "ON PC.Patron = PT.ID WHERE IC.ID = @RecordID " +
                "AND PC.Returned = 0";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", ActiveCopy);
            try
            {
                dbInfo = CreateDataTable(sqlRun);
            }
            catch (Exception ex)
            {
                this.Cursor = Cursors.Default;
                GeneralError("ItemCopy.StatusDetails_Click.Retrieve", ex);
                return;
            }
            if (dbInfo.Rows.Count == 0)
            {
                // ----- Perhaps the database, due to the JOIN clause, could not return
                //       the base record of a checked-in book.
                dbInfo.Dispose();
                sqlText = "SELECT Reference, Missing, NULL AS PatronMissing, " +
                    "NULL AS DueDate, NULL AS Patron, NULL AS LastName, " +
                    "NULL AS FirstName, NULL AS Fine, NULL AS Paid, " +
                    "NULL AS CheckOut FROM ItemCopy WHERE ID = @RecordID";
                sqlRun = new DataCommand(sqlText);
                sqlRun.AddLong("RecordID", ActiveCopy);
                try
                {
                    dbInfo = CreateDataTable(sqlRun);
                }
                catch (Exception ex)
                {
                    this.Cursor = Cursors.Default;
                    GeneralError("ItemCopy.StatusDetails_Click.Retrieve2", ex);
                    return;
                }
                if (dbInfo.Rows.Count == 0)
                {
                    // ----- What happened to the record?
                    response = "An error occurred while trying to determine the " +
                        "disposition of the item.";
                    goto ShowResponse;
                }
            }
            dbRow = dbInfo.Rows[0];

            // ----- We found the record. Show its details.
            if (Convert.IsDBNull(dbRow["Patron"]) == false)
            {
                // ----- This item is checked out.
                if ((DBGetBool(dbRow["Missing"]) == true) |
                        (DBGetBool(dbRow["PatronMissing"]) == true))
                    response = "This item was reported missing by patron ";
                else if (DBGetDate(dbRow["DueDate"]) < DateTime.Today)
                    response = "This item is overdue. It was last due on " +
                        DBGetDate(dbRow["DueDate"]).ToString("MMMM d, yyyy") + " by patron ";
                else
                    response = "This item is currently checked out. It is due on " +
                        DBGetDate(dbRow["DueDate"]).ToString("MMMM d, yyyy") + " by patron ";
                response += DBGetText(dbRow["FirstName"]) + " " +
                    DBGetText(dbRow["LastName"]) + ". It was originally checked out on " +
                    DBGetDate(dbRow["CheckOut"]).ToString("MMMM d, yyyy") + ".";
                amountDue = 0M;
                if (Convert.IsDBNull(dbRow["Fine"]) == false)
                    amountDue += DBGetDecimal(dbRow["Fine"]);
                if (Convert.IsDBNull(dbRow["Paid"]) == false)
                    amountDue -= DBGetDecimal(dbRow["Paid"]);
                if (amountDue > 0M)
                    response += " The patron owes " + amountDue.ToString("$0.00") +
                        " in fines on this item.";
            }
            else
            {
                // ----- This book is currently checked in.
                if (DBGetBool(dbRow["Missing"]) == true)
                    response = "This item has been reported as missing.";
                else if (DBGetBool(dbRow["Reference"]) == true)
                    response = "This item is in the reference collection.";
                else
                    response = "This item is currently checked in.";
            }
            dbInfo.Dispose();
            dbInfo = null;

    ShowResponse:
            // ----- Display the disposition.
            this.Cursor = Cursors.Default;
            MessageBox.Show(response, ProgramTitle,
                MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        private void RecordBarcode_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordBarcode.SelectAll();

            // ----- Don't allow Enter to close the form.
            this.AcceptButton = null;
        }

        private void RecordBarcode_Leave(object sender, EventArgs e)
        {
            // ----- Allow Enter to close the form again.
            this.AcceptButton = ActOK;
        }

        private void RecordBarcode_KeyPress(object sender, KeyPressEventArgs e)
        {
            // ----- Ignore the enter key.
            if (e.KeyChar == (char)Keys.Return)
                e.Handled = true;
        }
    }
}
