﻿// ----- The Library Project: License Generator
//       Start-to-Finish Visual Basic 2015
//       Copyright (c) 2015 by Tim Patrick

using System;
using System.Windows;

namespace LibraryLicensing
{
    internal static class General
    {
        public const string ProgramTitle = "Library License Generator";
        public const string ApplicationProductName = "Library Project";

        // ----- The filenames for the two keys are static.
        public const string PublicKeyFilename = "LibraryPublicKey.xml";
        public const string PrivateKeyFilename = "LibraryPrivateKey.xml";

        // ----- Indicates the status of the directory containing the
        //       public and private keys.
        public enum KeyFolderStatus
        {
            NoFolder,
            InvalidFolder,
            MissingBothFiles,
            MissingPrivateFile,
            MissingPublicFile,
            ValidFolder
        }

        public static bool AttemptCreateFolder(string whichFolder)
        {
            // ----- Ask the user to create a folder, and create it.
            //       Return True on success.
            if (whichFolder.Trim().Length == 0)
                return false;
            if (System.IO.Directory.Exists(whichFolder) == true)
                return true;

            // ----- Prompt the user.
            if (MessageBox.Show("The directory" + Environment.NewLine + Environment.NewLine +
                    "\t" + whichFolder + Environment.NewLine + Environment.NewLine +
                    "does not exist. Do you wish to create it?",
                    ProgramTitle, MessageBoxButton.YesNo, MessageBoxImage.Question) !=
                    MessageBoxResult.Yes)
                return false;

            // ----- Create the directory.
            try
            {
                System.IO.Directory.CreateDirectory(whichFolder);
            }
            catch (Exception ex)
            {
                MessageBox.Show("Could not create the directory due to error: " + ex.Message,
                    ProgramTitle, MessageBoxButton.OK, MessageBoxImage.Exclamation);
                return false;
            }

            // ----- Success.
            return true;
        }

        public static KeyFolderStatus GetFolderStatus(string whichFolder)
        {
            // ----- Given a folder, return the status.
            string usePath;
            bool privateExists;
            bool publicExists;

            // ----- Was anything even supplied?
            if (whichFolder.Trim().Length == 0)
                return KeyFolderStatus.NoFolder;

            // ----- Is this a valid directory.
            if (System.IO.Directory.Exists(whichFolder) == false)
                return KeyFolderStatus.InvalidFolder;

            // ----- Check for the files.
            usePath = System.IO.Path.Combine(whichFolder, PublicKeyFilename);
            publicExists = System.IO.File.Exists(usePath);
            usePath = System.IO.Path.Combine(whichFolder, PrivateKeyFilename);
            privateExists = System.IO.File.Exists(usePath);

            // ----- Return the file status.
            if ((publicExists == false) & (privateExists == false))
                return KeyFolderStatus.MissingBothFiles;
            else if (publicExists == false)
                return KeyFolderStatus.MissingPublicFile;
            else if (privateExists == false)
                return KeyFolderStatus.MissingPrivateFile;
            else
                return KeyFolderStatus.ValidFolder;
        }

        public static string GetFolderStatusText(KeyFolderStatus whichStatus)
        {
            // ----- Return a description of the current status.
            switch (whichStatus)
            {
                case KeyFolderStatus.NoFolder:
                    return "No directory specified.";
                case KeyFolderStatus.InvalidFolder:
                    return "Invalid directory specified.";
                case KeyFolderStatus.MissingPublicFile:
                    return "The directory lacks a public key file.";
                case KeyFolderStatus.MissingPrivateFile:
                    return "The directory lacks a private key file.";
                case KeyFolderStatus.MissingBothFiles:
                    return "The directory lacks the two key files.";
                case KeyFolderStatus.ValidFolder:
                    return "The keys are ready to use.";
                default:
                    return "Unknown.";
            }
        }
    }
}
