﻿// ----- The Library Project
//       Start-to-Finish Visual C# 2015
//       Copyright (c) 2016 by Tim Patrick

using System;
using System.Data;
using System.Drawing;
using System.Windows.Forms;
using static Library.General;

namespace Library
{
    public partial class MainForm : Form
    {
        private long ActiveCheckOutPatron = -1L;

        private const int StatusImageBad = 0;
        private const int StatusImageGood = 1;

        public MainForm()
        {
            InitializeComponent();
        }

        protected override void WndProc(ref Message m)
        {
            // ----- Watch for messages from other copies of the application.
            if (m.Msg == NativeMethods.WM_LibraryNotify)
            {
                // ----- Focus on this form.
                if (this.WindowState == FormWindowState.Minimized)
                    this.WindowState = FormWindowState.Normal;
                this.Activate();
            }
            base.WndProc(ref m);
        }

        private void ActHelpAbout_Click(object sender, EventArgs e)
        {
            // ----- Show the About form.
            (new AboutProgram()).ShowDialog();
        }

        private void ActLibraryItem_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            // ----- Library Item mode.
            TaskLibraryItem();
        }

        private void PicLibraryItem_Click(object sender, EventArgs e)
        {
            // ----- Library Item mode.
            TaskLibraryItem();
        }

        private void ActPatronRecord_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            // ----- Patron Record mode.
            TaskPatronRecord();
        }

        private void PicPatronRecord_Click(object sender, EventArgs e)
        {
            // ----- Patron Record mode.
            TaskPatronRecord();
        }

        private void ActHelp_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            // ----- Help mode.
            TaskHelp();
        }

        private void PicHelp_Click(object sender, EventArgs e)
        {
            // ----- Help mode.
            TaskHelp();
        }

        private void ActCheckOut_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            // ----- Check Out mode.
            TaskCheckOut();
        }

        private void PicCheckOut_Click(object sender, EventArgs e)
        {
            // ----- Check Out mode.
            TaskCheckOut();
        }

        private void ActCheckIn_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            // ----- Check In mode.
            TaskCheckIn();
        }

        private void PicCheckIn_Click(object sender, EventArgs e)
        {
            // ----- Check In mode.
            TaskCheckIn();
        }

        private void ActAdmin_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            // ----- Administration mode.
            TaskAdmin();
        }

        private void PicAdmin_Click(object sender, EventArgs e)
        {
            // ----- Administration mode.
            TaskAdmin();
        }

        private void ActProcess_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            // ----- Daily Processing mode.
            TaskProcess();
        }

        private void PicProcess_Click(object sender, EventArgs e)
        {
            // ----- Daily Processing mode.
            TaskProcess();
        }

        private void ActReports_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            // ----- Print Reports mode.
            TaskReports();
        }

        private void PicReports_Click(object sender, EventArgs e)
        {
            // ----- Print Reports mode.
            TaskReports();
        }

        private void TaskLibraryItem()
        {
            // ----- Library item task.

            // ----- Update the display.
            AllPanelsInvisible();
            PanelLibraryItem.Visible = true;
            ActLibraryItem.BackColor = SystemColors.Control;
            LabelSelected.Location = new Point(LabelSelected.Left, PicLibraryItem.Top);
            this.AcceptButton = ActSearch;

            // ----- Get ready for a new search.
            ActSearchClear.PerformClick();
            if (ActSearchLimits.Top == LabelMoreLimitsTop.Top)
                ActSearchLimits.PerformClick();
            if (SearchText.CanFocus)
                SearchText.Focus();
        }

        private void TaskPatronRecord()
        {
            // ----- Patron Record mode.

            // ----- Update the display.
            AllPanelsInvisible();
            PanelPatronRecord.Visible = true;
            ActPatronRecord.BackColor = SystemColors.Control;
            LabelSelected.Location = new Point(LabelSelected.Left, PicPatronRecord.Top);
            this.AcceptButton = ActAccessPatron;
        }

        private void TaskHelp()
        {
            // ----- Help mode.

            // ----- Update the display.
            AllPanelsInvisible();
            PanelHelp.Visible = true;
            ActHelp.BackColor = SystemColors.Control;
            LabelSelected.Location = new Point(LabelSelected.Left, PicHelp.Top);
            this.AcceptButton = null;
        }

        private void TaskCheckOut()
        {
            // ----- Check Out mode.
            bool userCanCheckOut;

            // ----- See if patrons can check out items by themselves.
            userCanCheckOut = GetBoolean(GetSystemValue("PatronCheckOut"));

            // ----- Update the display.
            AllPanelsInvisible();
            if (userCanCheckOut | SecurityProfile[(int)LibrarySecurity.CheckOutItems])
                PanelCheckOut.Visible = true;
            ActCheckOut.BackColor = SystemColors.Control;
            LabelSelected.Location = new Point(LabelSelected.Left, PicCheckOut.Top);
            this.AcceptButton = ActDoCheckOut;
        }

        private void TaskCheckIn()
        {
            // ----- Check In mode.

            // ----- Update the display.
            AllPanelsInvisible();
            if (SecurityProfile[(int)LibrarySecurity.CheckInItems])
                PanelCheckIn.Visible = true;
            ActCheckIn.BackColor = SystemColors.Control;
            LabelSelected.Location = new Point(LabelSelected.Left, PicCheckIn.Top);
            this.AcceptButton = ActDoCheckIn;

            // ----- Reset the input fields.
            CheckInDate.Value = DateTime.Today;
            CheckInDate.MaxDate = DateTime.Today;
            CheckInDate.MinDate = DateTime.Today.AddDays(-6);
            CheckInDay.Text = "Today";
            CheckInDay.BackColor = SystemColors.Control;
            CheckInDay.ForeColor = SystemColors.ControlText;
            CheckInBarcode.Text = "";
            LabelCheckedIn.Visible = false;
            CheckedInDetail.Visible = false;
            CheckedInDetail.Text = "No information available.";
            CheckInBarcode.Focus();
        }

        private void TaskAdmin()
        {
            // ----- Administration mode.

            // ----- Update the display.
            AllPanelsInvisible();

            if (SecurityProfile[(int)LibrarySecurity.AdminFeatures])
                PanelAdmin.Visible = true;

            ActAdmin.BackColor = SystemColors.Control;
            LabelSelected.Location = new Point(LabelSelected.Left, PicAdmin.Top);
            this.AcceptButton = null;
        }

        private void TaskProcess()
        {
            // ----- Daily Processing mode.
            string sqlText;
            DataTable dbInfo;

            // ----- Update the display.
            AllPanelsInvisible();
            if (SecurityProfile[(int)LibrarySecurity.DailyProcessing])
                PanelProcess.Visible = true;
            ActProcess.BackColor = SystemColors.Control;
            LabelSelected.Location = new Point(LabelSelected.Left, PicProcess.Top);
            this.AcceptButton = ActDoProcess;

            // ----- Refresh the list of locations.
            ProcessLocation.Items.Clear();
            ProcessLocation.Items.Add(new ListItemData("<All Locations>", -1L));
            ProcessLocation.SelectedIndex = 0;
            sqlText = "SELECT ID, FullName FROM CodeLocation ORDER BY FullName";
            try
            {
                dbInfo = CreateDataTable(sqlText);
            }
            catch (Exception ex)
            {
                GeneralError("MainForm.TaskProcess", ex);
                return;
            }
            foreach (DataRow dbRow in dbInfo.Rows)
                ProcessLocation.Items.Add(new ListItemData(
                    DBGetText(dbRow["FullName"]), DBGetLong(dbRow["ID"])));
            dbInfo.Dispose();
            dbInfo = null;
            RefreshProcessLocation();
        }

        private void TaskReports()
        {
            // ----- Print Reports mode.

            // ----- Update the display.
            AllPanelsInvisible();
            if (SecurityProfile[(int)LibrarySecurity.RunReports])
                PanelReports.Visible = true;
            ActReports.BackColor = SystemColors.Control;
            LabelSelected.Location = new Point(LabelSelected.Left, PicReports.Top);
            this.AcceptButton = ActDoReports;
        }

        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            // ----- Finish the in-process check-out if needed.
            if (ActiveCheckOutPatron != -1L)
                ActFinishCheckOut.PerformClick();

            // ----- Save any updated settings.
            Properties.Settings.Default.Save();
        }

        private void MainForm_Load(object sender, EventArgs e)
        {
            // ----- Prepare the form.
            string sqlText;
            DataTable dbInfo;

            // ----- Adjust the various panel positions.
            PanelLibraryItem.Location = new Point(TaskListBackground.Width, WelcomeBackground.Height);
            PanelPatronRecord.Location = PanelLibraryItem.Location;
            PanelHelp.Location = PanelLibraryItem.Location;
            PanelCheckOut.Location = PanelLibraryItem.Location;
            PanelCheckIn.Location = PanelLibraryItem.Location;
            PanelAdmin.Location = PanelLibraryItem.Location;
            PanelProcess.Location = PanelLibraryItem.Location;
            PanelReports.Location = PanelLibraryItem.Location;
            PanelLibraryItem.Visible = true;
            ActSearchLimits.PerformClick();

            // ----- Load in the list of search types.
            SearchType.Items.Add(new ListItemData("Lookup By Title", (long)LookupMethods.ByTitle));
            SearchType.SelectedIndex = 0;
            SearchType.Items.Add(new ListItemData("Lookup By Author", (long)LookupMethods.ByAuthor));
            SearchType.Items.Add(new ListItemData("Lookup By Subject", (long)LookupMethods.BySubject));
            SearchType.Items.Add(new ListItemData("Lookup By Keyword (Match Any)", (long)LookupMethods.ByKeywordAny));
            SearchType.Items.Add(new ListItemData("Lookup By Keyword (Match All)", (long)LookupMethods.ByKeywordAll));
            SearchType.Items.Add(new ListItemData("Lookup By Publisher", (long)LookupMethods.ByPublisher));
            SearchType.Items.Add(new ListItemData("Lookup By Series Name", (long)LookupMethods.BySeries));
            SearchType.Items.Add(new ListItemData("Lookup By Barcode", (long)LookupMethods.ByBarcode));

            // ----- Load in the media types.
            SearchMediaType.Items.Add(new ListItemData("<All Media Types>", -1L));
            SearchMediaType.SelectedIndex = 0;
            sqlText = "SELECT ID, FullName FROM CodeMediaType ORDER BY FullName";
            try
            {
                dbInfo = CreateDataTable(sqlText);
            }
            catch (Exception ex)
            {
                GeneralError("MainForm.MainForm_Load.Media", ex);
                dbInfo = null;
            }
            if (dbInfo != null)
            {
                foreach (DataRow dbRow in dbInfo.Rows)
                    SearchMediaType.Items.Add(new ListItemData(
                        DBGetText(dbRow["FullName"]), DBGetLong(dbRow["ID"])));
                dbInfo.Dispose();
            }

            // ----- Load in the locations.
            SearchLocation.Items.Add(new ListItemData("<All Locations>", -1L));
            SearchLocation.SelectedIndex = 0;
            sqlText = "SELECT ID, FullName FROM CodeLocation ORDER BY FullName";
            try
            {
                dbInfo = CreateDataTable(sqlText);
            }
            catch (Exception ex)
            {
                GeneralError("MainForm.MainForm_Load.Media", ex);
                dbInfo = null;
            }
            if (dbInfo != null)
            {
                foreach (DataRow dbRow in dbInfo.Rows)
                    SearchLocation.Items.Add(new ListItemData(
                        DBGetText(dbRow["FullName"]), DBGetLong(dbRow["ID"])));
                dbInfo.Dispose();
            }

            // ----- Prepare for a patron user.
            UpdateDisplayForUser();
        }

        private void AllPanelsInvisible()
        {
            // ----- Finish the in-process check-out if needed.
            if (ActiveCheckOutPatron != -1L)
                ActFinishCheckOut.PerformClick();

            // ----- Hide all work panels.
            PanelLibraryItem.Visible = false;
            PanelPatronRecord.Visible = false;
            PanelHelp.Visible = false;
            PanelCheckOut.Visible = false;
            PanelCheckIn.Visible = false;
            PanelAdmin.Visible = false;
            PanelProcess.Visible = false;
            PanelReports.Visible = false;

            // ----- Correctly color all action labels.
            ActLibraryItem.BackColor = Color.White;
            ActPatronRecord.BackColor = Color.White;
            ActHelp.BackColor = Color.White;
            ActCheckOut.BackColor = Color.White;
            ActCheckIn.BackColor = Color.White;
            ActAdmin.BackColor = Color.White;
            ActProcess.BackColor = Color.White;
            ActReports.BackColor = Color.White;
        }

        private void SearchText_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            SearchText.SelectAll();
        }

        private void ActSearchLimits_Click(object sender, EventArgs e)
        {
            // ----- Toggle the display of the media type and location fields.
            if (ActSearchLimits.Top == LabelMoreLimitsTop.Top)
            {
                // ----- Hide the two fields.
                LabelSearchMediaType.Visible = false;
                SearchMediaType.Visible = false;
                LabelSearchLocation.Visible = false;
                SearchLocation.Visible = false;
                ActSearchClear.Location = new Point(ActSearchClear.Left, SearchMediaType.Top);
                ActSearchLimits.Location = new Point(ActSearchLimits.Left, SearchMediaType.Top);
                ActSearch.Location = new Point(ActSearch.Left, SearchMediaType.Top);
                ActSearchLimits.Text = "Mor&e >>";
            }
            else
            {
                // ----- Show the two fields.
                ActSearchClear.Location = new Point(ActSearchClear.Left, LabelMoreLimitsTop.Top);
                ActSearchLimits.Location = new Point(ActSearchLimits.Left, LabelMoreLimitsTop.Top);
                ActSearch.Location = new Point(ActSearch.Left, LabelMoreLimitsTop.Top);
                LabelSearchMediaType.Visible = true;
                SearchMediaType.Visible = true;
                LabelSearchLocation.Visible = true;
                SearchLocation.Visible = true;
                ActSearchLimits.Text = "<< L&ess";
            }
        }

        private void CheckInBarcode_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            CheckInBarcode.SelectAll();
        }

        private void CheckOutBarcode_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            CheckOutBarcode.SelectAll();
        }

        private void MainForm_KeyDown(object sender, KeyEventArgs e)
        {
            // ----- The keys F2 through F9 access the different sections of the form.
            switch (e.KeyCode)
            {
                case Keys.F2:
                    TaskLibraryItem();
                    e.Handled = true;
                    break;
                case Keys.F3:
                    TaskPatronRecord();
                    e.Handled = true;
                    break;
                case Keys.F4:
                    // ----- Allow form to handle Alt+F4.
                    if (e.Alt == true)
                        this.Close();
                    else
                        TaskHelp();
                    e.Handled = true;
                    break;
                case Keys.F5:
                    TaskCheckOut();
                    e.Handled = true;
                    break;
                case Keys.F6:
                    TaskCheckIn();
                    e.Handled = true;
                    break;
                case Keys.F7:
                    TaskAdmin();
                    e.Handled = true;
                    break;
                case Keys.F8:
                    TaskProcess();
                    e.Handled = true;
                    break;
                case Keys.F9:
                    TaskReports();
                    e.Handled = true;
                    break;
                case Keys.F12:
                    // ----- Prompt the user for patron or administrative mode.
                    ShowLoginForm();
                    e.Handled = true;
                    break;
            }
        }

        private void AdminLinkBarcodeLabel_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            // ----- Let the user edit the list of barcode labels.
            if (SecurityProfile[(int)LibrarySecurity.ManageBarcodeTemplates] == false)
            {
                MessageBox.Show(NotAuthorizedMessage, ProgramTitle,
                    MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }

            // ----- Edit the records.
            (new ListEditRecords()).ManageRecords(new BarcodeLabel());
        }

        private void AdminLinkBarcodePage_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            // ----- Let the user edit the list of barcode pages.
            if (SecurityProfile[(int)LibrarySecurity.ManageBarcodeTemplates] == false)
            {
                MessageBox.Show(NotAuthorizedMessage, ProgramTitle,
                    MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }

            // ----- Edit the records.
            (new ListEditRecords()).ManageRecords(new BarcodePage());
        }

        private void AdminLinkGroups_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            // ----- Let the user edit the list of security groups.
            if (SecurityProfile[(int)LibrarySecurity.ManageGroups] == false)
            {
                MessageBox.Show(NotAuthorizedMessage, ProgramTitle,
                    MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }

            // ----- Edit the records.
            (new ListEditRecords()).ManageRecords(new GroupName());
        }

        private void AdminLinkUsers_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            // ----- Let the user edit the list of administrative users.
            if (SecurityProfile[(int)LibrarySecurity.ManageUsers] == false)
            {
                MessageBox.Show(NotAuthorizedMessage, ProgramTitle,
                    MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }

            // ----- Edit the records.
            (new ListEditRecords()).ManageRecords(new UserName());
        }

        private void ActLogin_Click(object sender, EventArgs e)
        {
            // ----- Prompt the user for patron or administrative mode.
            ShowLoginForm();
        }

        private void RefreshProcessLocation()
        {
            // ----- See if the processing is up-to-date based on the location selection.
            string sqlText;
            DataCommand sqlRun;
            DataTable dbInfo;
            DateTime lastDate;
            bool newLocation;

            this.Cursor = Cursors.WaitCursor;

            // ----- Look up the dates for each location.
            lastDate = DateTime.Parse("1/1/1900");
            newLocation = false;
            sqlText = "SELECT * FROM CodeLocation";
            if (ProcessLocation.SelectedIndex != -1)
            {
                if ((long)(ListItemData)ProcessLocation.SelectedItem != -1L)
                    sqlText += " WHERE ID = @RecordID";
            }
            sqlRun = new DataCommand(sqlText);
            if (ProcessLocation.SelectedIndex != -1)
                sqlRun.AddLong("RecordID", (long)(ListItemData)ProcessLocation.SelectedItem);
            try
            {
                dbInfo = CreateDataTable(sqlRun);
            }
            catch (Exception ex)
            {
                GeneralError("MainForm.RefreshProcessLocation", ex);
                dbInfo = null;
            }
            if (dbInfo != null)
            {
                foreach (DataRow dbRow in dbInfo.Rows)
                {
                    if (Convert.IsDBNull(dbRow["LastProcessing"]) == true)
                    {
                        // ----- Processing has not been done at this location.
                        newLocation = true;
                    }
                    else
                    {
                        if (DBGetDate(dbRow["LastProcessing"]) > lastDate)
                            lastDate = DBGetDate(dbRow["LastProcessing"]);
                    }
                }
                dbInfo.Dispose();
                dbInfo = null;
            }
            this.Cursor = Cursors.Default;

            // ----- Refresh the out-of-date display.
            if ((newLocation == true) | (lastDate < DateTime.Today))
            {
                // ----- Needs processing.
                ProcessStatus.Text = "      Processing is not up to date.";
                ProcessStatus.ImageIndex = StatusImageBad;
            }
            else
            {
                // ----- Up to date.
                ProcessStatus.Text = "      Processing is up to date.";
                ProcessStatus.ImageIndex = StatusImageGood;
            }
        }

        private void ShowLoginForm()
        {
            // ----- Prompt the user for patron or administrative mode.
            DialogResult userChoice;

            // ----- Finish the in-process check-out if needed.
            if (ActiveCheckOutPatron != -1L)
                ActFinishCheckOut.PerformClick();

            // ----- Prompt the user.
            userChoice = (new ChangeUser()).ShowDialog();
            if (userChoice == DialogResult.OK)
                UpdateDisplayForUser();
        }

        private void UpdateDisplayForUser()
        {
            // ----- Update the display based on the user settings.
            string sqlText;
            DataCommand sqlRun;
            bool userCanCheckOut;

            if (LoggedInUserID == -1L)
            {
                // ----- Set up the display for patron mode.
                ActiveMode.Text = "Welcome Library Patron";

                // ----- See if patrons can check out items by themselves.
                userCanCheckOut = GetBoolean(GetSystemValue("PatronCheckOut"));

                LabelTasks.Visible = userCanCheckOut;
                LineTasks.Visible = userCanCheckOut;
                PicCheckOut.Visible = userCanCheckOut;
                ActCheckOut.Visible = userCanCheckOut;
                PicCheckIn.Visible = false;
                ActCheckIn.Visible = false;
                PicAdmin.Visible = false;
                ActAdmin.Visible = false;
                PicProcess.Visible = false;
                ActProcess.Visible = false;
                PicReports.Visible = false;
                ActReports.Visible = false;

                // ----- Show or hide the Login button per the settings.
                ActLogin.Visible = !Properties.Settings.Default.HideLogin;

                // ----- Lookup items by default.
                TaskLibraryItem();
            }
            else
            {
                // ----- Set up the display for administrative mode.
                sqlText = "SELECT FullName FROM UserName WHERE ID = @RecordID";
                sqlRun = new DataCommand(sqlText);
                sqlRun.AddLong("RecordID", LoggedInUserID);
                try
                {
                    ActiveMode.Text = "Administrative User: " +
                        DBGetText(ExecuteSQLReturn(sqlRun), "Unknown");
                }
                catch (Exception ex)
                {
                    GeneralError("MainForm.UpdateDisplayForUser", ex);
                    ActiveMode.Text = "Administrative User: Unknown";
                }

                LabelTasks.Visible = true;
                LineTasks.Visible = true;
                PicCheckOut.Visible = true;
                ActCheckOut.Visible = true;
                PicCheckIn.Visible = true;
                ActCheckIn.Visible = true;
                PicAdmin.Visible = true;
                ActAdmin.Visible = true;
                PicProcess.Visible = true;
                ActProcess.Visible = true;
                PicReports.Visible = true;
                ActReports.Visible = true;
                ActLogin.Visible = true;
            }
        }

        private void AdminLinkItems_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            // ----- Make sure the user is allowed to do this.
            if (SecurityProfile[(int)LibrarySecurity.ManageItems] == false)
            {
                MessageBox.Show(NotAuthorizedMessage, ProgramTitle,
                    MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }

            // ----- Let the user edit the list of items.
            (new ListEditRecords()).ManageRecords(new NamedItem());
        }

        private void AdminLinkAuthors_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            // ----- Make sure the user is allowed to do this.
            if (SecurityProfile[(int)LibrarySecurity.ManageAuthors] == false)
            {
                MessageBox.Show(NotAuthorizedMessage, ProgramTitle,
                    MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }

            // ----- Let the user edit the list of authors.
            (new ListEditRecords()).ManageRecords(new Author());
        }

        private void AdminLinkAuthorTypes_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            // ----- Make sure the user is allowed to do this.
            if (SecurityProfile[(int)LibrarySecurity.ManageAuthorTypes] == false)
            {
                MessageBox.Show(NotAuthorizedMessage, ProgramTitle,
                    MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }

            // ----- Let the user edit the list of author types.
            (new ListEditRecords()).ManageRecords(new CodeAuthorType());
        }

        private void AdminLinkPublishers_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            // ----- Make sure the user is allowed to do this.
            if (SecurityProfile[(int)LibrarySecurity.ManagePublishers] == false)
            {
                MessageBox.Show(NotAuthorizedMessage, ProgramTitle,
                    MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }

            // ----- Let the user edit the list of publishers.
            (new ListEditRecords()).ManageRecords(new Publisher());
        }

        private void AdminLinkSeries_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            // ----- Make sure the user is allowed to do this.
            if (SecurityProfile[(int)LibrarySecurity.ManageSeries] == false)
            {
                MessageBox.Show(NotAuthorizedMessage, ProgramTitle,
                    MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }

            // ----- Let the user edit the list of series.
            (new ListEditRecords()).ManageRecords(new CodeSeries());
        }

        private void AdminLinkMediaTypes_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            // ----- Make sure the user is allowed to do this.
            if (SecurityProfile[(int)LibrarySecurity.ManageMediaTypes] == false)
            {
                MessageBox.Show(NotAuthorizedMessage, ProgramTitle,
                    MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }

            // ----- Let the user edit the list of media types.
            (new ListEditRecords()).ManageRecords(new CodeMediaType());
        }

        private void AdminLinkStatusCodes_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            // ----- Make sure the user is allowed to do this.
            if (SecurityProfile[(int)LibrarySecurity.ManageCopyStatus] == false)
            {
                MessageBox.Show(NotAuthorizedMessage, ProgramTitle,
                    MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }

            // ----- Let the user edit the list of copy status codes.
            (new ListEditRecords()).ManageRecords(new CodeCopyStatus());
        }

        private void AdminLinkLocations_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            // ----- Make sure the user is allowed to do this.
            if (SecurityProfile[(int)LibrarySecurity.ManageLocations] == false)
            {
                MessageBox.Show(NotAuthorizedMessage, ProgramTitle,
                    MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }

            // ----- Let the user edit the list of locations.
            (new ListEditRecords()).ManageRecords(new CodeLocation());
        }

        private void AdminLinkHolidays_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            // ----- Make sure the user is allowed to do this.
            if (SecurityProfile[(int)LibrarySecurity.ManageHolidays] == false)
            {
                MessageBox.Show(NotAuthorizedMessage, ProgramTitle,
                    MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }

            // ----- Let the user edit the list of holidays.
            (new ListEditRecords()).ManageRecords(new Holiday());

            // ----- Reload the holidays if they changed.
            RefreshHolidays();
        }

        private void AdminLinkPatrons_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            // ----- Make sure the user is allowed to do this.
            if (SecurityProfile[(int)LibrarySecurity.ManagePatrons] == false)
            {
                MessageBox.Show(NotAuthorizedMessage, ProgramTitle,
                    MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }

            // ----- Let the user edit the list of patrons.
            (new ListEditRecords()).ManageRecords(new Patron());
        }

        private void AdminLinkPatronGroups_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            // ----- Make sure the user is allowed to do this.
            if (SecurityProfile[(int)LibrarySecurity.ManagePatronGroups] == false)
            {
                MessageBox.Show(NotAuthorizedMessage, ProgramTitle,
                    MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }

            // ----- Let the user edit the list of patron groups.
            (new ListEditRecords()).ManageRecords(new CodePatronGroup());
        }

        private void AdminLinkValues_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            // ----- Make sure the user is allowed to do this.
            if (SecurityProfile[(int)LibrarySecurity.ManageValues] == false)
            {
                MessageBox.Show(NotAuthorizedMessage, ProgramTitle,
                    MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }

            // ----- Access the maintenance portion of the program.
            (new Maintenance()).ShowDialog();
        }

        private void ActDoReports_Click(object sender, EventArgs e)
        {
            // ----- Show the reports form.
            (new ReportSelect()).ShowDialog();
        }

        private void SearchType_SelectedIndexChanged(object sender, EventArgs e)
        {
            // ----- Update the search hints.
            switch ((long)(ListItemData)SearchType.SelectedItem)
            {
                case (long)LookupMethods.ByTitle:
                    LabelSearchHintsData.Text = "Enter a title or partial title in the " +
                        "Search Text field. You may include the asterisk character ( * ) " +
                        "as a wildcard character.";
                    break;
                case (long)LookupMethods.ByAuthor:
                    LabelSearchHintsData.Text = "Enter an author name or partial author " +
                        "name in the Search Text field. You may include the asterisk " +
                        "character ( * ) as a wildcard character. Enter the name in " +
                        "last name, first name format, separated by a comma. You may omit " +
                        "the last name or first name if needed.";
                    break;
                case (long)LookupMethods.BySubject:
                    LabelSearchHintsData.Text = "Enter a subject or partial subject in the " +
                        "Search Text field. You may include the asterisk character ( * ) " +
                        "as a wildcard character.";
                    break;
                case (long)LookupMethods.ByKeywordAny:
                    LabelSearchHintsData.Text = "Enter one or more keywords in the " +
                        "Search Text field, separated by commas. Note that wildcard " +
                        "characters are not permitted in keyword searches. Items that " +
                        "match any one of the keywords will be returned.";
                    break;
                case (long)LookupMethods.ByKeywordAll:
                    LabelSearchHintsData.Text = "Enter one or more keywords in the " +
                        "Search Text field, separated by commas. Note that wildcard " +
                        "characters are not permitted in keyword searches. Only items " +
                        "that match all of the keywords will be returned.";
                    break;
                case (long)LookupMethods.ByPublisher:
                    LabelSearchHintsData.Text = "Enter a publisher name or partial " +
                        "publisher name in the Search Text field. You may include " +
                        "the asterisk character ( * ) as a wildcard character.";
                    break;
                case (long)LookupMethods.BySeries:
                    LabelSearchHintsData.Text = "Enter a series name or partial " +
                        "series name in the Search Text field. You may include " +
                        "the asterisk character ( * ) as a wildcard character.";
                    break;
                case (long)LookupMethods.ByBarcode:
                    LabelSearchHintsData.Text = "Enter a numeric barcode for a library " +
                        "item in the Search Text field. Note that wildcard characters " +
                        "are not allowed in barcode searches.";
                    break;
                case (long)LookupMethods.ByDatabaseID:
                    LabelSearchHintsData.Text = "Enter a numeric database ID for a library " +
                        "item in the Search Text field. Note that wildcard characters " +
                        "are not allowed in database ID searches.";
                    break;
            }
        }

        private void ActSearchClear_Click(object sender, EventArgs e)
        {
            // ----- Clear the current search criteria.
            SearchType.SelectedIndex = SearchType.Items.IndexOf((long)LookupMethods.ByTitle);
            SearchText.Text = "";
            SearchMediaType.SelectedIndex = SearchMediaType.Items.IndexOf(-1L);
            SearchLocation.SelectedIndex = SearchLocation.Items.IndexOf(-1L);
        }

        private void ActSearch_Click(object sender, EventArgs e)
        {
            // ----- The user wants to lookup some items.
            long mediaLimit;
            long locationLimit;
            long searchMethod;

            // ----- Make sure the user supplied valid search criteria.
            searchMethod = (long)(ListItemData)SearchType.SelectedItem;
            switch (searchMethod)
            {
                case (long)LookupMethods.ByTitle:
                case (long)LookupMethods.BySubject:
                case (long)LookupMethods.ByPublisher:
                case (long)LookupMethods.BySeries:
                    // ----- A search text is required.
                    if (SearchText.Text.Trim().Length == 0)
                    {
                        MessageBox.Show("Please supply a search text.", ProgramTitle,
                            MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                        SearchText.Focus();
                        return;
                    }
                    break;
                case (long)LookupMethods.ByAuthor:
                    // ----- A search text with a maximum of one comma is required.
                    if ((CountSubStr(SearchText.Text, ",") >= 2) |
                        (SearchText.Text.Trim().Length == 0))
                    {
                        MessageBox.Show("Please supply a search text with a maximum of one comma.",
                            ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                        SearchText.Focus();
                        return;
                    }

                    // ----- Make sure at least one name component is supplied.
                    if ((SearchText.Text.GetSubStr(",", 1).Trim().Length == 0) &
                        (SearchText.Text.GetSubStr(",", 2).Trim().Length == 0))
                    {
                        MessageBox.Show("Please supply at least one name component.",
                            ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                        SearchText.Focus();
                        return;
                    }
                    break;
                case (long)LookupMethods.ByKeywordAny:
                case (long)LookupMethods.ByKeywordAll:
                    // ----- A search text is required.
                    if (SearchText.Text.Trim().Length == 0)
                    {
                        MessageBox.Show("Please supply a search text.", ProgramTitle,
                            MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                        SearchText.Focus();
                        return;
                    }

                    // ----- Make sure that no wildcards are included.
                    if (SearchText.Text.Contains("*") == true)
                    {
                        MessageBox.Show("Wildcard characters (*) are not permitted in keyword searches.",
                            ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                        SearchText.Focus();
                        return;
                    }
                    break;
                case (long)LookupMethods.ByBarcode:
                case (long)LookupMethods.ByDatabaseID:
                    // ----- A numeric search text is required.
                    if ((DigitsOnly(SearchText.Text) != SearchText.Text.Trim()) |
                        (SearchText.Text.Trim().Length == 0))
                    {
                        MessageBox.Show("Please supply a numeric search text.",
                            ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                        SearchText.Focus();
                        return;
                    }
                    break;
            }

            // ----- Get the extended criteria if needed.
            mediaLimit = -1L;
            locationLimit = -1L;
            if (SearchMediaType.Visible == true)
            {
                mediaLimit = (long)(ListItemData)SearchMediaType.SelectedItem;
                locationLimit = (long)(ListItemData)SearchLocation.SelectedItem;
            }

            // ----- Initiate the search.
            (new ItemLookup()).InitiateSearch((LookupMethods)searchMethod,
                SearchText.Text.Trim(), mediaLimit, locationLimit);
        }

        private void ActAccessPatron_Click(object sender, EventArgs e)
        {
            // ----- Look up the record of an active patron.
            long patronID;

            // ----- Get the ID of the patron.
            patronID = (new PatronAccess()).SelectPatron();
            if (patronID == -1L)
                return;

            // ----- Show the patron record.
            (new PatronRecord()).ViewPatronRecord(patronID, true);
        }

        private void ActReportsBarcode_Click(object sender, EventArgs e)
        {
            // ----- Make sure the user is allowed to do this.
            if (SecurityProfile[(int)LibrarySecurity.ManageBarcodeTemplates] == false)
            {
                MessageBox.Show(NotAuthorizedMessage, ProgramTitle,
                    MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }

            // ----- Show the barcode label printing form.
            (new BarcodePrint()).ShowDialog();
        }

        private void ActCheckOutPatron_Click(object sender, EventArgs e)
        {
            // ----- Select a different patron for check-out purposes.
            long patronID;
            string patronName;
            string sqlText;
            DataCommand sqlRun;
            decimal finesDue;
            DataTable dbInfo;

            // ----- Finish previous patron if needed.
            if (ActiveCheckOutPatron != -1L)
                ActFinishCheckOut.PerformClick();

            // ----- Get the ID of the patron.
            patronID = (new PatronAccess()).SelectPatron();
            if (patronID == -1L)
                return;

            // ----- Get the patron name.
            sqlText = "SELECT FirstName + ' ' + LastName FROM Patron " +
                "WHERE ID = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", patronID);
            try
            {
                patronName = DBGetText(ExecuteSQLReturn(sqlRun));
            }
            catch (Exception ex)
            {
                GeneralError("MainForm.ActCheckOutPatron_Click.Name", ex);
                patronName = "Unknown";
            }

            // ----- Is this patron active?
            sqlText = "SELECT Active FROM Patron WHERE ID = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", patronID);
            try
            {
                if (DBGetBool(ExecuteSQLReturn(sqlRun)) == false)
                {
                    MessageBox.Show("Patron '" + patronName + "' is marked as inactive.",
                        ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    return;
                }
            }
            catch (Exception ex)
            {
                GeneralError("MainForm.ActCheckOutPatron_Click.Active", ex);
                return;
            }

            // ----- OK to start using this patron.
            ActiveCheckOutPatron = patronID;
            CheckOutPatronName.Text = patronName;
            this.Cursor = Cursors.WaitCursor;

            // ----- Bring the patron record up-to-date.
            sqlText = "SELECT ID FROM PatronCopy " +
                "WHERE Returned = 0 AND Missing = 0 " +
                "AND DueDate < @TargetDate AND (ProcessDate IS NULL " +
                "OR ProcessDate < @TargetDate) AND Patron = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddDate("TargetDate", DateTime.Today);
            sqlRun.AddLong("RecordID", patronID);
            try
            {
                dbInfo = CreateDataTable(sqlRun);
            }
            catch (Exception ex)
            {
                this.Cursor = Cursors.Default;
                GeneralError("MainForm.ActCheckOutPatron_Click.Date", ex);
                dbInfo = null;
            }
            if (dbInfo != null)
            {
                foreach (DataRow dbRow in dbInfo.Rows)
                    DailyProcessByPatronCopy(DBGetLong(dbRow["ID"]), DateTime.Today);
                dbInfo.Dispose();
                dbInfo = null;
            }

            // ----- Show the patron if there are any fines due.
            finesDue = CalculatePatronFines(patronID);
            if (finesDue > 0M)
                MessageBox.Show("Patron: " + CheckOutPatronName.Text + Environment.NewLine +
                    "Fines Due: " + finesDue.ToString("C"), ProgramTitle,
                    MessageBoxButtons.OK, MessageBoxIcon.Information);

            // ----- Show the other fields.
            LabelBarcode.Visible = true;
            CheckOutBarcode.Text = "";
            CheckOutBarcode.Visible = true;
            ActDoCheckOut.Visible = true;
            LabelCheckedOutItems.Visible = true;
            CheckedOutItems.Items.Clear();
            CheckedOutItems.Visible = true;
            TotalCheckedOut.Text = "      0 items checked out.";
            TotalCheckedOut.ImageIndex = StatusImageGood;
            TotalCheckedOut.Visible = true;
            ActFinishCheckOut.Visible = true;
            this.AcceptButton = ActDoCheckOut;
            this.Cursor = Cursors.Default;
            CheckOutBarcode.Focus();
        }

        private void ActDoCheckOut_Click(object sender, EventArgs e)
        {
            // ----- The user wants to check out an item.
            string sqlText;
            DataCommand sqlRun;
            DataTable dbInfo;
            DataRow oneRow;
            long copyID;
            long namedID;
            int checkoutDays;
            DateTime untilDate;
            CheckedOutItem itemDetail;
            long somePatron;
            LibraryTransaction transInfo = null;

            // ----- A barcode or title is required.
            if (CheckOutBarcode.Text.Trim().Length == 0)
            {
                MessageBox.Show("Please supply a valid barcode or item title for check out.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }

            // ------ The user supplied either a barcode or a copy. We need to
            //        get the ID of the copy instead.
            if (CheckDecimal(CheckOutBarcode.Text) == true)
            {
                // ----- Probably a barcode supplied. Get the related ID.
                sqlText = "SELECT ID FROM ItemCopy WHERE Barcode = @TestBarcode";
                sqlRun = new DataCommand(sqlText);
                sqlRun.AddText("TestBarcode", DigitsOnly(CheckOutBarcode.Text));
                try
                {
                    copyID = DBGetLong(ExecuteSQLReturn(sqlRun), -1L);
                }
                catch (Exception ex)
                {
                    GeneralError("MainForm.ActDoCheckOut.Copy", ex);
                    copyID = -1L;
                }
                if (copyID == -1L)
                {
                    // ----- Invalid barcode.
                    MessageBox.Show("Barcode not found.", ProgramTitle,
                        MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    CheckOutBarcode.Focus();
                    CheckOutBarcode.SelectAll();
                    return;
                }
            }
            else
            {
                // ----- Lookup by title.
                copyID = (new CheckLookup()).CheckItemByTitle(
                    false, CheckOutBarcode.Text.Trim());
                if (copyID == -1L)
                    return;
            }
            this.Cursor = Cursors.WaitCursor;

            // ----- Get the named ID for this copy ID.
            sqlText = "SELECT ItemID FROM ItemCopy WHERE ID = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", copyID);
            try
            {
                namedID = DBGetLong(ExecuteSQLReturn(sqlRun));
            }
            catch (Exception ex)
            {
                this.Cursor = Cursors.Default;
                GeneralError("MainForm.ActDoCheckOut.Named", ex);
                return;
            }

            // ----- Build a record that will store the item details.
            itemDetail = new CheckedOutItem();

            // ----- Get some general information about the item.
            sqlText = "SELECT NI.Title, IC.CopyNumber, IC.Barcode " +
                "FROM ItemCopy AS IC INNER JOIN NamedItem AS NI ON IC.ItemID = NI.ID " +
                "WHERE IC.ID = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", copyID);
            try
            {
                dbInfo = CreateDataTable(sqlRun);
            }
            catch (Exception ex)
            {
                this.Cursor = Cursors.Default;
                GeneralError("MainForm.ActDoCheckOut.Info", ex);
                return;
            }
            oneRow = dbInfo.Rows[0];
            itemDetail.Barcode = DBGetText(oneRow["Barcode"]);
            itemDetail.ItemTitle = DBGetText(oneRow["Title"]);
            itemDetail.CopyNumber = DBGetInteger(oneRow["CopyNumber"]);
            dbInfo.Dispose();

            // ----- Make sure this copy is available for circulation.
            sqlText = "SELECT Available, Reference FROM ItemCopy WHERE ID = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", copyID);
            try
            {
                dbInfo = CreateDataTable(sqlRun);
            }
            catch (Exception ex)
            {
                this.Cursor = Cursors.Default;
                GeneralError("MainForm.ActDoCheckOut.Circ", ex);
                return;
            }
            oneRow = dbInfo.Rows[0];
            if ((DBGetBool(oneRow["Available"]) == false) |
                (DBGetBool(oneRow["Reference"]) == true))
            {
                this.Cursor = Cursors.Default;
                dbInfo.Dispose();
                dbInfo = null;
                MessageBox.Show("You cannot check out this copy of '" + itemDetail.ItemTitle +
                    "' because it is not available for circulation.", ProgramTitle,
                    MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                CheckOutBarcode.Focus();
                CheckOutBarcode.SelectAll();
                return;
            }
            dbInfo.Dispose();

            // ----- Make sure this title is not already checked out.
            sqlText = "SELECT Patron FROM PatronCopy " +
                "WHERE ItemCopy = @RecordID AND Returned = 0";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", copyID);
            try
            {
                somePatron = DBGetLong(ExecuteSQLReturn(sqlRun), -1L);
            }
            catch (Exception ex)
            {
                this.Cursor = Cursors.Default;
                GeneralError("MainForm.ActDoCheckOut.Out", ex);
                return;
            }
            if (somePatron != -1L)
            {
                this.Cursor = Cursors.Default;
                if (somePatron == ActiveCheckOutPatron)
                    MessageBox.Show("You cannot renew an item through the check out " +
                        "features of this system. Use the renewal features through " +
                        "the Patron Record portion of the system. Please see the " +
                        "library staff for assistance.", ProgramTitle,
                        MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                else
                    MessageBox.Show("You cannot check out this copy of '" +
                        itemDetail.ItemTitle + "' because it is currently checked " +
                        "out to another patron. Please see the " +
                        "library staff for assistance.", ProgramTitle,
                        MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                CheckOutBarcode.Focus();
                CheckOutBarcode.SelectAll();
                return;
            }

            // ----- Get the number of days this item can be checked out.
            sqlText = "SELECT CMT.CheckoutDays FROM NamedItem AS NI INNER JOIN " +
                "CodeMediaType AS CMT ON NI.MediaType = CMT.ID WHERE NI.ID = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", namedID);
            try
            {
                checkoutDays = DBGetInteger(ExecuteSQLReturn(sqlRun));
            }
            catch (Exception ex)
            {
                this.Cursor = Cursors.Default;
                GeneralError("MainForm.ActDoCheckOut.Days", ex);
                return;
            }
            if (checkoutDays == 0)
            {
                this.Cursor = Cursors.Default;
                MessageBox.Show("You cannot check out this copy of '" +
                    itemDetail.ItemTitle + "' because it has a check out limit " +
                    "of zero days. Please see the library staff for assistance.", ProgramTitle,
                    MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                CheckOutBarcode.Focus();
                CheckOutBarcode.SelectAll();
                return;
            }

            // ----- Don't allow the due date to be on a holiday.
            untilDate = DateTime.Today.AddDays(checkoutDays);
            while (IsHolidayDate(untilDate) == true)
                untilDate = untilDate.AddDays(1);
            itemDetail.DueDate = untilDate;

            try
            {
                // ----- Perform the check-out in a transaction.
                transInfo = TransactionBegin();

                // ----- Update patron copy record.
                sqlText = "INSERT INTO PatronCopy (Patron, ItemCopy, CheckOut, Renewal, " +
                    "DueDate, Returned, Missing, Fine, Paid) VALUES (@NewPatron, @NewCopy, " +
                    "@NewDate, 0, @NewDue, 0, 0, 0, 0)";
                sqlRun = new DataCommand(sqlText);
                sqlRun.AddLong("NewPatron", ActiveCheckOutPatron);
                sqlRun.AddLong("NewCopy", copyID);
                sqlRun.AddDate("NewDate", DateTime.Today);
                sqlRun.AddDate("NewDue", untilDate);
                ExecuteSQL(sqlRun, transInfo);

                // ----- Update the patron record.
                sqlText = "UPDATE Patron SET LastActivity = GETDATE() " +
                    "WHERE ID = @RecordID";
                sqlRun = new DataCommand(sqlText);
                sqlRun.AddLong("RecordID", ActiveCheckOutPatron);
                ExecuteSQL(sqlRun, transInfo);

                TransactionCommit(transInfo);
            }
            catch (Exception ex)
            {
                if (transInfo != null)
                    TransactionRollback(transInfo);
                this.Cursor = Cursors.Default;
                GeneralError("MainForm.ActDoCheckOut.CheckOut", ex);
                return;
            }

            // ----- Display results.
            CheckedOutItems.Items.Add(itemDetail);
            if (CheckedOutItems.Items.Count == 1)
                TotalCheckedOut.Text = "      1 item checked out.";
            else
                TotalCheckedOut.Text = "      " +
                    CheckedOutItems.Items.Count.ToString() + " item checked out.";
            this.Cursor = Cursors.Default;

            // ----- Finished.
            CheckOutBarcode.Text = "";
            CheckOutBarcode.Focus();
        }

        private void ActFinishCheckOut_Click(object sender, EventArgs e)
        {
            // ----- Print a check-out ticket if possible.
            TicketPrinting.PrintCheckoutTicket(ActiveCheckOutPatron, CheckedOutItems);

            // ----- Clear the active check-out patron.
            this.AcceptButton = ActCheckOutPatron;
            ActiveCheckOutPatron = -1L;
            CheckOutPatronName.Text = "No active patron.";
            LabelBarcode.Visible = false;
            CheckOutBarcode.Text = "";
            CheckOutBarcode.Visible = false;
            ActDoCheckOut.Visible = false;
            LabelCheckedOutItems.Visible = false;
            CheckedOutItems.Items.Clear();
            CheckedOutItems.Visible = false;
            TotalCheckedOut.Text = "      0 items checked out.";
            TotalCheckedOut.ImageIndex = StatusImageGood;
            TotalCheckedOut.Visible = false;
            ActFinishCheckOut.Visible = false;
        }

        private void CheckedOutItems_DrawItem(object sender, DrawItemEventArgs e)
        {
            // ----- Draw the matching items on two lines.
            Brush useBrush;
            string dateText;
            string titleText;
            CheckedOutItem itemDetail;
            StringFormat ellipsesText;

            // ----- Ignore undrawable items.
            if (e.Index == -1)
                return;

            // ----- Draw the background of the item.
            if ((e.State & DrawItemState.Selected) == DrawItemState.Selected)
                useBrush = SystemBrushes.HighlightText;
            else
                useBrush = SystemBrushes.WindowText;
            e.DrawBackground();

            // ----- Extract the detail from the list.
            itemDetail = (CheckedOutItem)(CheckedOutItems.Items[e.Index]);

            // ----- Format the display fields.
            dateText = itemDetail.DueDate.ToString("M/d/yyyy");
            titleText = itemDetail.Barcode;
            if (titleText.Length != 0)
                titleText += " - ";
            titleText += itemDetail.ItemTitle + " (Copy " +
                itemDetail.CopyNumber.ToString() + ")";
            ellipsesText = new StringFormat();
            ellipsesText.Trimming = StringTrimming.EllipsisCharacter;

            // ----- Draw the text of the item.
            e.Graphics.DrawString(dateText, e.Font, useBrush, new Rectangle(0,
                e.Bounds.Top, LabelCheckedOutCol2.Left - CheckedOutItems.Left - 5,
                e.Font.Height), ellipsesText);
            e.Graphics.DrawString(titleText, e.Font, useBrush,
                LabelCheckedOutCol2.Left - CheckedOutItems.Left, e.Bounds.Top);

            // ----- If the ListBox has focus, draw a focus rectangle.
            e.DrawFocusRectangle();
        }

        private void CheckInDate_ValueChanged(object sender, EventArgs e)
        {
            // ----- Adjust the day in the display.
            switch ((int)(DateTime.Today - CheckInDate.Value).TotalDays)
            {
                case 0: // ----- Today
                    CheckInDay.Text = "Today";
                    CheckInDay.BackColor = SystemColors.Control;
                    CheckInDay.ForeColor = SystemColors.ControlText;
                    break;
                case 1: // ----- Yesterday
                    CheckInDay.Text = "Yesterday";
                    CheckInDay.BackColor = Color.Red;
                    CheckInDay.ForeColor = Color.White;
                    break;
                default: // ----- X days ago
                    CheckInDay.Text = Convert.ToInt32((DateTime.Today -
                        CheckInDate.Value).TotalDays).ToString() + " days ago";
                    CheckInDay.BackColor = Color.Red;
                    CheckInDay.ForeColor = Color.White;
                    break;
            }
        }

        private void ActDoCheckIn_Click(object sender, EventArgs e)
        {
            // ----- The user wants to check in an item.
            string sqlText;
            DataCommand sqlRun;
            DataTable dbInfo;
            DataRow oneRow;
            long copyID;
            long patronCopyID;
            bool copyMissing;
            int daysLate;
            decimal fineOwed;
            string itemInfo;
            long patronID;
            LibraryTransaction transInfo = null;

            // ----- A barcode or title is required.
            if (CheckInBarcode.Text.Trim().Length == 0)
            {
                MessageBox.Show("Please supply a valid barcode or item title for check in.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }

            // ------ The user supplied either a barcode or a copy. We need to
            //        get the ID of the copy instead.
            if (CheckDecimal(CheckInBarcode.Text) == true)
            {
                // ----- Probably a barcode supplied. Get the related ID.
                sqlText = "SELECT ID FROM ItemCopy WHERE Barcode = @TestBarcode";
                sqlRun = new DataCommand(sqlText);
                sqlRun.AddText("TestBarcode", DigitsOnly(CheckInBarcode.Text));
                try
                {
                    copyID = DBGetLong(ExecuteSQLReturn(sqlRun), -1L);
                }
                catch (Exception ex)
                {
                    GeneralError("MainForm.ActDoCheckIn.Copy", ex);
                    copyID = -1L;
                }
                if (copyID == -1L)
                {
                    // ----- Invalid barcode.
                    LabelCheckedIn.Text = "      Barcode not found.";
                    LabelCheckedIn.ImageIndex = StatusImageBad;
                    LabelCheckedIn.Visible = true;
                    CheckedInDetail.Visible = false;
                    CheckInBarcode.Focus();
                    CheckInBarcode.SelectAll();
                    return;
                }
            }
            else
            {
                // ----- Lookup by title.
                copyID = (new CheckLookup()).CheckItemByTitle(
                    true, CheckInBarcode.Text.Trim());
                if (copyID == -1L)
                    return;
            }
            this.Cursor = Cursors.WaitCursor;

            // ----- Get some general information about the book.
            sqlText = "SELECT NI.Title, NI.Subtitle, IC.CopyNumber, IC.Barcode " +
                "FROM ItemCopy AS IC INNER JOIN NamedItem AS NI ON IC.ItemID = NI.ID " +
                "WHERE IC.ID = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", copyID);
            try
            {
                dbInfo = CreateDataTable(sqlRun);
            }
            catch (Exception ex)
            {
                this.Cursor = Cursors.Default;
                GeneralError("MainForm.ActDoCheckIn.Info", ex);
                return;
            }
            oneRow = dbInfo.Rows[0];
            itemInfo = DBGetText(oneRow["Title"]);
            if (Convert.IsDBNull(oneRow["Subtitle"]) == false)
                itemInfo += ": " + DBGetText(oneRow["Subtitle"]);
            itemInfo += Environment.NewLine + Environment.NewLine +
                "Copy: " + DBGetInteger(oneRow["CopyNumber"]).ToString();
            if (Convert.IsDBNull(oneRow["Barcode"]) == false)
                itemInfo += Environment.NewLine +
                    "Barcode: " + DBGetText(oneRow["Barcode"]);
            dbInfo.Dispose();

            // ----- Make sure this title is checked out.
            sqlText = "SELECT ID, Patron, Missing FROM PatronCopy " +
                "WHERE ItemCopy = @RecordID AND Returned = 0";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", copyID);
            try
            {
                dbInfo = CreateDataTable(sqlRun);
            }
            catch (Exception ex)
            {
                this.Cursor = Cursors.Default;
                GeneralError("MainForm.ActDoCheckIn.Verify", ex);
                return;
            }
            if (dbInfo.Rows.Count == 0)
            {
                this.Cursor = Cursors.Default;
                dbInfo.Dispose();
                dbInfo = null;
                LabelCheckedIn.Text = "      Indicated item was not checked out.";
                LabelCheckedIn.ImageIndex = StatusImageBad;
                LabelCheckedIn.Visible = true;
                CheckedInDetail.Text = itemInfo;
                CheckedInDetail.Visible = true;
                CheckInBarcode.Focus();
                CheckInBarcode.SelectAll();
                return;
            }
            oneRow = dbInfo.Rows[0];
            patronCopyID = DBGetLong(oneRow["ID"]);
            patronID = DBGetLong(oneRow["Patron"]);
            copyMissing = DBGetBool(oneRow["Missing"]);
            dbInfo.Dispose();

            // ----- Bring the status of the item up-to-date.
            DailyProcessByPatronCopy(patronCopyID, CheckInDate.Value);

            // ----- Handle missing items. Although it is not done here, a possible
            //       enhancement would be to recognize any fine charaged to the
            //       patron specifically for the missing book, and prompt a refund.
            if (copyMissing)
            {
                // ----- No longer missing.
                sqlText = "UPDATE PatronCopy SET Missing = 0 WHERE ID = @RecordID";
                sqlRun = new DataCommand(sqlText);
                sqlRun.AddLong("RecordID", patronCopyID);
                try
                {
                    ExecuteSQL(sqlRun);
                }
                catch (Exception ex)
                {
                    this.Cursor = Cursors.Default;
                    GeneralError("MainForm.ActDoCheckIn.Miss1", ex);
                    return;
                }

                sqlText = "UPDATE ItemCopy SET Missing = 0 WHERE ID = @RecordID";
                sqlRun = new DataCommand(sqlText);
                sqlRun.AddLong("RecordID", copyID);
                try
                {
                    ExecuteSQL(sqlRun);
                }
                catch (Exception ex)
                {
                    this.Cursor = Cursors.Default;
                    GeneralError("MainForm.ActDoCheckIn.Miss2", ex);
                    return;
                }
            }

            // ----- Handle overdue items.
            daysLate = 0;
            fineOwed = 0M;
            sqlText = "SELECT DueDate, Fine, Paid FROM PatronCopy " +
                "WHERE ID = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", patronCopyID);
            try
            {
                dbInfo = CreateDataTable(sqlRun);
            }
            catch (Exception ex)
            {
                this.Cursor = Cursors.Default;
                GeneralError("MainForm.ActDoCheckIn.Overdue", ex);
                return;
            }
            oneRow = dbInfo.Rows[0];
            if (DBGetDate(oneRow["DueDate"]) < DateTime.Today)
                daysLate = (int)(DateTime.Today - DBGetDate(oneRow["DueDate"])).TotalDays;
            if (Convert.IsDBNull(oneRow["Fine"]) == false)
                fineOwed = DBGetDecimal(oneRow["Fine"]);
            if (Convert.IsDBNull(oneRow["Paid"]) == false)
                fineOwed -= DBGetDecimal(oneRow["Paid"]);
            if (fineOwed < 0M)
                fineOwed = 0M;
            dbInfo.Dispose();

            try
            {
                // ----- Do the check-in in a transaction.
                transInfo = TransactionBegin();

                // ----- Update patron copy record.
                sqlText = "UPDATE PatronCopy SET CheckIn = @NewDate, " +
                    "Returned = 1 WHERE ID = @RecordID";
                sqlRun = new DataCommand(sqlText);
                sqlRun.AddDate("NewDate", CheckInDate.Value);
                sqlRun.AddLong("RecordID", patronCopyID);
                ExecuteSQL(sqlRun, transInfo);

                // ----- Update the patron record.
                sqlText = "UPDATE Patron SET LastActivity = GETDATE() " +
                    "WHERE ID = @RecordID";
                sqlRun = new DataCommand(sqlText);
                sqlRun.AddLong("RecordID", patronID);
                ExecuteSQL(sqlRun, transInfo);

                TransactionCommit(transInfo);
            }
            catch (Exception ex)
            {
                if (transInfo != null)
                    TransactionRollback(transInfo);
                this.Cursor = Cursors.Default;
                GeneralError("MainForm.ActDoCheckIn.CheckIn", ex);
                return;
            }

            // ----- Display results.
            LabelCheckedIn.Text = "      The following item was checked in successfully.";
            LabelCheckedIn.ImageIndex = StatusImageGood;
            LabelCheckedIn.Visible = true;
            if ((daysLate > 0) | (fineOwed > 0M))
                itemInfo += Environment.NewLine;
            if (daysLate > 0)
                itemInfo += Environment.NewLine + "Days Late: " + daysLate.ToString();
            if (fineOwed > 0M)
                itemInfo += Environment.NewLine + "Fine Owed: " + fineOwed.ToString("C");
            CheckedInDetail.Text = itemInfo;
            CheckedInDetail.Visible = true;
            this.Cursor = Cursors.Default;

            // ----- Finished.
            CheckInBarcode.Text = "";
            CheckInBarcode.Focus();
            dbInfo = null;
        }

        private void ProcessLocation_SelectedIndexChanged(object sender, EventArgs e)
        {
            // ----- Update the status based on the current location.
            RefreshProcessLocation();
        }

        private void ActDoProcess_Click(object sender, EventArgs e)
        {
            // ----- Process all of the checked out books.
            string sqlText;
            DataCommand sqlRun;
            DataTable dbInfo;
            long locationID;

            this.Cursor = Cursors.WaitCursor;

            // ----- Get the list of all items that likely need processing.
            sqlText = "SELECT PC.ID FROM PatronCopy AS PC INNER JOIN ItemCopy AS IC " +
                "ON PC.ItemCopy = IC.ID WHERE PC.Returned = 0 AND PC.Missing = 0 " +
                "AND IC.Missing = 0 AND PC.DueDate < @TargetDate " +
                "AND (PC.ProcessDate IS NULL OR PC.ProcessDate < @TargetDate)";
            if (ProcessLocation.SelectedIndex != -1)
            {
                locationID = (long)(ListItemData)ProcessLocation.SelectedItem;
                if (locationID != -1L)
                    sqlText += " AND IC.Location = @RecordID";
            }
            else
            {
                locationID = -1L;
            }
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddDate("TargetDate", DateTime.Today);
            if (locationID != -1L)
                sqlRun.AddLong("RecordID", locationID);
            try
            {
                dbInfo = CreateDataTable(sqlRun);
            }
            catch (Exception ex)
            {
                GeneralError("MainForm.ActDoProcess_Click.Retrieve", ex);
                dbInfo = null;
            }
            if (dbInfo != null)
            {
                foreach (DataRow dbRow in dbInfo.Rows)
                    DailyProcessByPatronCopy(DBGetLong(dbRow["ID"]), DateTime.Today);
                dbInfo.Dispose();
                dbInfo = null;
            }
            this.Cursor = Cursors.Default;
            MessageBox.Show("Processing complete.", ProgramTitle,
                MessageBoxButtons.OK, MessageBoxIcon.Information);

            // ----- Update the processing date.
            sqlText = "UPDATE CodeLocation SET LastProcessing = @TargetDate";
            if (locationID != -1L)
                sqlText += " WHERE ID = " + locationID.ToString();
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddDate("TargetDate", DateTime.Today);
            if (locationID != -1L)
                sqlRun.AddLong("RecordID", locationID);
            try
            {
                ExecuteSQL(sqlRun);
            }
            catch (Exception ex)
            {
                GeneralError("MainForm.ActDoProcess_Click.Update", ex);
            }

            // ----- Update the status display.
            ProcessStatus.Text = "      Processing is up to date.";
            ProcessStatus.ImageIndex = StatusImageGood;
        }
    }
}
