﻿// ----- The Library Project
//       Start-to-Finish Visual C# 2015
//       Copyright (c) 2016 by Tim Patrick

using System;
using System.Data;
using System.Drawing;
using System.Windows.Forms;
using static Library.General;

namespace Library
{
    public partial class MainForm : Form
    {
        public MainForm()
        {
            InitializeComponent();
        }

        protected override void WndProc(ref Message m)
        {
            // ----- Watch for messages from other copies of the application.
            if (m.Msg == NativeMethods.WM_LibraryNotify)
            {
                // ----- Focus on this form.
                if (this.WindowState == FormWindowState.Minimized)
                    this.WindowState = FormWindowState.Normal;
                this.Activate();
            }
            base.WndProc(ref m);
        }

        private void ActHelpAbout_Click(object sender, EventArgs e)
        {
            // ----- Show the About form.
            (new AboutProgram()).ShowDialog();
        }

        private void ActLibraryItem_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            // ----- Library Item mode.
            TaskLibraryItem();
        }

        private void PicLibraryItem_Click(object sender, EventArgs e)
        {
            // ----- Library Item mode.
            TaskLibraryItem();
        }

        private void ActPatronRecord_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            // ----- Patron Record mode.
            TaskPatronRecord();
        }

        private void PicPatronRecord_Click(object sender, EventArgs e)
        {
            // ----- Patron Record mode.
            TaskPatronRecord();
        }

        private void ActHelp_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            // ----- Help mode.
            TaskHelp();
        }

        private void PicHelp_Click(object sender, EventArgs e)
        {
            // ----- Help mode.
            TaskHelp();
        }

        private void ActCheckOut_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            // ----- Check Out mode.
            TaskCheckOut();
        }

        private void PicCheckOut_Click(object sender, EventArgs e)
        {
            // ----- Check Out mode.
            TaskCheckOut();
        }

        private void ActCheckIn_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            // ----- Check In mode.
            TaskCheckIn();
        }

        private void PicCheckIn_Click(object sender, EventArgs e)
        {
            // ----- Check In mode.
            TaskCheckIn();
        }

        private void ActAdmin_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            // ----- Administration mode.
            TaskAdmin();
        }

        private void PicAdmin_Click(object sender, EventArgs e)
        {
            // ----- Administration mode.
            TaskAdmin();
        }

        private void ActProcess_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            // ----- Daily Processing mode.
            TaskProcess();
        }

        private void PicProcess_Click(object sender, EventArgs e)
        {
            // ----- Daily Processing mode.
            TaskProcess();
        }

        private void ActReports_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            // ----- Print Reports mode.
            TaskReports();
        }

        private void PicReports_Click(object sender, EventArgs e)
        {
            // ----- Print Reports mode.
            TaskReports();
        }

        private void TaskLibraryItem()
        {
            // ----- Library item task.

            // ----- Update the display.
            AllPanelsInvisible();
            PanelLibraryItem.Visible = true;
            ActLibraryItem.BackColor = SystemColors.Control;
            LabelSelected.Location = new Point(LabelSelected.Left, PicLibraryItem.Top);
            this.AcceptButton = ActSearch;

            // ----- Get ready for a new search.
            if (SearchText.CanFocus)
                SearchText.Focus();
        }

        private void TaskPatronRecord()
        {
            // ----- Patron Record mode.

            // ----- Update the display.
            AllPanelsInvisible();
            PanelPatronRecord.Visible = true;
            ActPatronRecord.BackColor = SystemColors.Control;
            LabelSelected.Location = new Point(LabelSelected.Left, PicPatronRecord.Top);
            this.AcceptButton = ActAccessPatron;
        }

        private void TaskHelp()
        {
            // ----- Help mode.

            // ----- Update the display.
            AllPanelsInvisible();
            PanelHelp.Visible = true;
            ActHelp.BackColor = SystemColors.Control;
            LabelSelected.Location = new Point(LabelSelected.Left, PicHelp.Top);
            this.AcceptButton = null;
        }

        private void TaskCheckOut()
        {
            // ----- Check Out mode.

            // ----- Update the display.
            AllPanelsInvisible();
            PanelCheckOut.Visible = true;
            ActCheckOut.BackColor = SystemColors.Control;
            LabelSelected.Location = new Point(LabelSelected.Left, PicCheckOut.Top);
            this.AcceptButton = ActDoCheckOut;
        }

        private void TaskCheckIn()
        {
            // ----- Check In mode.

            // ----- Update the display.
            AllPanelsInvisible();
            PanelCheckIn.Visible = true;
            ActCheckIn.BackColor = SystemColors.Control;
            LabelSelected.Location = new Point(LabelSelected.Left, PicCheckIn.Top);
            this.AcceptButton = ActDoCheckIn;

            // ----- Reset the input fields.
            CheckInDate.Value = DateTime.Today;
            CheckInDate.MaxDate = DateTime.Today;
            CheckInDate.MinDate = DateTime.Today.AddDays(-6);
            CheckInDay.Text = "Today";
            CheckInDay.BackColor = SystemColors.Control;
            CheckInDay.ForeColor = SystemColors.ControlText;
            CheckInBarcode.Text = "";
            LabelCheckedIn.Visible = false;
            CheckedInDetail.Visible = false;
            CheckedInDetail.Text = "No information available.";
            CheckInBarcode.Focus();
        }

        private void TaskAdmin()
        {
            // ----- Administration mode.

            // ----- Update the display.
            AllPanelsInvisible();
            PanelAdmin.Visible = true;
            ActAdmin.BackColor = SystemColors.Control;
            LabelSelected.Location = new Point(LabelSelected.Left, PicAdmin.Top);
            this.AcceptButton = null;
        }

        private void TaskProcess()
        {
            // ----- Daily Processing mode.

            // ----- Update the display.
            AllPanelsInvisible();
            PanelProcess.Visible = true;
            ActProcess.BackColor = SystemColors.Control;
            LabelSelected.Location = new Point(LabelSelected.Left, PicProcess.Top);
            this.AcceptButton = ActDoProcess;
        }

        private void TaskReports()
        {
            // ----- Print Reports mode.

            // ----- Update the display.
            AllPanelsInvisible();
            PanelReports.Visible = true;
            ActReports.BackColor = SystemColors.Control;
            LabelSelected.Location = new Point(LabelSelected.Left, PicReports.Top);
            this.AcceptButton = ActDoReports;
        }

        private void MainForm_Load(object sender, EventArgs e)
        {
            // ----- Adjust the various panel positions.
            PanelLibraryItem.Location = new Point(TaskListBackground.Width, WelcomeBackground.Height);
            PanelPatronRecord.Location = PanelLibraryItem.Location;
            PanelHelp.Location = PanelLibraryItem.Location;
            PanelCheckOut.Location = PanelLibraryItem.Location;
            PanelCheckIn.Location = PanelLibraryItem.Location;
            PanelAdmin.Location = PanelLibraryItem.Location;
            PanelProcess.Location = PanelLibraryItem.Location;
            PanelReports.Location = PanelLibraryItem.Location;
            PanelLibraryItem.Visible = true;
            ActSearchLimits.PerformClick();
        }

        private void AllPanelsInvisible()
        {
            // ----- Hide all work panels.
            PanelLibraryItem.Visible = false;
            PanelPatronRecord.Visible = false;
            PanelHelp.Visible = false;
            PanelCheckOut.Visible = false;
            PanelCheckIn.Visible = false;
            PanelAdmin.Visible = false;
            PanelProcess.Visible = false;
            PanelReports.Visible = false;

            // ----- Correctly color all action labels.
            ActLibraryItem.BackColor = Color.White;
            ActPatronRecord.BackColor = Color.White;
            ActHelp.BackColor = Color.White;
            ActCheckOut.BackColor = Color.White;
            ActCheckIn.BackColor = Color.White;
            ActAdmin.BackColor = Color.White;
            ActProcess.BackColor = Color.White;
            ActReports.BackColor = Color.White;
        }

        private void SearchText_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            SearchText.SelectAll();
        }

        private void ActSearchLimits_Click(object sender, EventArgs e)
        {
            // ----- Toggle the display of the media type and location fields.
            if (ActSearchLimits.Top == LabelMoreLimitsTop.Top)
            {
                // ----- Hide the two fields.
                LabelSearchMediaType.Visible = false;
                SearchMediaType.Visible = false;
                LabelSearchLocation.Visible = false;
                SearchLocation.Visible = false;
                ActSearchClear.Location = new Point(ActSearchClear.Left, SearchMediaType.Top);
                ActSearchLimits.Location = new Point(ActSearchLimits.Left, SearchMediaType.Top);
                ActSearch.Location = new Point(ActSearch.Left, SearchMediaType.Top);
                ActSearchLimits.Text = "Mor&e >>";
            }
            else
            {
                // ----- Show the two fields.
                ActSearchClear.Location = new Point(ActSearchClear.Left, LabelMoreLimitsTop.Top);
                ActSearchLimits.Location = new Point(ActSearchLimits.Left, LabelMoreLimitsTop.Top);
                ActSearch.Location = new Point(ActSearch.Left, LabelMoreLimitsTop.Top);
                LabelSearchMediaType.Visible = true;
                SearchMediaType.Visible = true;
                LabelSearchLocation.Visible = true;
                SearchLocation.Visible = true;
                ActSearchLimits.Text = "<< L&ess";
            }
        }

        private void CheckInBarcode_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            CheckInBarcode.SelectAll();
        }

        private void CheckOutBarcode_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            CheckOutBarcode.SelectAll();
        }

        private void MainForm_KeyDown(object sender, KeyEventArgs e)
        {
            // ----- The keys F2 through F9 access the different sections of the form.
            switch (e.KeyCode)
            {
                case Keys.F2:
                    TaskLibraryItem();
                    e.Handled = true;
                    break;
                case Keys.F3:
                    TaskPatronRecord();
                    e.Handled = true;
                    break;
                case Keys.F4:
                    // ----- Allow form to handle Alt+F4.
                    if (e.Alt == true)
                        this.Close();
                    else
                        TaskHelp();
                    e.Handled = true;
                    break;
                case Keys.F5:
                    TaskCheckOut();
                    e.Handled = true;
                    break;
                case Keys.F6:
                    TaskCheckIn();
                    e.Handled = true;
                    break;
                case Keys.F7:
                    TaskAdmin();
                    e.Handled = true;
                    break;
                case Keys.F8:
                    TaskProcess();
                    e.Handled = true;
                    break;
                case Keys.F9:
                    TaskReports();
                    e.Handled = true;
                    break;
            }
        }
    }
}
