﻿// ----- The Library Project
//       Start-to-Finish Visual C# 2015
//       Copyright (c) 2016 by Tim Patrick

using System;
using System.Data;
using System.Drawing.Printing;
using System.Drawing.Text;
using System.Windows.Forms;
using static Library.General;

namespace Library
{
    public partial class Maintenance : Form
    {
        // ----- Store the connection string off-screen.
        private string LibraryConnection;

        private const long ClassifyDewey = 1;
        private const long ClassifyLC = 2;

        private const int DefaultReceiptPrinterWidth = 40;

        public Maintenance()
        {
            InitializeComponent();
        }

        private void Maintenance_KeyDown(object sender, KeyEventArgs e)
        {
            // ----- F1 shows online help.
            if (e.KeyCode == Keys.F1)
                OnlineHelp(this, "Maintenance.htm");
        }

        private void Maintenance_Load(object sender, EventArgs e)
        {
            // ----- Prepare the form.
            InstalledFontCollection allFonts = new InstalledFontCollection();
            string sqlText;
            DataTable dbInfo;

            // ----- Fill in the list of available system fonts.
            RecordFontName.Items.Add(new ListItemData("<Not Selected>", -1L));
            for (int counter = 0; counter < allFonts.Families.Length; counter++)
                RecordFontName.Items.Add(new ListItemData(
                    allFonts.Families[counter].Name, (long)counter));

            // ----- Fill in the list of locations.
            RecordDefaultLocation.Items.Add(new ListItemData("<Not Selected>", -1L));
            sqlText = "SELECT * FROM CodeLocation ORDER BY FullName";
            try
            {
                dbInfo = CreateDataTable(sqlText);
            }
            catch (Exception ex)
            {
                GeneralError("Maintenance.Maintenance_Load", ex);
                return;
            }
            foreach (DataRow dbRow in dbInfo.Rows)
                RecordDefaultLocation.Items.Add(new ListItemData(
                    DBGetText(dbRow["FullName"]), DBGetLong(dbRow["ID"])));
            dbInfo.Dispose();
            dbInfo = null;

            // ----- Fill in the list of classifications.
            RecordClassification.Items.Add(new ListItemData("Dewey Decimal", ClassifyDewey));
            RecordClassification.Items.Add(new ListItemData("Library of Congress", ClassifyLC));

            // ----- Fill in the list of local printers.
            foreach (string installedPrinter in PrinterSettings.InstalledPrinters)
                RecordPrinterLocation.Items.Add(installedPrinter);

            // ----- Fill in the current values.
            PopulateCurrentValues();
        }

        private string GetDBDisplayText(string connectionString)
        {
            // ----- Given a connection string, return just an informational portion
            //       for display purposes.
            string displayText = "";
            string oneKey;
            string oneValue;

            for (int counter = 1; counter <= CountSubStr(connectionString, ";") + 1; counter++)
            {
                // ----- Each comma-delimited part has the format "key=value".
                oneKey = connectionString.GetSubStr(";", counter);
                oneValue = oneKey.GetSubStr("=", 2).Trim();
                oneKey = oneKey.GetSubStr("=", 1).Trim().ToUpper().Replace(" ", "");

                if (oneKey == "DATASOURCE")
                {
                    if (displayText.Length != 0)
                        displayText = "\\" + displayText;
                    displayText = oneValue + displayText;
                }
                else if (oneKey == "INITIALCATALOG")
                {
                    if (displayText.Length != 0)
                        displayText += "\\";
                    displayText += oneValue;
                }
            }

            // ----- Check for no configuration string.
            if ((displayText.Length == 0) | (displayText == "\\"))
                displayText = "Not Specified";
            return displayText;
        }

        private void PopulateCurrentValues()
        {
            // ----- Prepare the form.
            string holdText;
            long holdValue;

            // ===== System-wide fields.

            // ----- Fill in the current default location.
            if (long.TryParse(GetSystemValue("DefaultLocation"), out holdValue) == false)
                holdValue = -1L;
            RecordDefaultLocation.SelectedIndex = RecordDefaultLocation.Items.IndexOf(holdValue);
            if (RecordDefaultLocation.SelectedIndex == -1)
                RecordDefaultLocation.SelectedIndex = RecordDefaultLocation.Items.IndexOf(-1L);

            RecordGraceDays.Text = GetSystemValue("FineGrace");
            RecordSearchLimit.Text = GetSystemValue("SearchLimit");

            // ----- Fill in the classification type.
            if (GetBoolean(GetSystemValue("UseLC")) == true)
                RecordClassification.SelectedIndex = RecordClassification.Items.IndexOf(ClassifyLC);
            else
                RecordClassification.SelectedIndex = RecordClassification.Items.IndexOf(ClassifyDewey);

            RecordSelfCheckout.Checked = GetBoolean(GetSystemValue("PatronCheckOut"));
            RecordBarcodeItem.Text = GetSystemValue("NextBarcodeItem");
            RecordBarcodePatron.Text = GetSystemValue("NextBarcodePatron");
            RecordBarcodeMisc.Text = GetSystemValue("NextBarcodeMisc");

            // ----- Fill in the current font name.
            holdText = GetSystemValue("BarcodeFont");
            RecordFontName.SelectedIndex = RecordFontName.FindStringExact(holdText);
            if (RecordFontName.SelectedIndex == -1)
                RecordFontName.SelectedIndex = RecordFontName.Items.IndexOf(-1L);

            RecordBarcode39.Checked = GetBoolean(GetSystemValue("BarcodeCode39"));
            RecordTicketTop.Text = GetSystemValue("TicketHeading");
            RecordTicketBottom.Text = GetSystemValue("TicketFooting");

            // ===== Workstation-specific fields.
            LibraryConnection = Properties.Settings.Default.DBConnection + "";
            RecordDBLocation.Text = GetDBDisplayText(LibraryConnection);
            RecordConfigLocation.Text = Properties.Settings.Default.ReportConfig + "";
            RecordBasicHelp.Text = Properties.Settings.Default.HelpFile + "";
            RecordAdminHelp.Text = Properties.Settings.Default.HelpFileAdmin + "";
            EnableReceipts.Checked = Properties.Settings.Default.UseReceipts;
            RecordPrinterLocation.Text = Properties.Settings.Default.ReceiptPrinter + "";
            RecordPrinterWidth.Text = Properties.Settings.Default.ReceiptWidth.ToString();
            RecordPostlude.Text = Properties.Settings.Default.ReceiptPostlude + "";
            HideLogin.Checked = Properties.Settings.Default.HideLogin;

        }

        private void ActAllValues_Click(object sender, EventArgs e)
        {
            // ----- Let the user edit the list of system values.
            (new ListEditRecords()).ManageRecords(new SystemValue());

            // ----- Refresh the display elements.
            PopulateCurrentValues();
        }

        private void RecordGraceDays_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordGraceDays.SelectAll();
        }

        private void RecordSearchLimit_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordSearchLimit.SelectAll();
        }

        private void RecordBarcodeItem_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordBarcodeItem.SelectAll();
        }

        private void RecordBarcodePatron_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordBarcodePatron.SelectAll();
        }

        private void RecordBarcodeMisc_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordBarcodeMisc.SelectAll();
        }

        private void RecordTicketTop_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordTicketTop.SelectAll();
        }

        private void RecordTicketBottom_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordTicketBottom.SelectAll();
        }

        private void RecordConfigLocation_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordConfigLocation.SelectAll();
        }

        private void RecordBasicHelp_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordBasicHelp.SelectAll();
        }

        private void RecordAdminHelp_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordAdminHelp.SelectAll();
        }

        private void RecordPrinterLocation_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordPrinterLocation.SelectAll();
        }

        private void RecordPrinterWidth_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordPrinterWidth.SelectAll();
        }

        private void RecordPostlude_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordPostlude.SelectAll();
        }

        private void ActOK_Click(object sender, EventArgs e)
        {
            // ----- Check the user-supplied data.
            if (ValidateFormData() == false)
                return;

            // ----- Save the updated values.
            if (SaveFormData() == false)
                return;

            // ----- Update some internal values immediately.
            MainHelpFile = RecordBasicHelp.Text;
            MainAdminHelpFile = RecordAdminHelp.Text;
            DefaultItemLocation = (long)(ListItemData)RecordDefaultLocation.SelectedItem;
            if (int.TryParse(RecordGraceDays.Text, out FineGraceDays) == false)
                FineGraceDays = 0;

            // ----- Close the form.
            this.DialogResult = DialogResult.OK;
        }

        private bool ValidateFormData()
        {
            // ----- Check the selections. Although the program will not run very
            //       well if any of these values are missing, there are other
            //       places in the program that impose the limits.

            // ----- If grace days is supplied, it must be numeric.
            if (RecordGraceDays.Text.Trim().Length != 0)
            {
                if (CheckInt(RecordGraceDays.Text) == false)
                {
                    MessageBox.Show("Fine Grace Days, if supplied, must be numeric.",
                        ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Information);
                    TabMaintenance.SelectedTab = TabSystem;
                    RecordGraceDays.Focus();
                    return false;
                }
            }

            // ----- If search limit is supplied, it must be numeric.
            if (RecordSearchLimit.Text.Trim().Length != 0)
            {
                if (CheckInt(RecordSearchLimit.Text) == false)
                {
                    MessageBox.Show("Search Result Limit, if supplied, must be numeric.",
                        ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Information);
                    TabMaintenance.SelectedTab = TabSystem;
                    RecordSearchLimit.Focus();
                    return false;
                }
            }

            // ----- If barcode item number is supplied, it must be numeric.
            if (RecordBarcodeItem.Text.Trim().Length != 0)
            {
                if (CheckDecimal(RecordBarcodeItem.Text) == false)
                {
                    MessageBox.Show("Barcode Item Number, if supplied, must be numeric.",
                        ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Information);
                    TabMaintenance.SelectedTab = TabSystem;
                    RecordBarcodeItem.Focus();
                    return false;
                }
            }

            // ----- If barcode patron number is supplied, it must be numeric.
            if (RecordBarcodePatron.Text.Trim().Length != 0)
            {
                if (CheckDecimal(RecordBarcodePatron.Text) == false)
                {
                    MessageBox.Show("Barcode Patron Number, if supplied, must be numeric.",
                        ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Information);
                    TabMaintenance.SelectedTab = TabSystem;
                    RecordBarcodePatron.Focus();
                    return false;
                }
            }

            // ----- If barcode misc number is supplied, it must be numeric.
            if (RecordBarcodeMisc.Text.Trim().Length != 0)
            {
                if (CheckDecimal(RecordBarcodeMisc.Text) == false)
                {
                    MessageBox.Show("Barcode Miscellaneous Number, if supplied, must be numeric.",
                        ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Information);
                    TabMaintenance.SelectedTab = TabSystem;
                    RecordBarcodeMisc.Focus();
                    return false;
                }
            }

            // ----- If printer width is supplied, it must be numeric.
            if ((EnableReceipts.Checked == true) &
                (RecordPrinterWidth.Text.Trim().Length != 0))
            {
                if (CheckInt(RecordPrinterWidth.Text) == false)
                {
                    MessageBox.Show("Receipt Width, if supplied, must be numeric.",
                        ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Information);
                    TabMaintenance.SelectedTab = TabWorkstation;
                    RecordPrinterWidth.Focus();
                    return false;
                }
            }

            // ----- Success.
            return true;
        }

        private bool SaveFormData()
        {
            // ----- Save all of the field values.
            string holdText;

            // ===== System-wide fields.
            this.Cursor = Cursors.WaitCursor;

            // ----- Save the default location.
            if ((long)(ListItemData)RecordDefaultLocation.SelectedItem == -1L)
                holdText = "";
            else
                holdText = ((long)(ListItemData)RecordDefaultLocation.SelectedItem).ToString();
            SetSystemValue("DefaultLocation", holdText);

            SetSystemValue("FineGrace", RecordGraceDays.Text.Trim());
            SetSystemValue("SearchLimit", RecordSearchLimit.Text.Trim());

            // ----- Save the classification type.
            if ((long)(ListItemData)RecordClassification.SelectedItem == ClassifyLC)
                holdText = "-1";
            else
                holdText = "0";
            SetSystemValue("UseLC", holdText);

            // ----- Save the self-checkout flag.
            if (RecordSelfCheckout.Checked == true)
                holdText = "-1";
            else
                holdText = "0";
            SetSystemValue("PatronCheckOut", holdText);

            SetSystemValue("NextBarcodeItem", RecordBarcodeItem.Text.Trim());
            SetSystemValue("NextBarcodePatron", RecordBarcodePatron.Text.Trim());
            SetSystemValue("NextBarcodeMisc", RecordBarcodeMisc.Text.Trim());

            // ----- Save the barcode font.
            if ((long)(ListItemData)RecordFontName.SelectedItem == -1L)
                holdText = "";
            else
                holdText = RecordFontName.Text;
            SetSystemValue("BarcodeFont", holdText);

            // ----- Save the code-39 barcode flag.
            if (RecordBarcode39.Checked == true)
                holdText = "-1";
            else
                holdText = "0";
            SetSystemValue("BarcodeCode39", holdText);

            SetSystemValue("TicketHeading", RecordTicketTop.Text.Trim());
            SetSystemValue("TicketFooting", RecordTicketBottom.Text.Trim());

            // ===== Workstation-specific fields.
            Properties.Settings.Default.DBConnection = LibraryConnection;
            Properties.Settings.Default.ReportConfig = RecordConfigLocation.Text.Trim();
            Properties.Settings.Default.HelpFile = RecordBasicHelp.Text.Trim();
            Properties.Settings.Default.HelpFileAdmin = RecordAdminHelp.Text.Trim();
            Properties.Settings.Default.HideLogin = HideLogin.Checked;
            Properties.Settings.Default.UseReceipts = EnableReceipts.Checked;
            Properties.Settings.Default.ReceiptPrinter = RecordPrinterLocation.Text.Trim();
            Properties.Settings.Default.ReceiptPostlude = RecordPostlude.Text;

            // ----- Save the receipt printer width.
            if (RecordPrinterWidth.Text.Trim().Length == 0)
                Properties.Settings.Default.ReceiptWidth = DefaultReceiptPrinterWidth;
            else
                Properties.Settings.Default.ReceiptWidth = Convert.ToInt32(RecordPrinterWidth.Text);

            // ----- Success.
            this.Cursor = Cursors.Default;
            return true;
        }

        private void ActDBLocation_Click(object sender, EventArgs e)
        {
            // ----- Prompt for the database connection details.
            string newConnection;

            // ----- Prompt the user for the new setting.
            newConnection = (new LocateDatabase()).PromptUser();
            if (newConnection.Length == 0)
                return;

            // ----- Store the new value.
            LibraryConnection = newConnection;
            RecordDBLocation.Text = GetDBDisplayText(LibraryConnection);
        }

        private void ActConfigLocation_Click(object sender, EventArgs e)
        {
            // ----- Let the user browse for the report configuration file.
            //       Set up the file structure.
            OpenFileDialog locateFileDialog;

            // ----- Prepare the dialog.
            locateFileDialog = new OpenFileDialog();
            locateFileDialog.DefaultExt = "xml";
            locateFileDialog.FileName = RecordConfigLocation.Text;
            locateFileDialog.Filter =
                "XML Files (*.xml)|*.xml|" +
                "All Files (*.*)|*.*";
            locateFileDialog.FilterIndex = 1;
            locateFileDialog.Title = "Locate Configuration File";

            // ----- Prompt the user.
            if (locateFileDialog.ShowDialog() != DialogResult.OK)
                return;

            // ----- Save the file path.
            RecordConfigLocation.Text = locateFileDialog.FileName;
        }

        private void ActBasicHelp_Click(object sender, EventArgs e)
        {
            // ----- Let the user browse for the basic help file.
            //       Set up the file structure.
            OpenFileDialog locateFileDialog;

            // ----- Prepare the dialog.
            locateFileDialog = new OpenFileDialog();
            locateFileDialog.DefaultExt = "chm";
            locateFileDialog.FileName = RecordBasicHelp.Text;
            locateFileDialog.Filter =
                "Help Files (*.chm)|*.chm|" +
                "All Files (*.*)|*.*";
            locateFileDialog.FilterIndex = 1;
            locateFileDialog.Title = "Locate Help";

            // ----- Prompt the user.
            if (locateFileDialog.ShowDialog() != DialogResult.OK)
                return;

            // ----- Save the file path.
            RecordBasicHelp.Text = locateFileDialog.FileName;
        }

        private void ActAdminHelp_Click(object sender, EventArgs e)
        {
            // ----- Let the user browse for the administrative help file.
            //       Set up the file structure.
            OpenFileDialog locateFileDialog;

            // ----- Prepare the dialog.
            locateFileDialog = new OpenFileDialog();
            locateFileDialog.DefaultExt = "chm";
            locateFileDialog.FileName = RecordAdminHelp.Text;
            locateFileDialog.Filter =
                "Help Files (*.chm)|*.chm|" +
                "All Files (*.*)|*.*";
            locateFileDialog.FilterIndex = 1;
            locateFileDialog.Title = "Locate Help";

            // ----- Prompt the user.
            if (locateFileDialog.ShowDialog() != DialogResult.OK)
                return;

            // ----- Save the file path.
            RecordAdminHelp.Text = locateFileDialog.FileName;
        }

        private void EnableReceipts_CheckedChanged(object sender, EventArgs e)
        {
            // ----- Toggle the availability of receipt-specific fields.
            LabelPrinterLocation.Enabled = EnableReceipts.Checked;
            RecordPrinterLocation.Enabled = EnableReceipts.Checked;
            LabelPrinterWidth.Enabled = EnableReceipts.Checked;
            RecordPrinterWidth.Enabled = EnableReceipts.Checked;
            LabelPostlude.Enabled = EnableReceipts.Checked;
            RecordPostlude.Enabled = EnableReceipts.Checked;
            LabelPostludeInfo.Enabled = EnableReceipts.Checked;
        }
    }
}
