﻿// ----- The Library Project
//       Start-to-Finish Visual C# 2015
//       Copyright (c) 2016 by Tim Patrick

using System;
using System.Windows.Forms;
using static Library.General;

namespace Library
{
    public partial class PatronPassword : Form
    {
        private long ActiveID;

        public PatronPassword()
        {
            InitializeComponent();
        }

        public bool ChangePassword(long patronID)
        {
            // ----- Edit an existing record.
            ActiveID = patronID;
            this.ShowDialog();
            return (this.DialogResult == DialogResult.OK);
        }

        private bool SaveFormData()
        {
            // ----- The user wants to save changes.
            string sqlText;
            DataCommand sqlRun;

            this.Cursor = Cursors.WaitCursor;

            // ----- Save the data.
            sqlText = "UPDATE Patron SET [Password] = @NewPassword " +
                "WHERE ID = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddText("NewPassword", EncryptPassword(
                "patron", RecordPassword.Text.Trim()));
            sqlRun.AddLong("RecordID", ActiveID);
            try
            {
                ExecuteSQL(sqlRun);
            }
            catch (Exception ex)
            {
                GeneralError("PatronPassword.SaveFormData", ex);
                return false;
            }
            finally
            {
                this.Cursor = Cursors.Default;
            }

            // ----- Success.
            return true;
        }

        private bool ValidateFormData()
        {
            // ----- Make sure the user supplied valid data. Return True on success.
            //       Password and confirm must match.
            if (RecordPassword.Text != RecordConfirm.Text)
            {
                MessageBox.Show("The Password and Confirm fields must match.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                RecordPassword.Focus();
                return false;
            }

            // ----- A password is required.
            if (RecordPassword.Text.Trim().Length == 0)
            {
                MessageBox.Show("The Password field is required.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                RecordPassword.Focus();
                return false;
            }

            // ----- Success.
            return true;
        }

        private void ActOK_Click(object sender, EventArgs e)
        {
            // ----- The user wants to save changes.
            if (ValidateFormData() == false)
                return;
            if (SaveFormData() == false)
                return;
            this.DialogResult = DialogResult.OK;
        }

        private void RecordPassword_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordPassword.Focus();
        }

        private void RecordConfirm_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordConfirm.Focus();
        }

        private void PatronPassword_KeyDown(object sender, KeyEventArgs e)
        {
            // ----- F1 shows online help.
            if (e.KeyCode == Keys.F1)
                OnlineHelp(this, "PatronPassword.htm");
        }

        private void PatronPassword_Load(object sender, EventArgs e)
        {
            // ----- Prepare the form.
            string sqlText;
            DataCommand sqlRun;

            // ----- Show the patron name.
            sqlText = "SELECT FirstName + ' ' + LastName FROM Patron " +
                "WHERE ID = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", ActiveID);
            try
            {
                RecordName.Text = DBGetText(ExecuteSQLReturn(sqlRun));
            }
            catch (Exception ex)
            {
                GeneralError("PatronPassword.Form_Load", ex);
                RecordName.Text = "Unknown";
            }

            // ----- Initialize other fields.
            RecordPassword.Clear();
            RecordConfirm.Clear();
        }
    }
}
