﻿// ----- The Library Project
//       Start-to-Finish Visual C# 2015
//       Copyright (c) 2016 by Tim Patrick

using System;
using System.Collections.Generic;
using System.Data;
using System.Drawing;
using System.Windows.Forms;
using static Library.General;

namespace Library
{
    public partial class PatronRecord : Form
    {
        private long ActivePatronID;
        private int SecondItemRow;
        private bool AllowPatronEdit;
        private List<PaymentItem> SessionPayments;

        public PatronRecord()
        {
            InitializeComponent();
        }

        public void ViewPatronRecord(long patronID, bool allowEdit)
        {
            // ----- Provide access to the patron record.
            ActivePatronID = patronID;
            AllowPatronEdit = allowEdit;
            this.ShowDialog();
        }

        private void RefreshPatronFines(long forcePatronCopy)
        {
            // ----- Refresh the list of fines due by and paid by the patron.
            //       Always include the forcePatronCopy item, unless it is -1.
            string sqlText;
            DataCommand sqlRun;
            DataTable dbInfo;
            decimal fineAmount;
            decimal paidAmount;
            decimal balanceAmount;
            decimal totalBalance;
            Font boldFont;
            string titleText;
            string copyText;
            PatronDetailItem newEntry;
            Graphics canvas;

            // ----- Create graphic objects used for measuring.
            boldFont = new Font(Fines.Font, FontStyle.Bold);
            canvas = Fines.CreateGraphics();

            // ----- Clear the current list.
            Fines.Items.Clear();
            totalBalance = 0M;
            BalanceDue.Text = (0M).ToString("C");
            this.Cursor = Cursors.WaitCursor;

            // ----- Retrieve the fine records for the patron.
            sqlText = "SELECT PC.ID, NI.Title, NI.Subtitle, PC.Fine, PC.Paid, " +
                "IC.CopyNumber FROM PatronCopy AS PC " +
                "INNER JOIN ItemCopy AS IC ON PC.ItemCopy = IC.ID " +
                "INNER JOIN NamedItem AS NI ON IC.ItemID = NI.ID " +
                "WHERE (PC.Patron = " + ActivePatronID.ToString() +
                " AND (PC.Fine <> 0 OR PC.Paid <> 0))";
            if (forcePatronCopy != -1L)
                sqlText += " OR (PC.ID = @RecordID)";
            sqlText += " ORDER BY NI.Title";
            sqlRun = new DataCommand(sqlText);
            if (forcePatronCopy != -1L)
                sqlRun.AddLong("RecordID", forcePatronCopy);
            try
            {
                dbInfo = CreateDataTable(sqlRun);
            }
            catch (Exception ex)
            {
                this.Cursor = Cursors.Default;
                GeneralError("PatronRecord.RefreshPatronFines.Retrieve", ex);
                canvas.Dispose();
                boldFont.Dispose();
                return;
            }

            foreach (DataRow dbRow in dbInfo.Rows)
            {
                // ----- Collect the values.
                fineAmount = DBGetDecimal(dbRow["Fine"]);
                paidAmount = DBGetDecimal(dbRow["Paid"]);
                balanceAmount = fineAmount - paidAmount;
                if (DBGetLong(dbRow["ID"]) != forcePatronCopy)
                {
                    if ((balanceAmount == 0M) & (ShowPaidFines.Checked == false))
                        continue;
                }
                totalBalance += balanceAmount;

                // ----- Format the display items.
                copyText = " [Copy #" + DBGetInteger(dbRow["CopyNumber"]).ToString() + "]";
                titleText = DBGetText(dbRow["Title"]);
                if (Convert.IsDBNull(dbRow["Subtitle"]) == false)
                    titleText += ": " + DBGetText(dbRow["Subtitle"]);
                titleText = FitTextToWidth(titleText, (int)(FineColEnd.Left -
                    FineColTitle.Left - canvas.MeasureString(copyText, boldFont).Width),
                    canvas, boldFont);

                // ----- Prepare a new list item.
                newEntry = new PatronDetailItem();
                newEntry.DetailID = DBGetLong(dbRow["ID"]);
                newEntry.TitleText = titleText + copyText;
                newEntry.FineAmount = fineAmount;
                newEntry.PaidAmount = paidAmount;
                newEntry.BalanceDue = balanceAmount;
                Fines.Items.Add(newEntry);
            }
            dbInfo.Dispose();
            dbInfo = null;

            // ----- Show the total balance.
            BalanceDue.Text = totalBalance.ToString("C");
            ActFinePay.Enabled = false;

            // ----- Dispense with the graphics elements.
            canvas.Dispose();
            boldFont.Dispose();
            this.Cursor = Cursors.Default;
        }

        private void ActRenewItemsOut_Click(object sender, EventArgs e)
        {
            // ----- Try to renew a checked-out item.
            string sqlText;
            DataCommand sqlRun;
            DataTable dbInfo;
            DataRow oneRow;
            DateTime checkOutDate;
            DateTime dueDate;
            int renewsSoFar;
            int maxRenew;
            int renewDays;
            PatronDetailItem itemDetail;
            LibraryTransaction transInfo = null;

            // ----- Make sure an item is selected.
            if (ItemsOut.SelectedIndex == -1)
                return;
            itemDetail = (PatronDetailItem)ItemsOut.SelectedItem;

            // ----- Make sure this is not a reference item.
            sqlText = "SELECT IC.Available, IC.Reference FROM PatronCopy AS PC " +
                "INNER JOIN ItemCopy AS IC ON PC.ItemCopy = IC.ID " +
                "WHERE PC.ID = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", itemDetail.DetailID);
            try
            {
                dbInfo = CreateDataTable(sqlRun);
            }
            catch (Exception ex)
            {
                this.Cursor = Cursors.Default;
                GeneralError("PatronRecord.ActRenewItemsOut_Click.Check", ex);
                return;
            }
            oneRow = dbInfo.Rows[0];
            if (DBGetBool(oneRow["Available"]) == false)
            {
                dbInfo.Dispose();
                dbInfo = null;
                MessageBox.Show("You cannot renew this item because it is no " +
                    "longer available for circulation or checkout.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }
            if (DBGetBool(oneRow["Reference"]) == true)
            {
                dbInfo.Dispose();
                dbInfo = null;
                MessageBox.Show("You cannot renew this item because it is a reference item.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }
            dbInfo.Dispose();

            // ----- Get the status of the checked out item.
            sqlText = "SELECT CheckOut, DueDate, Renewal FROM PatronCopy WHERE ID = " +
                itemDetail.DetailID.ToString();
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", itemDetail.DetailID);
            try
            {
                dbInfo = CreateDataTable(sqlRun);
            }
            catch (Exception ex)
            {
                this.Cursor = Cursors.Default;
                GeneralError("PatronRecord.ActRenewItemsOut_Click.Status", ex);
                return;
            }
            oneRow = dbInfo.Rows[0];
            checkOutDate = DBGetDate(oneRow["CheckOut"]);
            dueDate = DBGetDate(oneRow["DueDate"]);
            renewsSoFar = DBGetInteger(oneRow["Renewal"]);
            dbInfo.Dispose();

            // ----- Determine the limits on this type of item.
            sqlText = "SELECT * FROM CodeMediaType WHERE ID = (SELECT NI.MediaType " +
                "FROM PatronCopy AS PC INNER JOIN ItemCopy AS IC ON PC.ItemCopy = IC.ID " +
                "INNER JOIN NamedItem AS NI ON IC.ItemID = NI.ID " +
                "WHERE PC.ID = @RecordID)";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", itemDetail.DetailID);
            try
            {
                dbInfo = CreateDataTable(sqlRun);
            }
            catch (Exception ex)
            {
                this.Cursor = Cursors.Default;
                GeneralError("PatronRecord.ActRenewItemsOut_Click.Limit", ex);
                return;
            }
            oneRow = dbInfo.Rows[0];
            maxRenew = DBGetInteger(oneRow["RenewTimes"]);
            renewDays = DBGetInteger(oneRow["RenewDays"]);
            dbInfo.Dispose();
            dbInfo = null;

            // ----- See if this item has been renewed too often.
            if (renewsSoFar >= maxRenew)
            {
                if (maxRenew == 0)
                    MessageBox.Show("You cannot renew this item because this type of " +
                        "item does not permit renewals.", ProgramTitle,
                        MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                else
                    MessageBox.Show("You cannot renew this item because you have already renewed it " +
                        ((renewsSoFar == 1) ? "1 time" : renewsSoFar + " times") +
                        ", which is the limit for this type of item.", ProgramTitle,
                        MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }
            this.Cursor = Cursors.WaitCursor;

            // ----- Renew this item by adjusting the due date. While there is room,
            //       keep renewing the item until the item is not overdue, if needed.
            while (true)
            {
                dueDate = dueDate.AddDays(renewDays);
                renewsSoFar += 1;
                if (renewsSoFar >= maxRenew)
                    break;
                if (dueDate >= DateTime.Today)
                    break;
            }

            // ----- Don't allow the due date to be on a holiday.
            while (IsHolidayDate(dueDate) == true)
                dueDate = dueDate.AddDays(1);

            try
            {
                transInfo = TransactionBegin();

                // ----- Update the record.
                sqlText = "UPDATE PatronCopy SET DueDate = @TargetDate, " +
                    "Renewal = @RenewalPos WHERE ID = @RecordID";
                sqlRun = new DataCommand(sqlText);
                sqlRun.AddDate("TargetDate", dueDate);
                sqlRun.AddInteger("RenewalPos", renewsSoFar);
                sqlRun.AddLong("RecordID", itemDetail.DetailID);
                ExecuteSQL(sqlRun, transInfo);

                // ----- Update the patron record.
                sqlText = "UPDATE Patron SET LastActivity = GETDATE() " +
                    "WHERE ID = @RecordID";
                sqlRun = new DataCommand(sqlText);
                sqlRun.AddLong("RecordID", ActivePatronID);
                ExecuteSQL(sqlRun, transInfo);

                TransactionCommit(transInfo);
            }
            catch (Exception ex)
            {
                if (transInfo != null)
                    this.Cursor = Cursors.Default;
                TransactionRollback(transInfo);
                GeneralError("PatronRecord.ActRenewItemsOut_Click.Update", ex);
                return;
            }

            // ----- Update the display.
            itemDetail.DueDate = dueDate;
            ItemsOut.Invalidate(ItemsOut.GetItemRectangle(ItemsOut.SelectedIndex));
            this.Cursor = Cursors.Default;
        }

        private void PatronRecord_Load(object sender, EventArgs e)
        {
            // ----- Prepare the form.
            string sqlText;
            DataCommand sqlRun;
            DataTable dbInfo;
            DataRow oneRow;
            string holdText;
            Graphics canvas;
            PatronDetailItem newEntry;

            // ----- Adjust the line item height of the fines list.
            canvas = Fines.CreateGraphics();
            Fines.ItemHeight = Convert.ToInt32(canvas.MeasureString(
                "A" + Environment.NewLine + "g", Fines.Font).Height) + 3;
            SecondItemRow = Convert.ToInt32(canvas.MeasureString(
                "Ag", Fines.Font).Height) + 1;
            canvas.Dispose();

            // ----- Load in the patron name and barcode.
            sqlText = "SELECT LastName, FirstName, Active, Barcode FROM Patron " +
                "WHERE ID = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", ActivePatronID);
            try
            {
                dbInfo = CreateDataTable(sqlRun);
            }
            catch (Exception ex)
            {
                GeneralError("PatronRecord.PatronRecord_Load.Patron", ex);
                dbInfo = null;
            }
            if (dbInfo != null)
            {
                oneRow = dbInfo.Rows[0];
                PatronName.Text = DBGetText(oneRow["FirstName"]) + " " +
                    DBGetText(oneRow["LastName"]);
                if (DBGetBool(oneRow["Active"]) == false)
                    PatronName.Text += " [Inactive]";
                if (Convert.IsDBNull(oneRow["Barcode"]) == true)
                    PatronBarcode.Text = "None";
                else
                    PatronBarcode.Text = DBGetText(oneRow["Barcode"]);
                dbInfo.Dispose();
            }

            // ----- Load in the list of items checked out.
            sqlText = "SELECT PC.ID, NI.Title, NI.Subtitle, PC.DueDate " +
                "FROM PatronCopy AS PC INNER JOIN ItemCopy AS IC ON PC.ItemCopy = IC.ID " +
                "INNER JOIN NamedItem AS NI ON IC.ItemID = NI.ID " +
                "WHERE PC.Patron = @RecordID " +
                "AND PC.Returned = 0 AND PC.Missing = 0 ORDER BY PC.DueDate";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", ActivePatronID);
            try
            {
                dbInfo = CreateDataTable(sqlRun);
            }
            catch (Exception ex)
            {
                GeneralError("PatronRecord.PatronRecord_Load.Items", ex);
                dbInfo = null;
            }
            if (dbInfo != null)
            {
                foreach (DataRow dbRow in dbInfo.Rows)
                {
                    // ----- Add this checked-out or overdue book to the list.
                    holdText = DBGetText(dbRow["Title"]);
                    if (Convert.IsDBNull(dbRow["Subtitle"]) == false)
                        holdText += ": " + DBGetText(dbRow["Subtitle"]);

                    newEntry = new PatronDetailItem();
                    newEntry.DetailID = DBGetLong(dbRow["ID"]);
                    newEntry.TitleText = holdText;
                    newEntry.DueDate = DBGetDate(dbRow["DueDate"]);
                    ItemsOut.Items.Add(newEntry);
                }
                dbInfo.Dispose();
            }

            // ***** Insert Code Snippet #9 here.

            // ----- Load in the list of patron fines.
            RefreshPatronFines(-1L);

            // ----- Prepare a list of payments for printing.
            SessionPayments = new List<PaymentItem>();

            // ----- If the user has authorization, show the Edit Patron button.
            ActPasswordPatron.Location = new Point(
                ActEditPatron.Location.X, ActEditPatron.Location.Y);
            if (SecurityProfile[(int)LibrarySecurity.ManagePatrons] == true)
            {
                // ----- If this form was called ultimately from the edit patron form,
                //       then hide the Edit Patron button.
                ActEditPatron.Visible = AllowPatronEdit;
                ActPasswordPatron.Visible = !AllowPatronEdit;
            }
            else
            {
                // ----- This user is not authorized to edit patron records.
                ActEditPatron.Visible = false;
                ActPasswordPatron.Visible = true;
            }

            // ----- Only allow authorized users to adjust fines.
            ActFinePay.Visible = SecurityProfile[(int)LibrarySecurity.ProcessFees];
        }

        private void ActEditPatron_Click(object sender, EventArgs e)
        {
            // ----- Let the user edit the patron record.
            string sqlText;
            DataCommand sqlRun;
            DataTable dbInfo = null;
            DataRow oneRow;

            // ----- Display the patron form.
            if ((new Patron()).EditRecordLimited(ActivePatronID) != -1L)
            {
                // ----- Update the patron's name and barcode number.
                sqlText = "SELECT LastName, FirstName, Active, Barcode FROM Patron " +
                    "WHERE ID = " + ActivePatronID.ToString();
                sqlRun = new DataCommand(sqlText);
                sqlRun.AddLong("RecordID", ActivePatronID);
                try
                {
                    dbInfo = CreateDataTable(sqlRun);
                }
                catch (Exception ex)
                {
                    GeneralError("PatronRecord.ActEditPatron_Click", ex);
                    return;
                }

                oneRow = dbInfo.Rows[0];
                PatronName.Text = DBGetText(oneRow["FirstName"]) + " " +
                    DBGetText(oneRow["LastName"]);
                if (DBGetBool(oneRow["Active"]) == false)
                    PatronName.Text += " [Inactive]";
                if (Convert.IsDBNull(oneRow["Barcode"]) == true)
                    PatronBarcode.Text = "None";
                else
                    PatronBarcode.Text = DBGetText(oneRow["Barcode"]);
                dbInfo.Dispose();
            }
        }

        private void ItemsOut_SelectedIndexChanged(object sender, EventArgs e)
        {
            // ----- Enable the Renew button if needed.
            ActRenewItemsOut.Enabled = (ItemsOut.SelectedIndex != -1);
        }

        private void ItemsOut_Click(object sender, EventArgs e)
        {
            // ----- Enable the Renew button if needed.
            ActRenewItemsOut.Enabled = (ItemsOut.SelectedIndex != -1);
        }

        private void ItemsOut_DrawItem(object sender, DrawItemEventArgs e)
        {
            // ----- Specially format the checked out item.
            Brush useBrush;
            string dueDate;
            PatronDetailItem itemDetail;
            StringFormat ellipsesText;

            // ----- Draw the background of the item.
            e.DrawBackground();
            if (e.Index == -1)
                return;

            // ----- Extract the detail from the list item.
            itemDetail = (PatronDetailItem)(ItemsOut.Items[e.Index]);
            dueDate = itemDetail.DueDate.ToString("MMM d, yyyy");

            // ----- Print the title in the right color.
            if ((e.State & DrawItemState.Selected) == DrawItemState.Selected)
                useBrush = SystemBrushes.HighlightText;
            else
                useBrush = SystemBrushes.WindowText;
            ellipsesText = new StringFormat();
            ellipsesText.Trimming = StringTrimming.EllipsisCharacter;
            e.Graphics.DrawString(itemDetail.TitleText, e.Font, useBrush,
                new Rectangle(0, e.Bounds.Top, ItemsOutColDue.Left -
                ItemsOutColTitle.Left - 5, e.Font.Height), ellipsesText);

            // ----- Print the due date in the right color.
            if ((e.State & DrawItemState.Selected) == DrawItemState.Selected)
                useBrush = SystemBrushes.HighlightText;
            else if (Convert.ToDateTime(dueDate) < DateTime.Today)
            {
                // ----- Overdue.
                useBrush = Brushes.Red;
            }
            else
            {
                useBrush = SystemBrushes.WindowText;
            }
            e.Graphics.DrawString(dueDate, e.Font, useBrush,
                ItemsOutColDue.Left - ItemsOutColTitle.Left, e.Bounds.Top);

            // ----- If the ListBox has focus, draw a focus rectangle.
            e.DrawFocusRectangle();
        }

        private void ShowPaidFines_CheckedChanged(object sender, EventArgs e)
        {
            // ----- Refresh the list of fines based on the new setting.
            RefreshPatronFines(-1L);
        }

        private void Fines_SelectedIndexChanged(object sender, EventArgs e)
        {
            // ----- Enable the buttons if needed.
            ActFinePay.Enabled = (Fines.SelectedIndex != -1);
        }

        private void Fines_Click(object sender, EventArgs e)
        {
            // ----- Enable the buttons if needed.
            ActFinePay.Enabled = (Fines.SelectedIndex != -1);
        }

        private void Fines_DrawItem(object sender, DrawItemEventArgs e)
        {
            // ----- Draw the matching items on two lines.
            Brush useBrush;
            Brush useNotice;
            string titleText;
            string fineText;
            string paidText;
            string balanceText;
            Font boldFont;
            PatronDetailItem itemDetail;

            // ----- Draw the background of the item.
            if ((e.State & DrawItemState.Selected) == DrawItemState.Selected)
            {
                useBrush = SystemBrushes.HighlightText;
                useNotice = SystemBrushes.HighlightText;
            }
            else
            {
                useBrush = SystemBrushes.WindowText;
                useNotice = Brushes.Red;
            }
            e.DrawBackground();

            // ----- Make a bold version of the font.
            boldFont = new Font(e.Font, FontStyle.Bold);

            // ----- Extract the details from the list item.
            itemDetail = (PatronDetailItem)(Fines.Items[e.Index]);
            titleText = itemDetail.TitleText;
            fineText = itemDetail.FineAmount.ToString("C");
            paidText = itemDetail.PaidAmount.ToString("C");
            balanceText = itemDetail.BalanceDue.ToString("C");
            if (itemDetail.BalanceDue == 0M)
                useNotice = useBrush;

            // ----- Draw the text of the item.
            e.Graphics.DrawString(titleText, boldFont, useBrush, 0, e.Bounds.Top);
            e.Graphics.DrawString(fineText, e.Font, useBrush,
                FineColFine.Left - FineColTitle.Left, e.Bounds.Top + SecondItemRow);
            e.Graphics.DrawString(paidText, e.Font, useBrush,
                FineColPaid.Left - FineColTitle.Left, e.Bounds.Top + SecondItemRow);
            e.Graphics.DrawString(balanceText, e.Font, useNotice,
                FineColBalance.Left - FineColTitle.Left, e.Bounds.Top + SecondItemRow);

            // ----- If the ListBox has focus, draw a focus rectangle.
            e.DrawFocusRectangle();
            boldFont.Dispose();
        }

        private void ActFinePay_Click(object sender, EventArgs e)
        {
            // ----- Prompt the user to adjust the fines on an item.
            long patronCopyID;

            // ----- An item must be selected.
            if (Fines.SelectedIndex == -1)
            {
                MessageBox.Show("Please select a fine from the list.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }
            patronCopyID = ((PatronDetailItem)Fines.SelectedItem).DetailID;

            // ----- Prompt the user to make changes.
            (new PatronPayment()).ManagePayments(patronCopyID, SessionPayments);

            // ----- Refresh the fines list.
            ActFinePay.Enabled = false;
            RefreshPatronFines(patronCopyID);
            Fines.SelectedIndex = Fines.Items.IndexOf(patronCopyID);
        }

        private void ActPasswordPatron_Click(object sender, EventArgs e)
        {
            // ----- Let the user change their password.
            if ((new PatronPassword()).ChangePassword(ActivePatronID) == true)
                MessageBox.Show("Password changed successfully.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        private void ActClose_Click(object sender, EventArgs e)
        {
            // ----- Print out a ticket if needed.
            if (SessionPayments.Count > 0)
                TicketPrinting.PrintPaymentTicket(ActivePatronID, SessionPayments);
            SessionPayments.Clear();
            SessionPayments = null;

            // ----- Close the form.
            this.DialogResult = DialogResult.Cancel;
        }

        private void ActBalanceTicket_Click(object sender, EventArgs e)
        {
            // ----- Print a ticket of all balances.
            TicketPrinting.PrintBalanceTicket(ActivePatronID, Fines);
        }
    }
}
