﻿// ----- The Library Project
//       Start-to-Finish Visual C# 2015
//       Copyright (c) 2016 by Tim Patrick

using System;
using System.Drawing;
using System.Drawing.Printing;
using System.Windows.Forms;

namespace PrintPreviewTest
{
    public partial class Form1 : Form
    {
        private int WhichPage;

        public Form1()
        {
            InitializeComponent();
        }

        private void ActPrint_Click(object sender, EventArgs e)
        {
            // ----- Prompt the user for printer settings, and start printing.
            UserPrinter.Document = CountingDoc;
            UserPrinter.AllowSomePages = true;
            if (UserPrinter.ShowDialog() == DialogResult.OK)
                CountingDoc.Print();
        }

        private void CountingDoc_BeginPrint(object sender, PrintEventArgs e)
        {
            // ----- Start the counting over.
            WhichPage = CountingDoc.PrinterSettings.FromPage;
        }

        private void CountingDoc_PrintPage(object sender, PrintPageEventArgs e)
        {
            // ----- Print a single page.
            Font hugeFont;
            StringFormat centeredText;

            // ----- Let's go overboard on the font: 256 points!
            hugeFont = new Font("Arial", 256);

            // ----- Center the text on the page.
            centeredText = new StringFormat();
            centeredText.Alignment = StringAlignment.Center;
            centeredText.LineAlignment = StringAlignment.Center;

            // ----- Print the number.
            e.Graphics.DrawString(WhichPage.ToString(), hugeFont,
                Brushes.Black, e.MarginBounds, centeredText);

            // ----- Draw the page margins to make it clear where they are.
            e.Graphics.DrawRectangle(Pens.Blue, e.MarginBounds);

            // ----- Limit the output to five pages.
            WhichPage += 1;
            e.HasMorePages = (WhichPage <= CountingDoc.PrinterSettings.ToPage);
        }

        private void ActPreview_Click(object sender, EventArgs e)
        {
            // ----- Display a preview of the document.
            UserPreview.Document = CountingDoc;
            CountingDoc.PrinterSettings.PrintRange = PrintRange.AllPages;
            UserPreview.ShowDialog();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            // ----- Default to five pages.
            CountingDoc.PrinterSettings.MinimumPage = 1;
            CountingDoc.PrinterSettings.MaximumPage = 5;
            CountingDoc.PrinterSettings.FromPage = 1;
            CountingDoc.PrinterSettings.ToPage = 5;
        }
    }
}
