﻿// ----- The Library Project
//       Start-to-Finish Visual C# 2015
//       Copyright (c) 2016 by Tim Patrick

using System;
using System.Runtime.InteropServices;

namespace Library
{
    public class RawPrinterHelper
    {
        // ----- The code in this class is based on Microsoft knowledge
        //       base article number 322090.
        //       Web: http://support.microsoft.com/?id=322090

        public static bool SendStringToPrinter(string targetPrinter, string stringContent, string documentTitle)
        {
            // ----- Send an array of bytes to a printer queue. Return True on success.
            IntPtr printerHandle = default(IntPtr);
            Int32 errorCode = 0;
            DOCINFOW docDetail = new DOCINFOW();
            Int32 bytesWritten = 0;
            bool printSuccess = false;
            IntPtr contentBytes = default(IntPtr);
            Int32 contentSize = 0;

            // ----- Set up the indentity of this document.
            docDetail.pDocName = documentTitle;
            docDetail.pDataType = "RAW";

            // ----- Convert the string to ANSI text.
            contentSize = stringContent.Length;
            contentBytes = Marshal.StringToCoTaskMemAnsi(stringContent);

            // ----- Open the printer and print the document.
            printSuccess = false;
            if (NativeMethods.OpenPrinter(targetPrinter, ref printerHandle, 0))
            {
                if (NativeMethods.StartDocPrinter(printerHandle, 1, ref docDetail))
                {
                    if (NativeMethods.StartPagePrinter(printerHandle))
                    {
                        // ----- Send the content to the printer.
                        printSuccess = NativeMethods.WritePrinter(printerHandle,
                            contentBytes, contentSize, ref bytesWritten);
                        NativeMethods.EndPagePrinter(printerHandle);
                    }
                    NativeMethods.EndDocPrinter(printerHandle);
                }
                NativeMethods.ClosePrinter(printerHandle);
            }

            // ----- GetLastError may provide information on the last error.
            //       For now, just ignore it.
            if (printSuccess == false)
            {
                errorCode = Marshal.GetLastWin32Error();
            }

            // ----- Free up unused memory.
            Marshal.FreeCoTaskMem(contentBytes);

            // ----- Complete.
            return printSuccess;
        }
    }
}
