﻿// ----- The Library Project
//       Start-to-Finish Visual C# 2015
//       Copyright (c) 2016 by Tim Patrick

using System;
using System.Data;
using System.Windows.Forms;
using static Library.General;

namespace Library
{
    public partial class SubjectAdd : Form
    {
        private long ActiveID;

        public SubjectAdd()
        {
            InitializeComponent();
        }

        public long AddRecord()
        {
            // ----- Add a new subject.
            ActiveID = -1L;
            this.ShowDialog();
            if (this.DialogResult == DialogResult.OK)
                return ActiveID;
            else
                return -1L;
        }

        private bool SaveFormData()
        {
            // ----- Save the record to the database.
            string sqlText;
            DataCommand sqlRun;
            long newID;
            DataTable dbInfo;

            this.Cursor = Cursors.WaitCursor;

            // ----- See if this subject already exists.
            sqlText = "SELECT ID FROM Subject WHERE FullName LIKE @TestName";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddText("TestName", RecordSubject.Text.Trim());
            try
            {
                dbInfo = CreateDataTable(sqlRun);
            }
            catch (Exception ex)
            {
                GeneralError("SubjectAdd.SaveFormData.Test", ex);
                this.Cursor = Cursors.Default;
                return false;
            }
            if (dbInfo.Rows.Count > 0)
                ActiveID = DBGetLong(dbInfo.Rows[0]["ID"], -1L);
            dbInfo.Dispose();
            dbInfo = null;

            // ----- Create a new record if needed.
            if (ActiveID == -1L)
            {
                // ----- Create a new entry.
                sqlText = "INSERT INTO Subject (FullName) OUTPUT INSERTED.ID " +
                    "VALUES (@NewName)";
                sqlRun = new DataCommand(sqlText);
                sqlRun.AddText("NewName", RecordSubject.Text.Trim());
                try
                {
                    newID = DBGetLong(ExecuteSQLReturn(sqlRun));
                }
                catch (Exception ex)
                {
                    GeneralError("SubjectAdd.SaveFormData.Add", ex);
                    this.Cursor = Cursors.Default;
                    return false;
                }
                ActiveID = newID;
            }

            // ----- Success.
            this.Cursor = Cursors.Default;
            return true;
        }

        private bool ValidateFormData()
        {
            // ----- Subject is a required field.
            if (RecordSubject.Text.Trim().Length == 0)
            {
                MessageBox.Show("Please supply a subject.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                RecordSubject.Focus();
                return false;
            }

            // ----- Success.
            return true;
        }

        private void RecordSubject_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordSubject.SelectAll();
        }

        private void ActOK_Click(object sender, EventArgs e)
        {
            // ----- The user wants to save changes.
            if (ValidateFormData() == false)
                return;
            if (SaveFormData() == false)
                return;

            // ----- Continue with AddRecord()
            this.DialogResult = DialogResult.OK;
        }

        private void SubjectAdd_KeyDown(object sender, KeyEventArgs e)
        {
            // ----- F1 shows online help.
            if (e.KeyCode == Keys.F1)
                OnlineHelp(this, "SubjectAdd.htm");
        }
    }
}
